@@ -1,24 +1,25 @@
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
- * http://www.gnu.org/software/gnugo/ for more information.      *
- *                                                               *
- * Copyright 1999, 2000, 2001 by the Free Software Foundation.   *
- *                                                               *
- * This program is free software; you can redistribute it and/or *
- * modify it under the terms of the GNU General Public License   *
- * as published by the Free Software Foundation - version 2.     *
- *                                                               *
- * This program is distributed in the hope that it will be       *
- * useful, but WITHOUT ANY WARRANTY; without even the implied    *
- * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
- * PURPOSE.  See the GNU General Public License in file COPYING  *
- * for more details.                                             *
- *                                                               *
- * You should have received a copy of the GNU General Public     *
- * License along with this program; if not, write to the Free    *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
- * Boston, MA 02111, USA.                                        *
-\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
+ * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * http://www.gnu.org/software/gnugo/ for more information.          *
+ *                                                                   *
+ * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ *                                                                   *
+ * This program is free software; you can redistribute it and/or     *
+ * modify it under the terms of the GNU General Public License as    *
+ * published by the Free Software Foundation - version 2             *
+ *                                                                   *
+ * This program is distributed in the hope that it will be useful,   *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
+ * GNU General Public License in file COPYING for more details.      *
+ *                                                                   *
+ * You should have received a copy of the GNU General Public         *
+ * License along with this program; if not, write to the Free        *
+ * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Boston, MA 02111, USA.                                            *
+\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+
+#include "gnugo.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -41,20 +42,18 @@ static Hashnode *hashtable_enter_position(Hashtable *table, Hash_data *hd);
 static Hashnode *hashtable_search(Hashtable *table, Hash_data *hd);
 
 static Read_result *hashnode_search(Hashnode *node, int routine, int komaster,
-				    int kom_i, int kom_j, int i, int j);
+				    int kom_pos, int str1, int str2);
 static Read_result *hashnode_new_result(Hashtable *table, Hashnode *node, 
-					int routine, 
-					int komaster, int kom_i, int kom_j,
-					int ri, int rj);
+					int routine, int komaster,
+					int kom_pos, int str1, int str2);
 
 static void hashtable_unlink_closed_results(Hashnode *node, 
 					    int exclusions, 
 					    unsigned int stackplimit,
 					    int statistics[][20]);
 static void hashtable_partially_clear(Hashtable *table);
-static int do_get_read_result(int routine, int komaster, int kom_i, int kom_j,
-			      int *si, int *sj,
-			      Read_result **read_result);
+static int do_get_read_result(int routine, int komaster, int kom_pos,
+			      int str1, int str2, Read_result **read_result);
 
 /*
  * Dump an ASCII representation of the contents of a Read_result onto
@@ -67,18 +66,18 @@ void
 read_result_dump(Read_result *result, FILE *outfile)
 {
   fprintf(outfile, "Komaster %d (%d, %d) Routine %d, (%d, %d), depth: %d ",
-	  ((result->compressed_data) >> 29) & 0x03,
-	  (((result->compressed_data) >> 24) & 0x1f) - 1,
-	  (((result->compressed_data) >> 19) & 0x1f) - 1,
-	  ((result->compressed_data) >> 15) & 0x0f,
-	  ((result->compressed_data) >> 10) & 0x1f,
-	  ((result->compressed_data) >> 5)  & 0x1f,
-	  ((result->compressed_data) >> 0)  & 0x1f);
+	  rr_get_komaster(*result),
+	  I(rr_get_kom_pos(*result)),
+	  J(rr_get_kom_pos(*result)),
+	  rr_get_routine(*result),
+	  I(rr_get_str(*result)),
+	  J(rr_get_str(*result)),
+	  rr_get_stackp(*result));
   fprintf(outfile, "Result: %d %d, (%d, %d)\n",
-	  ((result->result_ri_rj) >> 24) & 0xff,
-	  ((result->result_ri_rj) >> 16) & 0xff,
-	  ((result->result_ri_rj) >> 8) & 0xff,
-	  ((result->result_ri_rj) >> 0) & 0xff);
+	  rr_get_status(*result),
+	  rr_get_result(*result),
+	  I(rr_get_move(*result)),
+	  J(rr_get_move(*result)));
 }
 
 
@@ -90,11 +89,13 @@ read_result_dump(Read_result *result, FILE *outfile)
 void
 hashnode_dump(Hashnode *node, FILE *outfile)
 {
-  Read_result  * result;
+  Read_result *result;
 
   /* Data about the node itself. */
   fprintf(outfile, "Hash value: %lx\n", (unsigned long) node->key.hashval);
+#if FULL_POSITION_IN_HASH
   hashposition_dump(&(node->key.hashpos), outfile);
+#endif
 
   for (result = node->results; result != NULL; result = result->next) {
     read_result_dump(result, outfile);
@@ -112,7 +113,7 @@ void
 hashtable_dump(Hashtable *table, FILE *outfile)
 {
   int i;
-  Hashnode * hn;
+  Hashnode *hn;
 
   /* Data about the table itself. */
   fprintf(outfile, "Dump of hashtable\n");
@@ -194,7 +195,7 @@ hashtable_dump2(Hashtable *table, FILE *outfile)
  */
 
 static int
-hashtable_init(Hashtable *table, 
+hashtable_init(Hashtable *table,
 	       int tablesize, int num_nodes, int num_results)
 {
   /* Make sure the hash system is initialized. */
@@ -244,7 +245,7 @@ hashtable_init(Hashtable *table,
 static Hashtable *
 hashtable_new(int tablesize, int num_nodes, int num_results)
 {
-  Hashtable  * table;
+  Hashtable *table;
 
   /* Make sure the hash system is initialized. */
   hash_init();
@@ -283,7 +284,7 @@ hashtable_clear(Hashtable *table)
 
   /* Mark all read_results as free. */
   for (i = 0; i < table->num_results; i++)
-    table->all_results[i].result_ri_rj = 0;
+    table->all_results[i].data2 = 0;
   
   /* Mark all nodes as free. */
   for (i = 0; i < table->num_nodes; i++)
@@ -308,20 +309,26 @@ hashtable_unlink_closed_results(Hashnode *node,
   
   while (current_result != NULL) {
     int stackp;
+    int routine;
 
-    stackp = rr_get_stackp(*current_result);
+    stackp = depth - rr_get_stackp(*current_result);
     if (stackp > 19)
       stackp = 19;
-    statistics[rr_get_routine(*current_result)][stackp]++;
+    if (stackp < 0)
+      stackp = 0;
+
+    routine = rr_get_routine(*current_result);
+    gg_assert(routine >= 0 && routine < NUM_ROUTINES);
+    statistics[routine][stackp]++;
 
     if (rr_get_status(*current_result) == 2
 	&& ((1 << rr_get_routine(*current_result)) & exclusions) == 0
-	&& rr_get_stackp(*current_result) >= stackplimit) {
+	&& depth - rr_get_stackp(*current_result) >= stackplimit) {
       if (previous_result == NULL)
 	node->results = current_result->next;
       else
 	previous_result->next = current_result->next;
-      current_result->result_ri_rj = 0;
+      current_result->data2 = 0;
     }
     else
       previous_result = current_result;
@@ -352,7 +359,8 @@ hashtable_partially_clear(Hashtable *table)
   
   int statistics[NUM_ROUTINES][20];
 
-  DEBUG(DEBUG_READING_PERFORMANCE, "Hashtable cleared because it was full.\n");
+  DEBUG(DEBUG_READING_PERFORMANCE,
+	"Hashtable cleared because it was full.\n");
 
   for (k = 0; k < NUM_ROUTINES; ++k)
     for (l = 0; l < 20; ++l)
@@ -363,7 +371,7 @@ hashtable_partially_clear(Hashtable *table)
    */
   for (k = 0; k < table->num_nodes; k++) {
     node = &(table->all_nodes[k]);
-    bucket = node->key.hashval % table->hashtablesize;
+    bucket = node->key.hashval[0] % table->hashtablesize;
     previous = NULL;
     current = table->hashtable[bucket];
 
@@ -373,7 +381,8 @@ hashtable_partially_clear(Hashtable *table)
 
     /* Remove all closed results for this node except OWL_{ATTACK,DEFEND}. */
     hashtable_unlink_closed_results(node, 
-				    (1 << OWL_ATTACK | 1 << OWL_DEFEND), 3,
+				    (1 << OWL_ATTACK | 1 << OWL_DEFEND
+				     | 1 << SEMEAI), 3,
 				    statistics);
     if (node->results != NULL)
       continue;
@@ -424,7 +433,8 @@ hashtable_partially_clear(Hashtable *table)
 
   /* FIXME: This is not entirely safe although it probably works more
    * than 99.999% of all cases.  If result no 0 or node no 0 is not
-   * free after the partial clearing, this will explode into our face. */
+   * free after the partial clearing, this will explode into our face.
+   */
   table->free_result = 0;
   table->free_node = 0;
 }
@@ -433,13 +443,14 @@ hashtable_partially_clear(Hashtable *table)
 /*
  * Enter a position with a given hash value into the table.  Return 
  * a pointer to the hash node where it was stored.  If it is already
- * there, don't enter it again, but return a pointer to the old one. */
+ * there, don't enter it again, but return a pointer to the old one.
+ */
 
 static Hashnode *
 hashtable_enter_position(Hashtable *table, Hash_data *hd)
 {
-  Hashnode  * node;
-  int         bucket;
+  Hashnode *node;
+  int bucket;
 
   /* If the position is already in the table, return a pointer to it. */
   node = hashtable_search(table, hd);
@@ -456,13 +467,13 @@ hashtable_enter_position(Hashtable *table, Hash_data *hd)
   if (table->free_node == table->num_nodes)
     return NULL;
 
-  /* It wasn't there and there is still room.  Allocate a new node for it... */
+  /* It wasn't there and there is still room. Allocate a new node for it... */
   node = &(table->all_nodes[table->free_node++]);
   node->key = *hd;
   node->results = NULL;
 
   /* ...and enter it into the table. */
-  bucket = hd->hashval % table->hashtablesize;
+  bucket = hd->hashval[0] % table->hashtablesize;
   node->next = table->hashtable[bucket];
   table->hashtable[bucket] = node;
 
@@ -484,15 +495,26 @@ hashtable_enter_position(Hashtable *table, Hash_data *hd)
 static Hashnode *
 hashtable_search(Hashtable *table, Hash_data *hd)
 {
-  Hashnode     * node;
-  int            bucket;
+  Hashnode *node;
+  int bucket;
+  int i;
 
-  bucket = hd->hashval % table->hashtablesize;
+  bucket = hd->hashval[0] % table->hashtablesize;
   for (node = table->hashtable[bucket]; node != NULL; node = node->next) {
-    if (node->key.hashval != hd->hashval)
+    if (node->key.hashval[0] != hd->hashval[0])
       continue;
-    if (hashposition_compare(&hd->hashpos, &node->key.hashpos) == 0)
+    for (i = 1; i < NUM_HASHVALUES; i++)
+      if (node->key.hashval[i] != hd->hashval[i]) {
+	stats.hash_collisions++;
+	break;
+      }
+    if (i >= NUM_HASHVALUES)
+#if FULL_POSITION_IN_HASH
+      if (hashposition_compare(&hd->hashpos, &node->key.hashpos) == 0)
+	break;
+#else
       break;
+#endif
   }
   return node;
 }
@@ -507,17 +529,20 @@ hashtable_search(Hashtable *table, Hash_data *hd)
  */
 
 static Read_result *
-hashnode_search(Hashnode *node, int routine, 
-		int komaster, int kom_i, int kom_j, int i, int j)
+hashnode_search(Hashnode *node, int routine, int komaster, int kom_pos,
+		int str1, int str2)
 {
-  Read_result  * result;
-  unsigned int   search_for;
+  Read_result *result;
+  unsigned int search_for1;
+  unsigned int search_for2;
 
-  search_for = rr_compress_data(rr, routine, komaster, kom_i, kom_j,
-				i, j, stackp);
+  search_for1 = rr_input_data1(routine, komaster, kom_pos, str1,
+			       depth - stackp);
+  search_for2 = rr_input_data2(str2);
 
   for (result = node->results; result != NULL; result = result->next) {
-    if (result->compressed_data == search_for)
+    if (result->data1 == search_for1
+	&& (result->data2 & RR_INPUT_DATA2) == search_for2)
       break;
     }
 
@@ -534,9 +559,9 @@ hashnode_search(Hashnode *node, int routine,
 
 static Read_result *
 hashnode_new_result(Hashtable *table, Hashnode *node, int routine, 
-		    int komaster, int kom_i, int kom_j, int i, int j)
+		    int komaster, int kom_pos, int str1, int str2)
 {
-  Read_result  * result;
+  Read_result *result;
 
   /* If the next result is not free, skip until we find one which is free. */
   while (table->free_result < table->num_results
@@ -554,13 +579,10 @@ hashnode_new_result(Hashtable *table, Hashnode *node, int routine,
   result->next = node->results;
   node->results = result;
 
-  /* Now, put the routine number into it. */
-  result->compressed_data = rr_compress_data(rr, routine, komaster,
-					     kom_i, kom_j, i, j, stackp);
+  /* Now, put the input data into it. This also sets status to open. */
+  rr_set_input_data2(*result, routine, komaster, kom_pos, str1, str2,
+		     depth - stackp);
 
-  /* And set the status to open. */
-  result->result_ri_rj = 1 << 24;
-  
   stats.read_result_entered++;
   return result;
 
@@ -584,6 +606,8 @@ reading_cache_init(int bytes)
 		 / (1.5 * sizeof(Hashnode *)
 		    + sizeof(Hashnode)
 		    + 1.4 * sizeof(Read_result)));
+  if (0)
+    gprintf("Allocated memory for %d hash nodes. \n", (int) nodes);
   /* If we get a zero size hash table, disable hashing completely. */
   if (nodes < 1.0)
     hashflags = HASH_NOTHING;
@@ -592,8 +616,9 @@ reading_cache_init(int bytes)
 			   (int) (1.4 * nodes)); /* read results */
   
   if (!movehash) {
-    fprintf(stderr, "Warning: failed to allocate hashtable, caching disabled.\n");
-      hashflags = HASH_NOTHING;
+    fprintf(stderr,
+	    "Warning: failed to allocate hashtable, caching disabled.\n");
+    hashflags = HASH_NOTHING;
   }
 }
 
@@ -610,8 +635,8 @@ reading_cache_clear()
  */
 
 int
-get_read_result(int routine, int komaster, int kom_i, int kom_j,
-		int *si, int *sj, Read_result **read_result)
+get_read_result(int routine, int komaster, int kom_pos, int *str,
+		Read_result **read_result)
 {
   int result;
   /* Only store the result if stackp <= depth. Above that, there
@@ -625,57 +650,98 @@ get_read_result(int routine, int komaster, int kom_i, int kom_j,
   /* Find the origin of the string containing (si, sj),
    * in order to make the caching of read results work better.
    */
-  find_origin(*si, *sj, si, sj);
+  *str = find_origin(*str);
   
-  result = do_get_read_result(routine, komaster, kom_i, kom_j,
-			      si, sj, read_result);
+  result = do_get_read_result(routine, komaster, kom_pos, *str, NO_MOVE,
+			      read_result);
   if (*read_result == NULL) {
     /* Clean up the hashtable and try once more. */
     hashtable_partially_clear(movehash);
-    result = do_get_read_result(routine, komaster, kom_i, kom_j, si, sj,
+    result = do_get_read_result(routine, komaster, kom_pos, *str, NO_MOVE,
 				read_result);
   }
   return result;
 }
 
-static int
-do_get_read_result(int routine, int komaster, int kom_i, int kom_j,
-		   int *si, int *sj, Read_result **read_result)
+/*
+ * Variant with two calling strings.
+ */
+
+int
+get_read_result2(int routine, int komaster, int kom_pos, int *str1, int *str2,
+		 Read_result **read_result)
 {
-  Hashnode      * hashnode;
-  int             retval;
+  int result;
+  /* Only store the result if stackp <= depth. Above that, there
+   * is no branching, so we won't gain anything.
+   */
+  if (stackp > depth) {
+    *read_result = NULL;
+    return 0;
+  }
+  
+  /* Find the origin of the string containing (si, sj),
+   * in order to make the caching of read results work better.
+   */
+  *str1 = find_origin(*str1);
+  *str2 = find_origin(*str2);
+  
+  result = do_get_read_result(routine, komaster, kom_pos, *str1, *str2,
+			      read_result);
+  if (*read_result == NULL) {
+    /* Clean up the hashtable and try once more. */
+    hashtable_partially_clear(movehash);
+    result = do_get_read_result(routine, komaster, kom_pos, *str1, *str2,
+				read_result);
+  }
+  return result;
+}
+
+
+static int
+do_get_read_result(int routine, int komaster, int kom_pos,
+		   int str1, int str2, Read_result **read_result)
+{
+  Hashnode *hashnode;
+  int retval;
 
 #if CHECK_HASHING
   Hash_data    key;
 
   /* Check the hash table to see if we have had this position before. */
-  hashdata_recalc(&key, p, ko_i, ko_j);
+  hashdata_recalc(&key, board, board_ko_pos);
+#if FULL_POSITION_IN_HASH
   gg_assert(hashdata_diff_dump(&key, &hashdata) == 0);
+#else
+  gg_assert(hashdata_compare(&key, &hashdata) == 0);
+#endif
 
   /* Find this position in the table.  If it wasn't found, enter it. */
   hashnode = hashtable_search(movehash, &hashdata);
   if (hashnode != NULL) {
     stats.position_hits++;
-    RTRACE("We found position %H in the hash table...\n",
-	   (unsigned long) hashdata.hashval);
-  } else {
+    DEBUG(DEBUG_READING_CACHE, "We found position %H in the hash table...\n",
+	  (unsigned long) hashdata.hashval);
+  }
+  else {
     hashnode = hashtable_enter_position(movehash, &hashdata);
     if (hashnode)
-      RTRACE("Created position %H in the hash table...\n",
-	     (unsigned long) hashdata.hashval);
+      DEBUG(DEBUG_READING_CACHE, "Created position %H in the hash table...\n",
+	    (unsigned long) hashdata.hashval);
   }
 #else
   /* Find this position in the table.  If it wasn't found, enter it. */
   hashnode = hashtable_search(movehash, &hashdata);
   if (hashnode != NULL) {
     stats.position_hits++;
-    RTRACE("We found position %H in the hash table...\n",
-	   (unsigned long) hashdata.hashval);
-  } else {
+    DEBUG(DEBUG_READING_CACHE, "We found position %H in the hash table...\n",
+	  (unsigned long) hashdata.hashval);
+  }
+  else {
     hashnode = hashtable_enter_position(movehash, &hashdata);
     if (hashnode)
-      RTRACE("Created position %H in the hash table...\n",
-	     (unsigned long) hashdata.hashval);
+      DEBUG(DEBUG_READING_CACHE, "Created position %H in the hash table...\n",
+	    (unsigned long) hashdata.hashval);
   }
 #endif
 
@@ -684,26 +750,30 @@ do_get_read_result(int routine, int komaster, int kom_i, int kom_j,
     /* No hash node, so we can't enter a result into it. */
     *read_result = NULL;
 
-  } else {
+  }
+  else {
 
     /* We found it!  Now see if we can find a previous result. */
-    *read_result = hashnode_search(hashnode, routine, komaster, kom_i, kom_j,
-				   *si, *sj);
+    *read_result = hashnode_search(hashnode, routine, komaster, kom_pos,
+				   str1, str2);
 
     if (*read_result != NULL) {
       stats.read_result_hits++;
       retval = 1;
-    } else {
-      RTRACE("...but no previous result for routine %d and (%m)...",
-	     routine, *si, *sj);
+    }
+    else {
+      DEBUG(DEBUG_READING_CACHE,
+	    "...but no previous result for routine %d and (%1m, %1m)...",
+	    routine, str1, str2);
 
       *read_result = hashnode_new_result(movehash, hashnode, routine,
-					 komaster, kom_i, kom_j, *si, *sj);
+					 komaster, kom_pos, str1, str2);
       
       if (*read_result == NULL)
-	RTRACE("%o...and unfortunately there was no room for one.\n");
+	DEBUG(DEBUG_READING_CACHE,
+	      "%o...and unfortunately there was no room for one.\n");
       else
-	RTRACE("%o...so we allocate a new one.\n");
+	DEBUG(DEBUG_READING_CACHE, "%o...so we allocate a new one.\n");
     }
   }
 
@@ -716,29 +786,61 @@ do_get_read_result(int routine, int komaster, int kom_i, int kom_j,
  */
 
 void
-sgf_trace(const char *func, int si, int sj, int i, int j,
-	  int result, const char *message)
+sgf_trace(const char *func, int str, int move, int result,
+	  const char *message)
 {
   char buf[100];
 
-  sprintf(buf, "%s %c%d: ", func, sj + 'A' + (sj>=8), board_size - si);
+  sprintf(buf, "%s %c%d: ", func, J(str) + 'A' + (J(str) >= 8),
+	  board_size - I(str));
   
   if (result == 0)
     sprintf(buf + strlen(buf), "0");
-  else if (ON_BOARD(i, j))
+  else if (ON_BOARD(move))
     sprintf(buf + strlen(buf), "%s %c%d", result_to_string(result), 
-	    j + 'A' + (j>=8),
-	    board_size - i);
-  else if (i == -1 && j == -1)
+	    J(move) + 'A' + (J(move) >= 8),
+	    board_size - I(move));
+  else if (is_pass(move))
     sprintf(buf + strlen(buf), "%s PASS", result_to_string(result));
   else
-    sprintf(buf + strlen(buf), "%s [%d,%d]", result_to_string(result), i, j);
+    sprintf(buf + strlen(buf), "%s [%d]", result_to_string(result), move);
 
   if (message)
     sprintf(buf + strlen(buf), " (%s)", message);
   
   sgftreeAddComment(sgf_dumptree, NULL, buf);
-}  
+}
+
+/* Write two group reading (connection or semeai) trace data to an SGF
+ * file. Normally called through the macro SGFTRACE2 in cache.h.
+ */
+
+void
+sgf_trace2(const char *func, int str1, int str2, int move, int result,
+	   const char *message)
+{
+  char buf[100];
+
+  sprintf(buf, "%s %c%d %c%d: ", func,
+	  J(str1) + 'A' + (J(str1) >= 8), board_size - I(str1),
+	  J(str2) + 'A' + (J(str2) >= 8), board_size - I(str2));
+  
+  if (result == 0)
+    sprintf(buf + strlen(buf), "0");
+  else if (ON_BOARD(move))
+    sprintf(buf + strlen(buf), "%s %c%d", result_to_string(result), 
+	    J(move) + 'A' + (J(move) >= 8),
+	    board_size - I(move));
+  else if (is_pass(move))
+    sprintf(buf + strlen(buf), "%s PASS", result_to_string(result));
+  else
+    sprintf(buf + strlen(buf), "%s [%d]", result_to_string(result), move);
+
+  if (message)
+    sprintf(buf + strlen(buf), " (%s)", message);
+  
+  sgftreeAddComment(sgf_dumptree, NULL, buf);
+}
 
 /*
  * Local Variables:
@@ -746,5 +848,3 @@ sgf_trace(const char *func, int si, int sj, int i, int j,
  * c-basic-offset: 2
  * End:
  */
-
-
