@@ -1,49 +1,45 @@
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
- * http://www.gnu.org/software/gnugo/ for more information.      *
- *                                                               *
- * Copyright 1999, 2000, 2001 by the Free Software Foundation.   *
- *                                                               *
- * This program is free software; you can redistribute it and/or *
- * modify it under the terms of the GNU General Public License   *
- * as published by the Free Software Foundation - version 2.     *
- *                                                               *
- * This program is distributed in the hope that it will be       *
- * useful, but WITHOUT ANY WARRANTY; without even the implied    *
- * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
- * PURPOSE.  See the GNU General Public License in file COPYING  *
- * for more details.                                             *
- *                                                               *
- * You should have received a copy of the GNU General Public     *
- * License along with this program; if not, write to the Free    *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
- * Boston, MA 02111, USA.                                        *
-\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
+ * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * http://www.gnu.org/software/gnugo/ for more information.          *
+ *                                                                   *
+ * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ *                                                                   *
+ * This program is free software; you can redistribute it and/or     *
+ * modify it under the terms of the GNU General Public License as    *
+ * published by the Free Software Foundation - version 2             *
+ *                                                                   *
+ * This program is distributed in the hope that it will be useful,   *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
+ * GNU General Public License in file COPYING for more details.      *
+ *                                                                   *
+ * You should have received a copy of the GNU General Public         *
+ * License along with this program; if not, write to the Free        *
+ * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Boston, MA 02111, USA.                                            *
+\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 
+#include "gnugo.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include "liberty.h"
 
-static int find_backfilling_move(int m, int n, int color, int *i, int *j);
-static int filllib_confirm_safety(int m, int n, int color, int *di, int *dj);
+static int find_backfilling_move(int move, int color, int *backfill_move);
+static int filllib_confirm_safety(int move, int color, int *defense_point);
 
 /* Determine whether a point is adjacent to at least one own string which
  * isn't dead.
  */
 static int
-living_neighbor(int m, int n, int color)
+living_neighbor(int pos, int color)
 {
   int k;
   for (k = 0; k < 4; k++) {
-    int dm = deltai[k];
-    int dn = deltaj[k];
-    if (ON_BOARD(m+dm, n+dn)
-	&& p[m+dm][n+dn] == color
-	&& dragon[m+dm][n+dn].matcher_status != DEAD)
+    if (board[pos + delta[k]] == color
+	&& dragon[pos + delta[k]].matcher_status != DEAD)
       return 1;
   }
 
@@ -51,30 +47,31 @@ living_neighbor(int m, int n, int color)
 }
 
 
-/* Determine whether (m, n) effectively is a black or white point.
+/* Determine whether (pos) effectively is a black or white point.
  * The test for inessentiality is to avoid filling the liberties
  * around a killing nakade string.
  */
 static void
-analyze_neighbor(int m, int n, int *found_black, int *found_white)
+analyze_neighbor(int pos, int *found_black, int *found_white)
 {
-  switch (p[m][n]) {
+  switch (board[pos]) {
     case EMPTY:
       if (!(*found_black)
-	  && living_neighbor(m, n, BLACK)
-	  && safe_move(m, n, WHITE) == 0)
+	  && living_neighbor(pos, BLACK)
+	  && safe_move(pos, WHITE) != 3)
 	*found_black = 1;
       
       if (!(*found_white)
-	  && living_neighbor(m, n, WHITE)
-	  && safe_move(m, n, BLACK) == 0)
+	  && living_neighbor(pos, WHITE)
+	  && safe_move(pos, BLACK) != 3)
 	*found_white = 1;
       
       break;
 
     case BLACK:
-      if (!worm[m][n].inessential && DRAGON2(m, n).safety != INESSENTIAL) {
-	if (dragon[m][n].matcher_status == ALIVE)
+      if (!worm[pos].inessential && DRAGON2(pos).safety != INESSENTIAL) {
+	if (dragon[pos].matcher_status == ALIVE
+	    || dragon[pos].matcher_status == UNKNOWN)
 	  *found_black = 1;
 	else
 	  *found_white = 1;
@@ -82,8 +79,9 @@ analyze_neighbor(int m, int n, int *found_black, int *found_white)
       break;
 
     case WHITE:
-      if (!worm[m][n].inessential && DRAGON2(m, n).safety != INESSENTIAL) {
-	if (dragon[m][n].matcher_status == ALIVE)
+      if (!worm[pos].inessential && DRAGON2(pos).safety != INESSENTIAL) {
+	if (dragon[pos].matcher_status == ALIVE
+	    || dragon[pos].matcher_status == UNKNOWN)
 	  *found_white = 1;
 	else
 	  *found_black = 1;
@@ -99,17 +97,17 @@ analyze_neighbor(int m, int n, int *found_black, int *found_white)
  * that several backfilling moves are ultimately necessary.
  *
  * If a move for color is found, return 1, otherwise return 0.
- * The move is returned in (*i, *j).
+ * The move is returned in (*move).
  */
 
 int 
-fill_liberty(int *i, int *j, int color)
+fill_liberty(int *move, int color)
 {
   int m, n;
   int k;
   int other = OTHER_COLOR(color);
-  int di, dj;
-  int potential_color[MAX_BOARD][MAX_BOARD];
+  int defense_point;
+  int potential_color[BOARDMAX];
 
   /* We first make a fast scan for intersections which are potential
    * candidates for liberty filling. This is not very accurate, but it
@@ -120,33 +118,32 @@ fill_liberty(int *i, int *j, int color)
   memset(potential_color, 0, sizeof(potential_color));
   for (m = 0; m < board_size; m++)
     for (n = 0; n < board_size; n++) {
-      if (p[m][n] == EMPTY)
+      int pos = POS(m, n);
+      if (board[pos] == EMPTY)
 	continue;
 
-      if (worm[m][n].inessential || DRAGON2(m, n).safety == INESSENTIAL)
+      if (worm[pos].inessential || DRAGON2(pos).safety == INESSENTIAL)
 	continue;
 
-      if (dragon[m][n].matcher_status != ALIVE) {
+      if (dragon[pos].matcher_status != ALIVE) {
 	for (k = 0; k < 4; k++) {
-	  int di = deltai[k];
-	  int dj = deltaj[k];
-	  if (ON_BOARD(m+di, n+dj) && p[m+di][n+dj] == EMPTY)
-	    potential_color[m+di][n+dj] |= OTHER_COLOR(p[m][n]);
+	  int pos2 = pos + delta[k];
+	  if (board[pos2] == EMPTY)
+	    potential_color[pos2] |= OTHER_COLOR(board[pos]);
 	}
       }
       
-      if (dragon[m][n].matcher_status != DEAD) {
+      if (dragon[pos].matcher_status != DEAD) {
 	for (k = 0; k < 12; k++) {
-	  int di = deltai[k%8];
-	  int dj = deltaj[k%8];
+	  int d = delta[k%8];
+
 	  if (k >= 8) {
- 	    if (ON_BOARD(m+di, n+dj) && p[m+di][n+dj] != EMPTY)
+ 	    if (board[pos + d] != EMPTY)
  	      continue;
-	    di *= 2;
-	    dj *= 2;
+	    d *= 2;
 	  }
-	  if (ON_BOARD(m+di, n+dj) && p[m+di][n+dj] == EMPTY)
-	    potential_color[m+di][n+dj] |= p[m][n];
+	  if (board[pos + d] == EMPTY)
+	    potential_color[pos + d] |= board[pos];
 	}
       }
     }
@@ -154,6 +151,7 @@ fill_liberty(int *i, int *j, int color)
   
   for (m = 0; m < board_size; m++)
     for (n = 0; n < board_size; n++) {
+      int pos = POS(m, n);
       /* It seems we can't trust an empty liberty to be gray-colored
        * either as a cave or as a cavity. Instead we look for empty
        * intersections with at least one neighbor of each color, where
@@ -163,19 +161,18 @@ fill_liberty(int *i, int *j, int color)
       int found_white = 0;
       int found_black = 0;
 
-      if (p[m][n] != EMPTY)
+      if (board[pos] != EMPTY)
 	continue;
 
       /* Quick rejection based on preliminary test above. */
-      if (potential_color[m][n] != GRAY)
+      if (potential_color[pos] != GRAY)
 	continue;
 
       /* Loop over the neighbors. */
       for (k = 0; k < 4; k++) {
-	int dm = deltai[k];
-	int dn = deltaj[k];
-	if (ON_BOARD(m+dm, n+dn))
-	  analyze_neighbor(m+dm, n+dn, &found_black, &found_white);
+	int d = delta[k];
+	if (ON_BOARD(pos + d))
+	  analyze_neighbor(pos + d, &found_black, &found_white);
       }
       
       /* Do we have neighbors of both colors? */
@@ -192,32 +189,31 @@ fill_liberty(int *i, int *j, int color)
        * 6. Move would violate confirm_safety.
        */
 
-      DEBUG(DEBUG_FILLLIB, "Filllib: Considering move at %m.\n", m, n);
+      DEBUG(DEBUG_FILLLIB, "Filllib: Considering move at %1m.\n", pos);
       
       /* Legal and tactically safe, play it if it passes
        * confirm_safety test, i.e. that it isn't a blunder which
        * causes problems for other strings.
        */
-      if (safe_move(m, n, color) == WIN) {
+      if (safe_move(pos, color) == WIN) {
 	DEBUG(DEBUG_FILLLIB, "Filllib: Tactically safe.\n");
-	if (filllib_confirm_safety(m, n, color, &di, &dj)) {
+	if (filllib_confirm_safety(pos, color, &defense_point)) {
 	  /* Safety confirmed. */
 	  DEBUG(DEBUG_FILLLIB, "Filllib: Safety confirmed.\n");
-	  *i = m;
-	  *j = n;
+	  *move = pos;
 	  return 1;
 	}
-	else if (di != -1) {
-	  /* Safety not confirmed because the move at (m, n) would set
-           * up a double threat. (di, dj) is assumed to defend against
-           * this threat.
+	else if (defense_point != NO_MOVE) {
+	  /* Safety not confirmed because the move at (pos) would set
+           * up a double threat. (defense_point) is assumed to defend
+           * against this threat.
 	   *
-	   * FIXME: We should verify that (di, dj) really is effective.
+	   * FIXME: We should verify that (defense_point) really is effective.
 	   */
 	  DEBUG(DEBUG_FILLLIB,
-		"Filllib: Safety not confirmed, but %m defends.\n", di, dj);
-	  *i = di;
-	  *j = dj;
+		"Filllib: Safety not confirmed, but %1m defends.\n",
+		defense_point);
+	  *move = defense_point;
 	  return 1;
 	}
 	else {
@@ -232,30 +228,29 @@ fill_liberty(int *i, int *j, int color)
       }
       
       /* Try to play the move. */
-      if (trymove(m, n, color, "fill_liberty", -1, -1, EMPTY, -1, -1)) {
+      if (trymove(pos, color, "fill_liberty", NO_MOVE, EMPTY, NO_MOVE)) {
 	popgo();
 	/* Legal, but not safe. Look for backfilling move. */
 	DEBUG(DEBUG_FILLLIB,
 	      "Filllib: Legal but not safe, looking for backfilling move.\n");
 
-	if (find_backfilling_move(m, n, color, i, j)) {
-	  DEBUG(DEBUG_FILLLIB, "Filllib: Backfilling move at %m.\n", *i, *j);
+	if (find_backfilling_move(pos, color, move)) {
+	  DEBUG(DEBUG_FILLLIB, "Filllib: Backfilling move at %1m.\n", *move);
 	  /* In certain positions it may happen that an illegal move
 	   * is found. This probably only can happen if we try to play
 	   * a move inside a lost semeai. Anyway we should discard the
 	   * move.
 	   */
-	  if (!is_legal(*i, *j, color)) {
+	  if (!is_legal(*move, color)) {
 	    DEBUG(DEBUG_FILLLIB, "Filllib: Was illegal, discarded.\n");
-	    *i = -1;
-	    *j = -1;
+	    *move = NO_MOVE;
 	    continue;
 	  }
 
 	  /* If the move turns out to be strategically unsafe, or
            * setting up a double threat elsewhere, also discard it.
 	   */
-	  if (!filllib_confirm_safety(*i, *j, color, &di, &dj)) {
+	  if (!filllib_confirm_safety(*move, color, &defense_point)) {
 	    DEBUG(DEBUG_FILLLIB,
 		  "Filllib: Safety not confirmed, discarded.\n");
 	    continue;
@@ -266,16 +261,15 @@ fill_liberty(int *i, int *j, int color)
 	}
 	else {
 	  /* If we captured some stones, this move should be ok anyway. */
-	  if (does_capture_something(m, n, color)) {
+	  if (does_capture_something(pos, color)) {
 	    DEBUG(DEBUG_FILLLIB,
 		  "Filllib: Not tactically safe, but captures stones.\n");
-	    if (!filllib_confirm_safety(m, n, color, &di, &dj)) {
+	    if (!filllib_confirm_safety(pos, color, &defense_point)) {
 	      DEBUG(DEBUG_FILLLIB,
 		    "Filllib: Safety not confirmed, discarded.\n");
 	      continue;
 	    }
-	    *i = m;
-	    *j = n;
+	    *move = pos;
 	    return 1;
 	  }
 	}
@@ -286,14 +280,11 @@ fill_liberty(int *i, int *j, int color)
 	 */
 	DEBUG(DEBUG_FILLLIB, "Filllib: Illegal, looking for back-capture.\n");
 	for (k = 0; k < 4; k++) {
-	  int dm = deltai[k];
-	  int dn = deltaj[k];
-	  if (ON_BOARD(m+dm, n+dn)
-	      && p[m+dm][n+dn] == other
-	      && worm[m+dm][n+dn].attack_code == WIN) {
-	    *i = worm[m+dm][n+dn].attacki;
-	    *j = worm[m+dm][n+dn].attackj;
-	    DEBUG(DEBUG_FILLLIB, "Filllib: Found at %m.\n", *i, *j);
+	  int d = delta[k];
+	  if (board[pos + d] == other
+	      && worm[pos + d].attack_codes[0] == WIN) {
+	    *move = worm[pos + d].attack_points[0];
+	    DEBUG(DEBUG_FILLLIB, "Filllib: Found at %1m.\n", *move);
 	    return 1;
 	  }
 	}
@@ -301,16 +292,12 @@ fill_liberty(int *i, int *j, int color)
 	DEBUG(DEBUG_FILLLIB,
 	      "Filllib: Nothing found, looking for ko back-capture.\n");
 	for (k = 0; k < 4; k++) {
-	  int dm = deltai[k];
-	  int dn = deltaj[k];
-	  if (ON_BOARD(m+dm, n+dn)
-	      && p[m+dm][n+dn] == other
-	      && worm[m+dm][n+dn].attack_code != 0
-	      && is_legal(worm[m+dm][n+dn].attacki, worm[m+dm][n+dn].attackj,
-			  color)) {
-	    *i = worm[m+dm][n+dn].attacki;
-	    *j = worm[m+dm][n+dn].attackj;
-	    DEBUG(DEBUG_FILLLIB, "Filllib: Found at %m.\n", *i, *j);
+	  int d = delta[k];
+	  if (board[pos + d] == other
+	      && worm[pos + d].attack_codes[0] != 0
+	      && is_legal(worm[pos + d].attack_points[0], color)) {
+	    *move = worm[pos + d].attack_points[0];
+	    DEBUG(DEBUG_FILLLIB, "Filllib: Found at %1m.\n", *move);
 	    return 1;
 	  }
 	}
@@ -318,23 +305,17 @@ fill_liberty(int *i, int *j, int color)
 	DEBUG(DEBUG_FILLLIB,
 	      "Filllib: Nothing found, looking for threat to back-capture.\n");
 	for (k = 0; k < 4; k++) {
-	  int dm = deltai[k];
-	  int dn = deltaj[k];
-	  if (ON_BOARD(m+dm, n+dn)
-	      && p[m+dm][n+dn] == other
-	      && worm[m+dm][n+dn].attack_code != 0) {
+	  int d = delta[k];
+	  if (board[pos + d] == other
+	      && worm[pos + d].attack_codes[0] != 0) {
 	    /* Just pick some other liberty. */
-	    int libi[2], libj[2];
-	    if (findlib(m+dm, n+dn, 2, libi, libj) > 1) {
-	      if (is_legal(libi[0], libj[0], color)) {
-		*i = libi[0];
-		*j = libj[0];
-	      }
-	      else if (is_legal(libi[1], libj[1], color)) {
-		*i = libi[1];
-		*j = libj[1];
-	      }
-	      DEBUG(DEBUG_FILLLIB, "Filllib: Found at %m.\n", *i, *j);
+	    int libs[2];
+	    if (findlib(pos + d, 2, libs) > 1) {
+	      if (is_legal(libs[0], color))
+		*move = libs[0];
+	      else if (is_legal(libs[1], color))
+		*move = libs[1];
+	      DEBUG(DEBUG_FILLLIB, "Filllib: Found at %1m.\n", *move);
 	      return 1;
 	    }
 	  }
@@ -351,15 +332,15 @@ fill_liberty(int *i, int *j, int color)
 /* The strategy for finding a backfilling move is to first identify
  * moves that
  *
- * 1. defends the position obtained after playing (m,n).
- * 2. captures a stone adjacent to our neighbors to (m,n), before
- *    (m,n) is played.
+ * 1. defends the position obtained after playing (move).
+ * 2. captures a stone adjacent to our neighbors to (move), before
+ *    (move) is played.
  *
- * Then we check which of these are legal before (m,n) is played. If
+ * Then we check which of these are legal before (move) is played. If
  * there is at least one, we take one of these arbitrarily as a
  * backfilling move.
  *
- * Now it may happen that (m,n) still isn't a safe move. In that case
+ * Now it may happen that (move) still isn't a safe move. In that case
  * we recurse to find a new backfilling move. To do things really
  * correctly we should also give the opponent the opportunity to keep
  * up the balance of the position by letting him do a backfilling move
@@ -367,46 +348,40 @@ fill_liberty(int *i, int *j, int color)
  * function. Currently we only do a half-hearted attempt to find
  * opponent moves.
  */
-static int adji[MAXCHAIN];
-static int adjj[MAXCHAIN];
-
-static int libi[MAXLIBS];
-static int libj[MAXLIBS];
+static int adjs[MAXCHAIN];
+static int libs[MAXLIBS];
 
 static int
-find_backfilling_move(int m, int n, int color, int *i, int *j)
+find_backfilling_move(int move, int color, int *backfill_move)
 {
   int k;
-  int libs;
+  int liberties;
   int neighbors;
   int found_one = 0;
-  int ai = -1;
-  int aj = -1;
+  int apos = NO_MOVE;
+  int bpos = NO_MOVE;
   int extra_pop = 0;
   int success = 0;
   int acode;
-  int bi = -1;
-  int bj = -1;
-  int savei = -1;
-  int savej = -1;
+  int saved_move = NO_MOVE;
   
-  /* Play (m,n) and identify all liberties and adjacent strings. */
-  if (!trymove(m, n, color, "find_backfilling_move", m, n, EMPTY, -1, -1))
+  /* Play (move) and identify all liberties and adjacent strings. */
+  if (!trymove(move, color, "find_backfilling_move", move, EMPTY, NO_MOVE))
     return 0; /* This shouldn't happen, I believe. */
 
   /* The move wasn't safe, so there must be an attack for the
    * opponent. Save it for later use.
    */
-  acode = attack(m, n, &ai, &aj);
-  gg_assert(acode != 0 && ai >= 0 && aj >= 0);
+  acode = attack(move, &apos);
+  gg_assert(acode != 0 && apos > NO_MOVE);
   
   /* Find liberties. */
-  libs = findlib(m, n, MAXLIBS, libi, libj);
+  liberties = findlib(move, MAXLIBS, libs);
 
   /* Find neighbors. */
-  neighbors = chainlinks(m, n, adji, adjj);
+  neighbors = chainlinks(move, adjs);
 
-  /* Remove (m,n) again. */
+  /* Remove (move) again. */
   popgo();
   
   /* It's most fun to capture stones. Start by trying to take some
@@ -426,25 +401,21 @@ find_backfilling_move(int m, int n, int color, int *i, int *j)
    * good enough.
    */
   for (k = 0; k < neighbors; k++) {
-    if (attack(adji[k], adjj[k], &bi, &bj) == WIN) {
-      if (liberty_of_string(bi, bj, adji[k], adjj[k])) {
-	*i = bi;
-	*j = bj;
+    if (attack(adjs[k], &bpos) == WIN) {
+      if (liberty_of_string(bpos, adjs[k])) {
+	*backfill_move = bpos;
 	return 1;
       }
-      else {
-	savei = bi;
-	savej = bj;
-      }
+      else
+	saved_move = bpos;
     }
   }
   
   /* Otherwise look for a safe move at a liberty. */
   if (!found_one) {
-    for (k = 0; k < libs; k++) {
-      if (safe_move(libi[k], libj[k], color) == WIN) {
-	*i = libi[k];
-	*j = libj[k];
+    for (k = 0; k < liberties; k++) {
+      if (safe_move(libs[k], color) == WIN) {
+	*backfill_move = libs[k];
 	found_one = 1;
 	break;
       }
@@ -453,35 +424,49 @@ find_backfilling_move(int m, int n, int color, int *i, int *j)
 
   /* If no luck so far, try with superstring liberties. */
   if (!found_one) {
-    trymove(m, n, color, "find_backfilling_move", m, n, EMPTY, -1, -1);
-    find_proper_superstring_liberties(m, n, &libs, libi, libj, 0);
+    trymove(move, color, "find_backfilling_move", move, EMPTY, NO_MOVE);
+    find_proper_superstring_liberties(move, &liberties, libs, 0);
     popgo();
-    for (k = 0; k < libs; k++) {
-      if (safe_move(libi[k], libj[k], color) == WIN) {
-	*i = libi[k];
-	*j = libj[k];
+    for (k = 0; k < liberties; k++) {
+      if (safe_move(libs[k], color) == WIN) {
+	*backfill_move = libs[k];
 	found_one = 1;
 	break;
       }
     }
   }
 
+  /* If no luck so far, try attacking superstring neighbors. */
+  if (!found_one) {
+    trymove(move, color, "find_backfilling_move", move, EMPTY, NO_MOVE);
+    superstring_chainlinks(move, &neighbors, adjs, 4);
+    popgo();
+    for (k = 0; k < neighbors; k++) {
+      if (attack(adjs[k], &bpos) == WIN) {
+	if (liberty_of_string(bpos, adjs[k])) {
+	  *backfill_move = bpos;
+	  return 1;
+	}
+      }
+    }
+  }
 
   if (found_one) {
   
-    if (!trymove(*i, *j, color, "find_backfilling_move", m, n, EMPTY, -1, -1))
+    if (!trymove(*backfill_move, color, "find_backfilling_move", move,
+		 EMPTY, NO_MOVE))
       return 0; /* This really shouldn't happen. */
     
     /* Allow opponent to get a move in here. */
-    if (trymove(ai, aj, OTHER_COLOR(color), "find_backfilling_move", m, n, 
-		EMPTY, -1, -1))
+    if (trymove(apos, OTHER_COLOR(color), "find_backfilling_move", move, 
+		EMPTY, NO_MOVE))
       extra_pop = 1;
     
     /* If still not safe, recurse to find a new backfilling move. */
-    if (safe_move(m, n, color) == WIN)
+    if (safe_move(move, color) == WIN)
       success = 1;
     else
-      success = find_backfilling_move(m, n, color, i, j);
+      success = find_backfilling_move(move, color, backfill_move);
 
     /* Pop move(s) and return. */
     if (extra_pop)
@@ -489,9 +474,8 @@ find_backfilling_move(int m, int n, int color, int *i, int *j)
     popgo();
   }
 
-  if (!success && savei != -1) {
-    *i = savei;
-    *j = savej;
+  if (!success && saved_move != NO_MOVE) {
+    *backfill_move = saved_move;
     success = 1;
   }
 
@@ -499,40 +483,38 @@ find_backfilling_move(int m, int n, int color, int *i, int *j)
 }
 
 
-/* Confirm that (m, n) is a safe move for color. In addition to
+/* Confirm that (move) is a safe move for color. In addition to
  * calling the global confirm_safety(), this function also calls the
  * owl code to verify the strategical viability of the move.
  */
 static int
-filllib_confirm_safety(int m, int n, int color, int *di, int *dj)
+filllib_confirm_safety(int move, int color, int *defense_point)
 {
   int k;
-  int ai = -1, aj = -1;
+  int apos = NO_MOVE;
   int save_verbose;
 
   gg_assert(stackp == 0);
-  *di = -1;
-  *dj = -1;
+  gg_assert(defense_point != NULL);
+  *defense_point = NO_MOVE;
 
   /* Before we can call the owl code, we need to find a neighbor of
    * our color.
    */
   for (k = 0; k < 4; k++)
-    if (ON_BOARD(m + deltai[k], n + deltaj[k]) && p[m + deltai[k]][n + deltaj[k]] == color) {
-      ai = m + deltai[k];
-      aj = n + deltaj[k];
+    if (board[move + delta[k]] == color) {
+      apos = move + delta[k];
       break;
     }
 
   /* If none found, look for a neighbor of an attacked adjacent string. */
-  if (ai == -1)
+  if (apos == NO_MOVE)
     for (k = 0; k < 4; k++) {
-      int i = m + deltai[k];
-      int j = n + deltaj[k];
-      if (ON_BOARD(i, j) && p[i][j] == OTHER_COLOR(color)
-	  && !play_attack_defend_n(color, 0, 1, m, n, i, j)) {
+      int pos2 = move + delta[k];
+      if (board[pos2] == OTHER_COLOR(color)
+	  && !play_attack_defend_n(color, 0, 1, move, pos2)) {
 	int adj;
-	adj = chainlinks(i, j, adji, adjj);
+	adj = chainlinks(pos2, adjs);
 	/* It seems unlikely that we would ever get adjacent strings
          * here, but if it should happen we simply give up and say the
          * move is unsafe.
@@ -540,8 +522,7 @@ filllib_confirm_safety(int m, int n, int color, int *di, int *dj)
 	if (adj == 0)
 	  return 0;
 	
-	ai = adji[0];
-	aj = adjj[0];
+	apos = adjs[0];
 	break;
       }
     }
@@ -549,7 +530,7 @@ filllib_confirm_safety(int m, int n, int color, int *di, int *dj)
   /* We should have found something by now. If not something's
    * probably broken elsewhere. Declare the move unsafe if it happens.
    */
-  if (ai == -1)
+  if (apos == NO_MOVE)
     return 0;
 
   /* Ask the owl code whether this move is strategically viable. */
@@ -557,11 +538,11 @@ filllib_confirm_safety(int m, int n, int color, int *di, int *dj)
   save_verbose = verbose;
   if (verbose > 0)
     verbose--;
-  if (!owl_does_defend(m, n, ai, aj))
+  if (!owl_does_defend(move, apos))
     return 0;
   verbose = save_verbose;
   
-  return confirm_safety(m, n, color, 0, di, dj);
+  return confirm_safety(move, color, 0, defense_point, NULL, NULL);
 }
 
 
