@@ -1,25 +1,25 @@
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
- * http://www.gnu.org/software/gnugo/ for more information.      *
- *                                                               *
- * Copyright 1999, 2000, 2001 by the Free Software Foundation.   *
- *                                                               *
- * This program is free software; you can redistribute it and/or *
- * modify it under the terms of the GNU General Public License   *
- * as published by the Free Software Foundation - version 2.     *
- *                                                               *
- * This program is distributed in the hope that it will be       *
- * useful, but WITHOUT ANY WARRANTY; without even the implied    *
- * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
- * PURPOSE.  See the GNU General Public License in file COPYING  *
- * for more details.                                             *
- *                                                               *
- * You should have received a copy of the GNU General Public     *
- * License along with this program; if not, write to the Free    *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
- * Boston, MA 02111, USA.                                        *
-\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
+ * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * http://www.gnu.org/software/gnugo/ for more information.          *
+ *                                                                   *
+ * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ *                                                                   *
+ * This program is free software; you can redistribute it and/or     *
+ * modify it under the terms of the GNU General Public License as    *
+ * published by the Free Software Foundation - version 2             *
+ *                                                                   *
+ * This program is distributed in the hope that it will be useful,   *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
+ * GNU General Public License in file COPYING for more details.      *
+ *                                                                   *
+ * You should have received a copy of the GNU General Public         *
+ * License along with this program; if not, write to the Free        *
+ * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Boston, MA 02111, USA.                                            *
+\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
+#include "gnugo.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -27,6 +27,7 @@
 
 #include "liberty.h"
 #include "sgftree.h"
+#include "random.h"
 
 /* Return one if x doesn't equal position_number and 0 otherwise.
  * After using this macro x will always have the value
@@ -35,13 +36,22 @@
 #define NEEDS_UPDATE(x) (x != position_number ? (x = position_number, 1) : 0)
 
 static int get_level(int *level);
-static int do_genmove(int *i, int *j, int color, float pure_threat_value);
+static int do_genmove(int *move, int color, float pure_threat_value);
 
-static double slowest_time = 0.;
-static int slowest_i = -1;
-static int slowest_j = -1;
-static int slowest_movenum = 0;
-static double total_time = 0.;
+static double slowest_time = 0.0;
+static int    slowest_move = NO_MOVE;
+static int    slowest_movenum = 0;
+static double total_time = 0.0;
+
+/* Position numbers for which various examinations were last made. */
+static int worms_examined = -1;
+static int initial_influence_examined = -1;
+static int dragons_examined_without_owl = -1;
+static int dragons_examined = -1;
+static int initial_influence2_examined = -1;
+
+static int revise_semeai(int color);
+static int revise_thrashing_dragon(int color, float advantage);
 
 void sgfShowConsideredMoves(void);
 
@@ -55,9 +65,22 @@ void sgfShowConsideredMoves(void);
 void
 reset_engine()
 {
+  /* To improve the reproducability of games, we restart the random
+   * number generator with the same seed for each move. Thus we don't
+   * have to know how many previous moves have been played, nor
+   * actually play through them, in order to get the right random
+   * numbers.
+   */
+  gg_srand(random_seed);
+
   /* Initialize things for hashing of positions. */
   reading_cache_clear();
-  hashdata_recalc(&hashdata, p, board_ko_i, board_ko_j);
+  hashdata_recalc(&hashdata, board, board_ko_pos);
+  worms_examined = -1;
+  initial_influence_examined = -1;
+  dragons_examined_without_owl = -1;
+  dragons_examined = -1;
+  initial_influence2_examined = -1;
 
   /* Prepare our table of move reasons. */
   clear_move_reasons();
@@ -82,38 +105,24 @@ reset_engine()
 void
 examine_position(int color, int how_much)
 {
-  int save_verbose;
-  int m, n;
-  double t1 = 0., t2 = 0.;
-
-  /* Position numbers for which various examinations were last made. */
-  static int worms_examined = -1;
-  static int initial_influence_examined = -1;
-  static int dragons_examined_without_owl = -1;
-  static int dragons_examined = -1;
-  static int owl_reasons_examined = -1;
-  static int initial_influence2_examined = -1;
+  int save_verbose = verbose;
 
   purge_persistent_reading_cache();
   
   /* Don't print reading traces during make_worms and make_dragons unless 
    * the user really wants it (verbose == 3). 
    */
-  save_verbose = verbose;
-  if ((verbose == 1) || (verbose == 2))
+  if (verbose == 1 || verbose == 2)
     --verbose;
 
   if (NEEDS_UPDATE(worms_examined)) {
-    if (showtime)
-      t1 = gg_gettimeofday();
-    make_worms();
-    if (showtime) {
-      t2 = gg_gettimeofday();
-      fprintf(stderr, "  make worms: %.2f sec\n", t2-t1);
-    }
+    start_timer(0);
+    make_worms(save_verbose);
+    time_report(0, "  make worms", NO_MOVE, 1.0);
   }
   if (how_much == EXAMINE_WORMS) {
     verbose = save_verbose;
+    gg_assert(test_gray_border() < 0);
     return;
   }
 
@@ -122,61 +131,77 @@ examine_position(int color, int how_much)
       compute_initial_influence(color, 0);
     if (how_much == EXAMINE_INITIAL_INFLUENCE) {
       verbose = save_verbose;
+      gg_assert(test_gray_border() < 0);
       return;
     }
 
     if (how_much == EXAMINE_DRAGONS_WITHOUT_OWL) {
       if (NEEDS_UPDATE(dragons_examined_without_owl))
-	make_dragons(color, 1);
+	make_dragons(color, 1, save_verbose);
       verbose = save_verbose;
+      gg_assert(test_gray_border() < 0);
       return;
     }
     
     if (NEEDS_UPDATE(dragons_examined)) {
-      make_dragons(color, 0);
+      make_dragons(color, 0, save_verbose);
       /* We have automatically done a partial dragon analysis as well. */
       dragons_examined_without_owl = position_number;
     }
     if (how_much == EXAMINE_DRAGONS) {
       verbose = save_verbose;
-      return;
-    }
-
-    if (NEEDS_UPDATE(owl_reasons_examined))
-      owl_reasons(color);
-    if (how_much == EXAMINE_OWL_REASONS) {
-      verbose = save_verbose;
+      gg_assert(test_gray_border() < 0);
       return;
     }
   }
   else if (how_much == EXAMINE_INITIAL_INFLUENCE
 	   || how_much == EXAMINE_DRAGONS
-	   || how_much == EXAMINE_OWL_REASONS) {
+	   || how_much == EXAMINE_ALL) {
+    initialize_dragon_data();
     verbose = save_verbose;
+    gg_assert(test_gray_border() < 0);
     return;
   }
   
   verbose = save_verbose;
 
-  if (printworms) {
+  if (printworms)
     show_dragons();
-  }
-  else if (verbose) {
-    for (m = 0; m < board_size; m++)
-      for (n = 0; n < board_size; n++)
-	if (p[m][n]
-	    && worm[m][n].origini == m
-	    && worm[m][n].originj == n
-	    && worm[m][n].cutstone) {
-	  TRACE("%scutting stones found at %m\n",
-		worm[m][n].cutstone==2 ? "potential " : "", m,n); 
-	}
-  }	  
 
   if (NEEDS_UPDATE(initial_influence2_examined))
     compute_initial_influence(color, 1);
-  if (how_much == EXAMINE_INITIAL_INFLUENCE2)
+  if (how_much == EXAMINE_INITIAL_INFLUENCE2) {
+    gg_assert(test_gray_border() < 0);
     return;
+  }
+}
+
+
+/* The same as examine_position(), except that all traces, debug
+ * output, and sgf traces are turned off.
+ */
+void
+silent_examine_position(int color, int how_much)
+{
+  int save_verbose = verbose;
+  SGFTree *save_sgf_dumptree = sgf_dumptree;
+  int save_count_variations = count_variations;
+  int save_debug = debug;
+  int save_printmoyo = printmoyo;
+  
+  verbose = 0;
+  sgf_dumptree = NULL;
+  count_variations = 0;
+  debug = 0;
+  printmoyo = 0;
+  
+  examine_position(color, how_much);
+
+  verbose = save_verbose;
+  sgf_dumptree = save_sgf_dumptree;
+  count_variations = save_count_variations;
+  debug = save_debug;
+  printmoyo = save_printmoyo;
 }
 
 
@@ -201,6 +226,7 @@ old_estimate_score(int color, float *lower_bound, float *upper_bound)
     /* Find out information about the worms and dragons. */
     examine_position(color, EXAMINE_ALL);
     compute_initial_influence(OTHER_COLOR(color), 1);
+
     /* The coefficients should match those close to the end of
      * estimate_influence_value() in move_reasons.c.
      */
@@ -228,6 +254,7 @@ old_estimate_score(int color, float *lower_bound, float *upper_bound)
 
   *lower_bound = lower + komi;
   *upper_bound = upper + komi;
+  gg_assert(test_gray_border() < 0);
 }
 
 
@@ -241,7 +268,15 @@ old_estimate_score(int color, float *lower_bound, float *upper_bound)
 int
 genmove(int *i, int *j, int color)
 {
-  return do_genmove(i, j, color, 0.4);
+  int move;
+  int retval;
+
+  retval = do_genmove(&move, color, 0.4);
+
+  if (i) *i = I(move);
+  if (j) *j = J(move);
+
+  return retval;
 }
 
 
@@ -253,9 +288,16 @@ genmove(int *i, int *j, int color)
 int
 genmove_conservative(int *i, int *j, int color)
 {
-  return do_genmove(i, j, color, 0.0);
-}
+  int move;
+  int retval;
 
+  retval = do_genmove(&move, color, 0.0);
+
+  if (i) *i = I(move);
+  if (j) *j = J(move);
+
+  return retval;
+}
 
 
 /* 
@@ -263,14 +305,12 @@ genmove_conservative(int *i, int *j, int color)
  */
   
 static int
-do_genmove(int *i, int *j, int color, float pure_threat_value)
+do_genmove(int *move, int color, float pure_threat_value)
 {
   float val;
-  double t1 = 0., t2 = 0.;
-  double t3 = 0., t4 = 0.;
+  int save_verbose;
 
-  if (showtime)
-    t1 = gg_gettimeofday();
+  start_timer(0);
   
   /* Prepare our table of moves considered. */
   memset(potential_moves, 0, sizeof(potential_moves));
@@ -284,8 +324,7 @@ do_genmove(int *i, int *j, int color, float pure_threat_value)
   stats.hash_collisions     = 0;
   
   /* no move is found yet. */
-  *i = -1;  
-  *j = -1;  
+  *move = NO_MOVE;  
   val = -1; 
   if (get_level(&level))
     fprintf(stderr, "level = %d\n", level);
@@ -297,14 +336,14 @@ do_genmove(int *i, int *j, int color, float pure_threat_value)
   reset_engine();
 
   /* Find out information about the worms and dragons. */
-  if (showtime)
-    t3 = gg_gettimeofday();
+  start_timer(1);
   examine_position(color, EXAMINE_ALL);
-  if (showtime) {
-    t4 = gg_gettimeofday();
-    fprintf(stderr, "EXAMINE POSITION took %.2f sec\n", t4-t3);
-    t3 = t4;
-  }
+  time_report(1, "examine position", NO_MOVE, 1.0);
+
+  /* Make a score estimate. This can be used in later stages of the 
+   * move generation.  If we are ahead, we can play safely and if
+   * we are behind, we have to play more daringly.
+   */
   if (level >= 8) {
     estimate_score(&lower_bound, &upper_bound);
     if (verbose || showscore) {
@@ -315,12 +354,10 @@ do_genmove(int *i, int *j, int color, float pure_threat_value)
 	gprintf("\nScore estimate: %s %f to %s %f\n",
 		lower_bound > 0 ? "W " : "B ", gg_abs(lower_bound),
 		upper_bound > 0 ? "W " : "B ", gg_abs(upper_bound));
+      fflush(stderr);
     }
-    if (showtime) {
-      t4 = gg_gettimeofday();
-      fprintf(stderr, "ESTIMATE SCORE took %.2f sec\n", t4-t3);
-      t3 = t4;
-    }
+    time_report(1, "estimate score", NO_MOVE, 1.0);
+
     /* The score will be used to determine when we are safely
      * ahead. So we want the most conservative score.
      */
@@ -329,23 +366,23 @@ do_genmove(int *i, int *j, int color, float pure_threat_value)
     else
       score = upper_bound;
   }
-  else score = 0.;
+  else
+    score = 0.0;
 
   /*
    * Print some of the information if the user wants to.
    */
-  /* compute_influence(); */
   if (printmoyo)
     print_moyo();
   
   if (printboard) {
     if (printboard == 1)
-      fprintf(stderr,"\n          dragon_status display:\n\n");
+      fprintf(stderr, "\n          dragon_status display:\n\n");
     if (printboard == 2)
-      fprintf(stderr,"\n          eye display:\n\n");
+      fprintf(stderr, "\n          eye display:\n\n");
     showboard(printboard); 
     if (printboard == 1) {
-      fprintf(stderr,"\n           owl_status display:\n\n");      
+      fprintf(stderr, "\n           owl_status display:\n\n");      
       showboard(3);
     }
   }
@@ -355,72 +392,64 @@ do_genmove(int *i, int *j, int color, float pure_threat_value)
   /*
    * Ok, information gathering is complete. Now start to find some moves!
    */
+
+  /* Pick up tactical moves. */
+  worm_reasons(color);
+  
+  /* Pick up owl moves. */
+  save_verbose = verbose;
+  if (verbose > 0)
+    verbose--;
+  owl_reasons(color);
+  verbose = save_verbose;
   
   /* Try to find empty corner moves. */
   fuseki(color);
   gg_assert(stackp == 0);
 
-  /* Pattern matcher. */
-  if (showtime)
-    t3 = gg_gettimeofday();
+  /* The general pattern database. */
+  start_timer(1);
   shapes(color);
-  if (showtime) {
-    t4 = gg_gettimeofday();
-    fprintf(stderr, "SHAPES took %.2f sec\n", t4-t3);
-    t3 = t4;
-  }
+  time_report(1, "shapes", NO_MOVE, 1.0);
   gg_assert(stackp == 0);
 
-  /* Look for combination attacks. */
-  {
-    int ai, aj;
-    int other = OTHER_COLOR(color);
-    int aa_val;
-
-    int save_verbose = verbose;
-    if (verbose > 0)
-      verbose--;
-
-    if (save_verbose)
-      gprintf("\nlooking for combination attacks ...\n");
-    aa_val = atari_atari(color, &ai, &aj, save_verbose);
-    if (aa_val)
-      add_my_atari_atari_move(ai, aj, aa_val);
-    aa_val = atari_atari(other, &ai, &aj, save_verbose);
-    if (aa_val && safe_move(ai, aj, color))
-      add_your_atari_atari_move(ai, aj, aa_val);
-    verbose = save_verbose;
-  }
-  if (showtime) {
-    t4 = gg_gettimeofday();
-    fprintf(stderr, "ATARI ATARI took %.2f sec\n", t4-t3);
-    t3 = t4;
-  }
+  /* Look for combination attacks and defenses against them. */
+  combinations(color);
+  time_report(1, "combinations", NO_MOVE, 1.0);
+  gg_assert(stackp == 0);
 
   /* Review the move reasons and estimate move values. */
-  if (review_move_reasons(i, j, &val, color, 
+  if (review_move_reasons(move, &val, color, 
 			  pure_threat_value, lower_bound))
-    TRACE("Move generation likes %m with value %f\n", *i, *j, val);
+    TRACE("Move generation likes %1m with value %f\n", *move, val);
   gg_assert(stackp == 0);
-  if (showtime) {
-    t4 = gg_gettimeofday();
-    fprintf(stderr, "REVIEW MOVE REASONS took %.2f sec\n", t4-t3);
-    t3 = t4;
+  time_report(1, "review move reasons", NO_MOVE, 1.0);
+
+  /* If we are ahead by 15 points or more, consider a thrashing
+   * dragon dangerous and change its status from DEAD to
+   * UNKNOWN. This may generate a move.
+   */
+  if (val < 10.0 && !doing_scoring) {
+    if (revise_thrashing_dragon(color, 15.0)) {
+      shapes(color);
+      if (!disable_endgame_patterns)
+	endgame_shapes(color);
+      if (review_move_reasons(move, &val, color, pure_threat_value, score)) {
+	TRACE("Upon reconsideration move generation likes %1m with value %f\n",
+	      *move, val); 
+      }
+    }
+    time_report(1, "move reasons with revised semeai status", NO_MOVE, 1.0);
   }
 
   /* If the move value is 6 or lower, we look for endgame patterns too. */
   if (val <= 6.0 && !disable_endgame_patterns) {
     endgame_shapes(color);
     gg_assert(stackp == 0);
-    if (review_move_reasons(i, j, &val, color, pure_threat_value, score))
-      TRACE("Move generation likes %m with value %f\n", *i, *j, val);
+    if (review_move_reasons(move, &val, color, pure_threat_value, score))
+      TRACE("Move generation likes %1m with value %f\n", *move, val);
     gg_assert(stackp == 0);
-    if (showtime) {
-      t4 = gg_gettimeofday();
-      if (t4-t3 > 1.)
-	fprintf(stderr, "ENDGAME took %.2f sec\n", t4-t3);
-      t3 = t4;
-    }
+    time_report(1, "endgame", NO_MOVE, 1.0);
   }
   
   /* If no move found yet, revisit any semeai and change the
@@ -428,54 +457,47 @@ do_genmove(int *i, int *j, int color, float pure_threat_value)
    * run shapes and endgame_shapes again. This may turn up a move.
    */
   if (val < 0.0) {
-    if (revise_semeai(color)) {
+    if (revise_thrashing_dragon(color, 0.0)
+	|| revise_semeai(color)) {
       shapes(color);
       endgame_shapes(color);
-      if (review_move_reasons(i, j, &val, color, pure_threat_value, score)) {
-	TRACE("Upon reconsideration move generation likes %m with value %f\n",
-	      *i, *j, val); 
+      if (review_move_reasons(move, &val, color, pure_threat_value, score)) {
+	TRACE("Upon reconsideration move generation likes %1m with value %f\n",
+	      *move, val); 
       }
     }
-    if (showtime) {
-      t4 = gg_gettimeofday();
-      if (t4-t3 > 1.)
-	fprintf(stderr, 
-		"MOVE REASONS WITH REVISED SEMEAI STATUS took %.2f sec\n", 
-		t4-t3);
-      t3 = t4;
-    }
+    time_report(1, "move reasons with revised semeai status", NO_MOVE, 1.0);
   }
 
   /* If still no move, fill a remaining liberty. This should pick up
    * all missing dame points.
    */
   if (val < 0.0 
-      && fill_liberty(i, j, color)) {
+      && fill_liberty(move, color)) {
     val = 1.0;
-    TRACE("Filling a liberty at %m\n", *i, *j);
-    move_considered(*i, *j, val);
-    if (showtime) {
-      t4 = gg_gettimeofday();
-      fprintf(stderr, "FILL LIBERTY took %.2f sec\n", t4-t3);
-      t3 = t4;
-    }
+    TRACE("Filling a liberty at %1m\n", *move);
+    move_considered(*move, val);
+    time_report(1, "fill liberty", NO_MOVE, 1.0);
   }
 
   /* If we're instructed to play out the aftermath or capture all dead
-   * opponent stones, generate an aftermath move.
+   * opponent stones, or if the opponent is trying to live inside
+   * our territory and we are clearly ahead, generate an aftermath move.
    */
+
   if (val < 0.0
       && !doing_scoring
-      && (play_out_aftermath || capture_all_dead)
-      && aftermath_genmove(i, j, color, NULL, 0) > 0) {
+      && (play_out_aftermath 
+	  || capture_all_dead 
+	  || (thrashing_dragon
+	      && ((color == BLACK && score < -15.0)
+		  || (color == WHITE && score > 15.0))))
+      && aftermath_genmove(move, color, NULL, 0) > 0) {
+    ASSERT1(is_legal(*move, color), *move);
     val = 1.0;
-    TRACE("Aftermath move at %m\n", *i, *j);
-    move_considered(*i, *j, val);
-    if (showtime) {
-      t4 = gg_gettimeofday();
-      fprintf(stderr, "AFTERMATH_GENMOVE took %.2f sec\n", t4-t3);
-      t3 = t4;
-    }
+    TRACE("Aftermath move at %1m\n", *move);
+    move_considered(*move, val);
+    time_report(1, "aftermath_genmove", NO_MOVE, 1.0);
   }
 
   /* If we're instructed to capture all dead opponent stones, generate
@@ -483,63 +505,56 @@ do_genmove(int *i, int *j, int color, float pure_threat_value)
    */
   if (val < 0.0
       && !doing_scoring
-      && capture_all_dead
-      && aftermath_genmove(i, j, color, NULL, 1) > 0) {
+      && capture_all_dead 
+      && aftermath_genmove(move, color, NULL, 1) > 0) {
+    ASSERT1(is_legal(*move, color), *move);
     val = 1.0;
-    TRACE("Aftermath move at %m\n", *i, *j);
-    move_considered(*i, *j, val);
-    if (showtime) {
-      t4 = gg_gettimeofday();
-      fprintf(stderr, "AFTERMATH_GENMOVE took %.2f sec\n", t4-t3);
-      t3 = t4;
-    }
+    TRACE("Aftermath move at %1m\n", *move);
+    move_considered(*move, val);
+    time_report(1, "aftermath_genmove", NO_MOVE, 1.0);
   }
 
   /* If no move is found then pass. */
   if (val < 0.0) {
     TRACE("I pass.\n");
-    *i = -1;
-    *j = -1;
+    *move = NO_MOVE;
   }
   else
-    TRACE("genmove() recommends %m with value %f\n", *i, *j, val);
+    TRACE("genmove() recommends %1m with value %f\n", *move, val);
   
   /* If statistics is turned on, this is the place to show it. */
   if (showstatistics) {
-    printf("Nodes:                %d\n", stats.nodes);
-    printf("Positions entered:    %d\n", stats.position_entered);
-    printf("Position hits:        %d\n", stats.position_hits);
-    printf("Read results entered: %d\n", stats.read_result_entered);
-    printf("Read result hits:     %d\n", stats.read_result_hits);
-    printf("Hash collisions:      %d\n", stats.hash_collisions);
+    gprintf("Nodes:                %d\n", stats.nodes);
+    gprintf("Positions entered:    %d\n", stats.position_entered);
+    gprintf("Position hits:        %d\n", stats.position_hits);
+    gprintf("Read results entered: %d\n", stats.read_result_entered);
+    gprintf("Read result hits:     %d\n", stats.read_result_hits);
+    gprintf("Hash collisions:      %d\n", stats.hash_collisions);
   }
-
-  if (showtime) {
-    t2 = gg_gettimeofday();
-    gprintf("*** TIME to generate move at %m: ", *i, *j);
-    fprintf(stderr, "%.2f sec\n", t2-t1);
-    total_time += t2-t1;
-    if (t2-t1 > slowest_time) {
-      slowest_time = t2-t1;
-      slowest_i = *i;
-      slowest_j = *j;
-      slowest_movenum = movenum+1;
+ 
+ if (showtime) {
+    double spent = time_report(0, "TIME to generate move at ", *move, 1.0);
+    total_time += spent;
+    if (spent > slowest_time) {
+      slowest_time = spent;
+      slowest_move = *move;
+      slowest_movenum = movenum + 1;
     }
-    if (*i == -1) {
-      gprintf("\nSLOWEST MOVE: %d at %m ", slowest_movenum,
-	      slowest_i, slowest_j);
+    if (*move == NO_MOVE) {
+      gprintf("\nSLOWEST MOVE: %d at %1m ", slowest_movenum, slowest_move);
       fprintf(stderr, "(%.2f seconds)\n", slowest_time);
       fprintf(stderr, "\nAVERAGE TIME: %.2f seconds per move\n",
-	      total_time/movenum);
+	      total_time / movenum);
       fprintf(stderr, "\nTOTAL TIME: %.2f seconds\n",
 	      total_time);
 
     }
   }
   
+  gg_assert(test_gray_border() < 0);
   return val;
-  
-}  /* end genmove */
+}
+
 
 
 /* This is called for each move which has been considered. For
@@ -548,8 +563,10 @@ do_genmove(int *i, int *j, int color, float pure_threat_value)
  */
 
 void 
-move_considered(int i, int j, float val)
+move_considered(int move, float val)
 {
+  int i = I(move);
+  int j = J(move);
   if (val > potential_moves[i][j]) {
     potential_moves[i][j] = val;
   }
@@ -566,9 +583,9 @@ get_level(int *level)
   char buffer[128];
   FILE *fp;
 
-  const char filename[]="level";
+  const char filename[] = "level";
 
-  if ((fp=fopen(filename,"r"))==NULL)
+  if ((fp = fopen(filename, "r")) == NULL)
     return 0;
 
   if (fgets(buffer, 128, fp)) {
@@ -583,11 +600,11 @@ get_level(int *level)
   
 
 /* ================================================================ */
-/*       Set up fixed placement handicap pieces for black side      */
+/*       Set up fixed placement handicap stones for black side      */
 /* ================================================================ */
 
 
-/* Handicap pieces are set up according to the following diagrams:
+/* Handicap stones are set up according to the following diagrams:
  *  
  * 2 stones:                    3 stones:           
  *
@@ -660,7 +677,8 @@ get_level(int *level)
  *
  * Notice that this function only deals with fixed handicap placement.
  * Larger handicaps can be added by free placement if the used
- * interface supports it. */
+ * interface supports it.
+ */
 
 
 /* This table contains the (coded) positions of the stones.
@@ -669,21 +687,21 @@ get_level(int *level)
  * -ve numbers map to  board_size - number
  *
  * The stones are placed in this order, *except* if there are
- * 5 or 7 stones, in which case center ( {0,0} ) is placed, and
+ * 5 or 7 stones, in which case center ({0, 0}) is placed, and
  * then as for 4 or 6.
  */
 
 static const int places[][2] = {
 
-  {2,-2}, {-2,2}, {2,2}, {-2,-2},  /* first 4 are easy */
-                                   /* for 5, {0,0} is explicitly placed */
+  {2, -2}, {-2, 2}, {2, 2}, {-2, -2}, /* first 4 are easy */
+                                      /* for 5, {0,0} is explicitly placed */
   
-  {0,2},  {0,-2},                  /* for 6 these two are placed */
-                                   /* for 7, {0,0} is explicitly placed */
+  {0, 2}, {0, -2},                    /* for 6 these two are placed */
+                                      /* for 7, {0,0} is explicitly placed */
   
-  {2,0}, {-2,0},                   /* for 8, these two are placed */
+  {2, 0}, {-2, 0},                    /* for 8, these two are placed */
 
-  {0,0},                           /* finally tengen for 9 */
+  {0, 0},                             /* finally tengen for 9 */
 };
 
 
@@ -723,7 +741,7 @@ placehand(int handicap)
 
   /* special cases: 5 and 7 */
   if (handicap == 5 || handicap == 7) {
-    add_stone(mid, mid, BLACK);
+    add_stone(POS(mid, mid), BLACK);
     handicap--;
   }
 
@@ -741,16 +759,83 @@ placehand(int handicap)
     if (i == 0) i = mid;
     if (j == 0) j = mid;
 
-    if ( i < 0) i += board_size-1;
-    if ( j < 0) j += board_size-1;
+    if (i < 0) i += board_size-1;
+    if (j < 0) j += board_size-1;
 
-    add_stone(i, j, BLACK);
+    add_stone(POS(i, j), BLACK);
   }
 
   return retval;
 }
 
 
+/* revise_semeai(color) changes the status of any DEAD dragon of
+ * OPPOSITE_COLOR(color) which occurs in a semeai to UNKNOWN.
+ * It returns true if such a dragon is found.
+ */
+
+static int
+revise_semeai(int color)
+{
+  int pos;
+  int found_one = 0;
+  int other = OTHER_COLOR(color);
+
+  gg_assert(dragon2 != NULL);
+
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+    if (ON_BOARD(pos)
+	&& dragon[pos].color == other
+	&& DRAGON2(pos).semeai
+	&& dragon[pos].matcher_status == DEAD) {
+      found_one = 1;
+      dragon[pos].matcher_status = UNKNOWN;
+      if (dragon[pos].origin == pos)
+	TRACE("revise_semeai: changed status of dragon %1m from DEAD to UNKNOWN\n",
+	      pos);
+    }
+  }
+  
+  return found_one;
+}
+
+
+/* If the opponent's last move added a stone to a dead dragon,
+ * revise it's status to UNKNOWN. This will cause genmove to
+ * generate moves restraining the dragon. We only do this if
+ * we are ahead by 'advantage', and no owl threat has been found.
+ */
+
+static int
+revise_thrashing_dragon(int color, float advantage)
+{
+  int pos;
+
+  /* Trust the owl code's opinion if we are behind. */
+  if ((color == BLACK && score > -advantage)
+      || (color == WHITE && score < advantage))
+    return 0;
+
+  /* If an owl threat has been found, a move reason has
+   * probably been generated, so we skip this step. */
+  if (dragon[thrashing_dragon].owl_threat_status == CAN_THREATEN_DEFENSE)
+    return 0;
+
+  if (disable_threat_computation
+      || !thrashing_dragon 
+      || dragon[thrashing_dragon].matcher_status != DEAD)
+    return 0;
+  
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++)
+    if (ON_BOARD(pos)
+	&& is_same_dragon(pos, thrashing_dragon))
+      dragon[pos].matcher_status = UNKNOWN;
+  return 1;
+}
+
+
+
+
 /*
  * Local Variables:
  * tab-width: 8
