@@ -1,25 +1,23 @@
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
- * http://www.gnu.org/software/gnugo/ for more information.      *
- *                                                               *
- * Copyright 1999, 2000, 2001 by the Free Software Foundation.   *
- *                                                               *
- * This program is free software; you can redistribute it and/or *
- * modify it under the terms of the GNU General Public License   *
- * as published by the Free Software Foundation - version 2.     *
- *                                                               *
- * This program is distributed in the hope that it will be       *
- * useful, but WITHOUT ANY WARRANTY; without even the implied    *
- * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
- * PURPOSE.  See the GNU General Public License in file COPYING  *
- * for more details.                                             *
- *                                                               *
- * You should have received a copy of the GNU General Public     *
- * License along with this program; if not, write to the Free    *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
- * Boston, MA 02111, USA.                                        *
-\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
+ * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * http://www.gnu.org/software/gnugo/ for more information.          *
+ *                                                                   *
+ * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ *                                                                   *
+ * This program is free software; you can redistribute it and/or     *
+ * modify it under the terms of the GNU General Public License as    *
+ * published by the Free Software Foundation - version 2             *
+ *                                                                   *
+ * This program is distributed in the hope that it will be useful,   *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
+ * GNU General Public License in file COPYING for more details.      *
+ *                                                                   *
+ * You should have received a copy of the GNU General Public         *
+ * License along with this program; if not, write to the Free        *
+ * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Boston, MA 02111, USA.                                            *
+\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 /* ---------------------------------------------------------------- *
  * gnugo.h
@@ -58,8 +56,12 @@ void init_gnugo(float memory);
 #define GRAY_BORDER  3
 #define WHITE_BORDER 4
 #define BLACK_BORDER 5
+#define ANCHOR_COLOR 6
+#define ANCHOR_OTHER 7
+
 
 #define OTHER_COLOR(color)  (WHITE+BLACK-(color))
+#define IS_STONE(arg)  ((arg) == WHITE || (arg) == BLACK)
 
 /* Return codes for reading functions */
 
@@ -103,11 +105,12 @@ void init_gnugo(float memory);
 
 
 /* Board sizes */
-#define MIN_BOARD     3		/* minimum supported board size */
-#define MAX_BOARD    21         /* maximum supported board size */
-#define MAX_HANDICAP  9		/* maximum supported handicap   */
+#define MIN_BOARD          5	   /* Minimum supported board size.   */
+#define MAX_BOARD         19       /* Maximum supported board size.   */
+#define MAX_HANDICAP       9	   /* Maximum supported handicap.     */
+#define MAX_MOVE_HISTORY 500       /* Max number of moves remembered. */
 
-/* This type is used to store each piece on the board.
+/* This type is used to store each intersection on the board.
  *
  * On a 486, char is best, since the time taken to push and pop
  * becomes significant otherwise. On other platforms, an int may
@@ -117,45 +120,36 @@ void init_gnugo(float memory);
 
 typedef unsigned char Intersection;
 
-typedef struct {
-  int          boardsize;
-  Intersection board[MAX_BOARD][MAX_BOARD];
-  int          ko_i;
-  int          ko_j;
-  int          last_i[2];
-  int          last_j[2];
 
-  float        komi;
-  int          white_captured;
-  int          black_captured;
-} Position;
+void gnugo_clear_board(int boardsize);
+void gnugo_set_komi(float new_komi);
+void gnugo_add_stone(int i, int j, int color);
+void gnugo_remove_stone(int i, int j);
+int  gnugo_is_pass(int i, int j);
+void gnugo_play_move(int i, int j, int color);
+int  gnugo_undo_move(int n);
+int  gnugo_play_sgfnode(SGFNode *node, int to_move);
+int  gnugo_play_sgftree(SGFNode *root, int *until, SGFNode **curnode);
+int  gnugo_is_legal(int i, int j, int color);
+int  gnugo_is_suicide(int i, int j, int color);
 
-void gnugo_clear_position(Position *pos, int boardsize, float komi);
-void gnugo_copy_position(Position *to, Position *from);
-void gnugo_add_stone(Position *pos, int i, int j, int color);
-void gnugo_remove_stone(Position *pos, int i, int j);
-void gnugo_play_move(Position *pos, int i, int j, int color);
-int  gnugo_play_sgfnode(Position *pos, SGFNode *node, int to_move);
-int  gnugo_play_sgftree(Position *position, SGFNode *root, int *until, 
-			SGFNode **curnode);
-int  gnugo_is_legal(Position *pos, int i, int j, int color);
-int  gnugo_is_suicide(Position *pos, int i, int j, int color);
+int  gnugo_placehand(int handicap);
+int  gnugo_sethand(int handicap, SGFNode *root);
+void gnugo_recordboard(SGFNode *node);
 
-int  gnugo_placehand(Position *pos, int handicap);
-int  gnugo_sethand(Position *pos, int handicap, SGFNode *root);
-void gnugo_recordboard(Position *pos, SGFNode *node);
+int  gnugo_genmove(int *i, int *j, int color);
 
-int  gnugo_genmove(Position *pos, int *i, int *j, int color,
-		   int move_number);
+int  gnugo_attack(int m, int n, int *i, int *j);
+int  gnugo_find_defense(int m, int n, int *i, int *j);
 
-int  gnugo_attack(Position *pos, int m, int n, int *i, int *j);
-int  gnugo_find_defense(Position *pos, int m, int n, int *i, int *j);
-
-void  gnugo_who_wins(Position *pos, int color, FILE *outfile);
-float gnugo_estimate_score(Position *pos, float *upper, float *lower);
-void  gnugo_force_to_globals(Position *pos);
-void  gnugo_examine_position(Position *pos, int color, int how_much);
+void  gnugo_who_wins(int color, FILE *outfile);
+float gnugo_estimate_score(float *upper, float *lower);
+void  gnugo_examine_position(int color, int how_much);
 
+int  gnugo_get_komi(void);
+void gnugo_get_board(int b[MAX_BOARD][MAX_BOARD]);
+int  gnugo_get_boardsize(void);
+int  gnugo_get_move_number(void);
 
 /* ================================================================ */
 /*                           Game handling                          */
@@ -165,12 +159,9 @@ void  gnugo_examine_position(Position *pos, int color, int how_much);
 typedef struct {
   int       handicap;
 
-  Position  position;
-  int       move_number;
   int       to_move;		/* whose move it currently is */
-  SGFTree   moves;		/* The moves in the game. */
+  SGFTree   game_record;	/* Game record in sgf format. */
 
-  int       seed;		/* random seed */
   int       computer_player;	/* BLACK, WHITE, or EMPTY (used as BOTH) */
 
   char      outfilename[128];	/* Trickle file */
@@ -181,6 +172,8 @@ void gameinfo_clear(Gameinfo *ginfo, int boardsize, float komi);
 void gameinfo_print(Gameinfo *ginfo);
 void gameinfo_load_sgfheader(Gameinfo *gameinfo, SGFNode *head);
 void gameinfo_play_move(Gameinfo *ginfo, int i, int j, int color);
+int  gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFNode *head,
+			       const char *untilstr, int orientation);
 int  gameinfo_play_sgftree(Gameinfo *gameinfo, SGFNode *head,
 			   const char *untilstr);
 
@@ -189,6 +182,8 @@ int  gameinfo_play_sgftree(Gameinfo *gameinfo, SGFNode *head,
 /*                           global variables                       */
 /* ================================================================ */
 
+/* Random seed */
+extern int random_seed;
 
 /* Miscellaneous debug options. */
 extern int quiet;		/* Minimal output. */
@@ -198,34 +193,33 @@ extern int printworms;		/* print full data on each string */
 extern int printmoyo;		/* print moyo board each move */
 extern int printdragons;	/* print full data on each dragon */
 extern int printboard;		/* print board each move */
-extern int showstack;		/* debug stack pointer */
 extern int showstatistics;	/* print statistics */
 extern int profile_patterns;	/* print statistics of pattern usage */
 
-extern int last_moves_i[2];
-extern int last_moves_j[2];
-
 /* debug flag bits */
 /* NOTE : can specify -d0x... */
-#define DEBUG_INFLUENCE           0x00001
-#define DEBUG_EYES                0x00002
-#define DEBUG_OWL                 0x00004
-#define DEBUG_ESCAPE              0x00008
-#define DEBUG_MATCHER             0x00010
-#define DEBUG_DRAGONS             0x00020
-#define DEBUG_SEMEAI              0x00040
-#define DEBUG_LOADSGF             0x00080
-#define DEBUG_HELPER              0x00100
-#define DEBUG_READING             0x00200
-#define DEBUG_WORMS               0x00400 /* unused */
-#define DEBUG_MOVE_REASONS        0x00800
-#define DEBUG_OWL_PERFORMANCE     0x01000
-#define DEBUG_LIFE                0x02000
-#define DEBUG_FILLLIB             0x04000
-#define DEBUG_READING_PERFORMANCE 0x08000
-#define DEBUG_SCORING             0x10000
-#define DEBUG_AFTERMATH           0x20000
-#define DEBUG_ATARI_ATARI         0x40000
+#define DEBUG_INFLUENCE             0x0001
+#define DEBUG_EYES                  0x0002
+#define DEBUG_OWL                   0x0004
+#define DEBUG_ESCAPE                0x0008
+#define DEBUG_MATCHER               0x0010
+#define DEBUG_DRAGONS               0x0020
+#define DEBUG_SEMEAI                0x0040
+#define DEBUG_LOADSGF               0x0080
+#define DEBUG_HELPER                0x0100
+#define DEBUG_READING               0x0200
+#define DEBUG_WORMS                 0x0400
+#define DEBUG_MOVE_REASONS          0x0800
+#define DEBUG_OWL_PERFORMANCE       0x1000
+#define DEBUG_LIFE                  0x2000
+#define DEBUG_FILLLIB               0x4000
+#define DEBUG_READING_PERFORMANCE   0x8000
+#define DEBUG_SCORING               0x010000
+#define DEBUG_AFTERMATH             0x020000
+#define DEBUG_ATARI_ATARI           0x040000
+#define DEBUG_READING_CACHE         0x080000
+#define DEBUG_TERRITORY             0x100000
+#define DEBUG_OWL_PERSISTENT_CACHE  0x200000
 /* hash flag bits 
  *
  * Regarding HASH_DEFAULT:
@@ -243,18 +237,24 @@ extern int last_moves_j[2];
 #define HASH_ATTACK3      0x0080
 #define HASH_OWL_ATTACK   0x0100
 #define HASH_OWL_DEFEND   0x0200
+#define HASH_SEMEAI       0x0400
+#define HASH_CONNECT      0x0800
+#define HASH_DISCONNECT   0x1000
 #define HASH_NOTHING      0
 #define HASH_ALL          0xffff
 #define HASH_DEFAULT      (HASH_ATTACK | HASH_FIND_DEFENSE\
-			   | HASH_OWL_ATTACK | HASH_OWL_DEFEND)
+			   | HASH_OWL_ATTACK | HASH_OWL_DEFEND | HASH_SEMEAI\
+                           | HASH_CONNECT | HASH_DISCONNECT)
 
 extern int debug;		/* debug flags */
 extern int hashflags;		/* hash flags */
 extern int life;                /* use "life" reading code */
 extern int life_eyesize;        /* max eye size for life code */
 extern int fusekidb;            /* use fuseki database */
+extern int disable_fuseki;      /* do not generate fuseki moves */
 extern int josekidb;            /* use joseki database */
 extern int level;		/* controls depth of reading */
+extern int semeai_variations;   /* max variations considered reading semeai */
 extern int urgent;              /* urgent move on board */
 extern int showtime;		/* print genmove time */
 extern int showscore;		/* print score */
@@ -262,9 +262,19 @@ extern float score;
 extern float lower_bound;
 extern float upper_bound;
 extern int chinese_rules;       /* use chinese (area) rules for counting */
-extern int allow_suicide;       /* allow opponent to make suicide moves */
-extern int capture_all_dead;    /* capture all dead opponent stones */
-extern int play_out_aftermath;  /* make everything unconditionally settled */
+extern int experimental_semeai;      /* use experimental semeai module */
+extern int experimental_connections; /* use experimental connection module */
+extern int alternate_connections;    /* use alternate connection module */
+extern int owl_threats;              /* compute owl threats */
+extern int experimental_influence;   /* use experimental influence module */
+extern int allow_suicide;            /* allow opponent to make suicide moves */
+extern int capture_all_dead;         /* capture all dead opponent stones */
+extern int play_out_aftermath; /* make everything unconditionally settled */
+
+#if EXPERIMENTAL_READING
+extern int defend_by_pattern;  /* use patterns for tactical reading defense */
+extern int attack_by_pattern;  /* use patterns for tactical reading attack */
+#endif
 
 /* Mandatory values of reading parameters. Normally -1, if set at 
  * these override the values derived from the level. */
@@ -276,17 +286,15 @@ extern int mandated_fourlib_depth;
 extern int mandated_ko_depth;
 extern int mandated_branch_depth;
 extern int mandated_aa_depth;
+extern int mandated_aa_threat_depth;
 extern int mandated_owl_distrust_depth;
 extern int mandated_owl_branch_depth;
 extern int mandated_owl_reading_depth;
 extern int mandated_owl_node_limit; 
 
+/* Keep this as 2D until we change the entire API. */
 extern float potential_moves[MAX_BOARD][MAX_BOARD];
 
-extern float best_move_values[10];
-extern int best_movei[10];
-extern int best_movej[10];
-
 extern volatile int time_to_die;   /* set by signal handlers */
 
 /* ================================================================ */
@@ -311,13 +319,16 @@ void end_draw_board(void);
 void showboard(int xo);  /* ascii rep. of board to stdout */
 
 /* printutils.c */
-void gprintf(const char *fmt, ...);
+int gprintf(const char *fmt, ...);
 void mprintf(const char *fmt, ...);
+void gfprintf(FILE *outfile, const char *fmt, ...);
 const char *color_to_string(int color);
-const char *location_to_string(int i, int j);
+const char *location_to_string(int pos);
+void        location_to_buffer(int pos, char *buf);
 const char *status_to_string(int status);
 const char *safety_to_string(int status);
 const char *result_to_string(int result);
+const char *routine_to_string(int routine);
 int string_to_location(int boardsize, char *str, int *m, int *n);
 double gg_gettimeofday(void);
 
@@ -329,23 +340,34 @@ void debug_influence_move(int i, int j);
 
 /* gnuc allows variadic macros, so the tests can be done inline */
 #define TRACE(fmt, args...) \
-    do { if (verbose) gprintf(fmt, ##args); } while(0)
+    do { if (verbose) gprintf(fmt, ##args); } while (0)
 #define RTRACE(fmt, args...) \
-    do { if (verbose >= 3) gprintf(fmt, ##args); } while(0)
+    do { if (verbose >= 3) gprintf(fmt, ##args); } while (0)
 #define VTRACE(fmt, args...) \
-    do { if (verbose >= 4) gprintf(fmt, ##args); } while(0)
+    do { if (verbose >= 4) gprintf(fmt, ##args); } while (0)
 #define DEBUG(level, fmt, args...) \
-    do { if ((debug & (level))) gprintf(fmt, ##args); } while(0)
+    do { if ((debug & (level))) gprintf(fmt, ##args); } while (0)
 
-#else
+#else /*HAVE_VARIADIC_DEFINE*/
 
-/* else we need fns to do these */
-void TRACE(const char *fmt, ...);
-void RTRACE(const char *fmt, ...);
-void VTRACE(const char *fmt, ...);
-void DEBUG(int level, const char *fmt, ...);
+/* Using these without assignment causes compiler warnings on some systems. */
+extern int trace_dummy;
+# ifdef HAVE_VISUAL_C
+#  define TRACE_ASSIGN 
+# else
+#  define TRACE_ASSIGN trace_dummy =
+# endif
+
+#define TRACE  TRACE_ASSIGN (verbose) && gprintf
+#define RTRACE TRACE_ASSIGN (verbose >= 3) && gprintf
+#define VTRACE TRACE_ASSIGN (verbose >= 4) && gprintf
+/* if debug == 0, then can skip the function call. */
+#define DEBUG  TRACE_ASSIGN (debug) && DEBUG_func
+
+int DEBUG_func(int level, const char *fmt, ...);
+
+#endif  /*HAVE_VARIADIC_DEFINE*/
 
-#endif
 
 /* genmove.c */
 #define EXAMINE_WORMS               1
@@ -359,6 +381,7 @@ void DEBUG(int level, const char *fmt, ...);
 
 void reset_engine(void);
 void examine_position(int color, int how_much);
+void silent_examine_position(int color, int how_much);
 
 
 /* ================================================================ */
@@ -368,13 +391,15 @@ void examine_position(int color, int how_much);
 
 /* These are mostly used for GTP examination. */
 void reset_life_node_counter(void);
-int  get_life_node_counter(void);
+int get_life_node_counter(void);
 void reset_owl_node_counter(void);
-int  get_owl_node_counter(void);
+int get_owl_node_counter(void);
 void reset_reading_node_counter(void);
-int  get_reading_node_counter(void);
+int get_reading_node_counter(void);
 void reset_trymove_counter(void);
-int  get_trymove_counter(void);
+int get_trymove_counter(void);
+void reset_connection_node_counter(void);
+int get_connection_node_counter(void);
 
 
 /* ================================================================ */
@@ -385,29 +410,32 @@ int  get_trymove_counter(void);
 /* board.c */
 /* General board handling. */
 void clear_board(void);
-void setup_board(Intersection new_p[MAX_BOARD][MAX_BOARD], int koi, int koj,
-		 int *last_i, int *last_j, 
-		 float new_komi, int w_captured, int b_captured);
+int test_gray_border(void);
+void setup_board(Intersection new_board[MAX_BOARD][MAX_BOARD], int ko_pos,
+		 int *last, float new_komi, int w_captured, int b_captured);
 
 
 /* Putting stones on the board.. */
-void add_stone(int i, int j, int color);
-void remove_stone(int i, int j);
-void remove_string(int i, int j);
-void play_move(int i, int j, int color);
-int is_legal(int i, int j, int color);
-int is_suicide(int i, int j, int color);
-int is_illegal_ko_capture(int i, int j, int color);
-int trymove(int i, int j, int color, const char *message, int k, int l, 
-	    int komaster, int kom_i, int kom_j);
-int tryko(int i, int j, int color, const char *message, 
-	  int komaster, int kom_i, int kom_j);
+void add_stone(int pos, int color);
+void remove_stone(int pos);
+void play_move(int pos, int color);
+int undo_move(int n);
+int get_last_move(void);
+int get_last_opponent_move(int color);
+int is_pass(int pos);
+int is_legal(int pos, int color);
+int is_suicide(int pos, int color);
+int is_illegal_ko_capture(int pos, int color);
+int trymove(int pos, int color, const char *message, int str, 
+	    int komaster, int kom_pos);
+int semeai_trymove(int pos, int color, const char *message, 
+		   int str1, int str2, int owl_phase, int value);
+int tryko(int pos, int color, const char *message, 
+	  int komaster, int kom_pos);
 void popgo(void);
 
 /* utils.c */
-void change_matcher_status(int x, int y, int status);
-int same_dragon(int ai, int aj, int bi, int bj);
-int is_worm_origin(int wi, int wj, int i, int j);
+void change_matcher_status(int dr, int status);
 void who_wins(int color, FILE* outfile);
 
 /* high-level routine to generate the best move for the given color */
@@ -415,19 +443,20 @@ int genmove(int *i, int *j, int color);
 int genmove_conservative(int *i, int *j, int color);
 
 /* Play through the aftermath. */
-float aftermath_compute_score(int color, float komi);
-int aftermath_final_status(int color, int m, int n);
+float aftermath_compute_score(int color, float komi, SGFTree *tree);
+int aftermath_final_status(int color, int pos);
 
 /* Basic information gathering. */
-/* worms.c */
-void build_worms(void);
-void make_worms(void);
-int is_same_worm(int ai, int aj, int bi, int bj);
+/* worm.c */
+void make_worms(int save_verbose);
 
-void make_dragons(int color, int stop_before_owl);
+/* dragon.c */
+void make_dragons(int color, int stop_before_owl, int save_verbose);
+void initialize_dragon_data(void);
 void show_dragons(void);
-int dragon_status(int i, int j);
-int matcher_status(int i, int j);
+int dragon_status(int pos);
+int matcher_status(int pos);
+int same_dragon(int dr1, int dr2);
 
 /* moyo functions */
 void print_moyo(void);
@@ -440,18 +469,23 @@ void report_pattern_profiling(void);
 void sgffile_move_made(int i, int j, int color, int value);
 void sgffile_put_stone(int i, int j, int color);
 
-int  sgffile_open_file(const char *);
-int  sgffile_close_file(void);
-int  sgffile_write_gameinfo(Gameinfo *gameinfo, const char *);
+int sgffile_open_file(const char *);
+int sgffile_close_file(void);
+int sgffile_write_gameinfo(Gameinfo *gameinfo, const char *);
 void sgffile_write_comment(const char *comment);
+void sgffile_write_circle_mark(int i, int j);
 void sgffile_printboard(int next);
 void sgffile_recordboard(SGFNode *node);
 
-void decidestring(int i, int j, const char *sgf_output);
-void decidedragon(int i, int j, const char *sgf_output);
-void decidesemeai(int ai, int aj, int bi, int bj, const char *sgf_output);
-void decideposition(int color, const char *sgf_output);
-void decideeye(int m, int n, const char *sgf_output);
+/* sgfdecide.c */
+void decide_string(int pos, const char *sgf_output);
+void decide_connection(int apos, int bpos, const char *sgf_output);
+void decide_dragon(int pos, const char *sgf_output);
+void decide_semeai(int apos, int bpos, const char *sgf_output);
+void decide_tactical_semeai(int apos, int bpos, const char *sgf_output);
+void decide_position(int color, const char *sgf_output);
+void decide_eye(int pos, const char *sgf_output);
+void decide_combination(int color, const char *sgf_output);
 
 
 #endif  /* _GNUGO_H_ */
