@@ -1,25 +1,23 @@
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
- * http://www.gnu.org/software/gnugo/ for more information.      *
- *                                                               *
- * Copyright 1999, 2000, 2001 by the Free Software Foundation.   *
- *                                                               *
- * This program is free software; you can redistribute it and/or *
- * modify it under the terms of the GNU General Public License   *
- * as published by the Free Software Foundation - version 2.     *
- *                                                               *
- * This program is distributed in the hope that it will be       *
- * useful, but WITHOUT ANY WARRANTY; without even the implied    *
- * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
- * PURPOSE.  See the GNU General Public License in file COPYING  *
- * for more details.                                             *
- *                                                               *
- * You should have received a copy of the GNU General Public     *
- * License along with this program; if not, write to the Free    *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
- * Boston, MA 02111, USA.                                        *
-\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
+ * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * http://www.gnu.org/software/gnugo/ for more information.          *
+ *                                                                   *
+ * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ *                                                                   *
+ * This program is free software; you can redistribute it and/or     *
+ * modify it under the terms of the GNU General Public License as    *
+ * published by the Free Software Foundation - version 2             *
+ *                                                                   *
+ * This program is distributed in the hope that it will be useful,   *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
+ * GNU General Public License in file COPYING for more details.      *
+ *                                                                   *
+ * You should have received a copy of the GNU General Public         *
+ * License along with this program; if not, write to the Free        *
+ * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Boston, MA 02111, USA.                                            *
+\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 /*
  * This file is about life & death.
@@ -67,6 +65,7 @@
  *   will have to use some scheme like this.
  */
 
+#include "gnugo.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -80,38 +79,36 @@
 #define MAX_BOUNDARY_STRINGS 20
 
 /* List of eye point coordinates. */
-static int eyei[MAX_EYE_SIZE+1];
-static int eyej[MAX_EYE_SIZE+1];
+static int eye[MAX_EYE_SIZE+1];
 
 /* Array with inverse mapping. The index MAX_EYE_SIZE is by convention
  * used to encode a pass move.
  */
-static int eyeindex[MAX_BOARD][MAX_BOARD];
+static int eyeindex[BOARDMAX];
 
 /* Proper eye points, i.e. not margins or external diagonals. */
-static int proper_eye[MAX_BOARD][MAX_BOARD];
+static int proper_eye[BOARDMAX];
 
 /* Eye point restrictions. */
-static int eye_restrictions[MAX_BOARD][MAX_BOARD];
+static int eye_restrictions[BOARDMAX];
 #define DEFENDER_NOT_PLAY         0x01
 #define ATTACKER_PLAY_SAFE        0x02
 #define DEFENDER_PLAY_IF_CAPTURE  0x04
 
 /* List of boundary strings. */
-static int boundaryi[MAX_BOUNDARY_STRINGS];
-static int boundaryj[MAX_BOUNDARY_STRINGS];
+static int boundary[MAX_BOUNDARY_STRINGS];
 
 static int eyesize;
 static int eye_color;
 static int boundary_size;
 
 
-static int minimize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
-			 struct half_eye_data heye[MAX_BOARD][MAX_BOARD],
+static int minimize_eyes(struct eye_data eyedata[BOARDMAX],
+			 struct half_eye_data heye[BOARDMAX],
 			 int *min, int *ko_out, int ko_in, int ko_master,
 			 int *attack_point, int cutoff_eyes, int cutoff_ko);
-static int maximize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
-			 struct half_eye_data heye[MAX_BOARD][MAX_BOARD],
+static int maximize_eyes(struct eye_data eyedata[BOARDMAX],
+			 struct half_eye_data heye[BOARDMAX],
 			 int *max, int *ko_out, int ko_in, int ko_master,
 			 int *defense_point, int cutoff_eyes, int cutoff_ko);
 static void life_showboard(void);
@@ -231,7 +228,7 @@ compute_hashvalue(void)
   int k;
   unsigned int hash = 0;
   for (k=0; k<eyesize; k++)
-    hash |= p[eyei[k]][eyej[k]] << 2*k;
+    hash |= board[eye[k]] << 2*k;
 
   return hash;
 }
@@ -249,8 +246,8 @@ get_eyehash_node(int attack)
   int invalid_bit = (attack ? ATTACK_INVALID_BIT : DEFENSE_INVALID_BIT);
 
   /* Add the ko point to the hash value. */
-  if (board_ko_i != -1 && eyeindex[board_ko_i][board_ko_j] >= 0)
-    hash |= 3 << (2 * eyeindex[board_ko_i][board_ko_j]);
+  if (board_ko_pos != NO_MOVE && eyeindex[board_ko_pos] >= 0)
+    hash |= 3 << (2 * eyeindex[board_ko_pos]);
   key = hash % EYEHASH_TABLE_SIZE;
 
   /* Search through the linked list for this entry. */
@@ -295,85 +292,100 @@ get_eyehash_node(int attack)
 /*                          Prepare the eyespace                    */
 /* ================================================================ */
 
+
 static void
 clear_eyepoints(void)
 {
   int i, j;
+  int ii;
+
   eyesize = 0;
   for (i = 0; i < board_size; ++i)
     for (j = 0; j < board_size; ++j) {
-      eyeindex[i][j] = -1;
-      proper_eye[i][j] = 0;
-      eye_restrictions[i][j] = 0;
+      ii = POS(i, j);
+
+      eyeindex[ii] = -1;
+      proper_eye[ii] = 0;
+      eye_restrictions[ii] = 0;
     }
 }
 
 static void
-include_eyepoint(int i, int j, int proper, int restrictions)
+include_eyepoint(int pos, int proper, int restrictions)
 {
-  if (eyeindex[i][j] == -1) {
-    eyeindex[i][j] = eyesize;
-    eyei[eyesize] = i;
-    eyej[eyesize] = j;
+  if (eyeindex[pos] == -1) {
+    /* FIXME: This is only a temporary workaround to avoid a crash in
+     * the assertion below.
+     */
+    if (eyesize > MAX_EYE_SIZE)
+      return;
+    /* FIXME:
+     * Should this be < MAX_EYE_SIZE?
+     * The list of points in eye[] is ended with (NO_MOVE)
+     */
+    ASSERT1(eyesize <= MAX_EYE_SIZE, pos);
+    eyeindex[pos] = eyesize;
+    eye[eyesize] = pos;
     eyesize++;
   }
   if (proper)
-    proper_eye[i][j] = 1;
+    proper_eye[pos] = 1;
 
-  eye_restrictions[i][j] |= restrictions;
+  eye_restrictions[pos] |= restrictions;
 }
 
+
 /* Explanations can be found above the calls in prepare_eyespace(). */
 static void
-check_vulnerability(int i, int j, int m, int n)
+check_vulnerability(int marginal_eyepoint, int stone)
 {
-  if (!ON_BOARD(m, n))
+  if (!ON_BOARD(stone))
     return;
   
-  if (p[m][n] == EMPTY) {
-    if (eyeindex[m][n] == -1)
-      include_eyepoint(m, n, 0, DEFENDER_PLAY_IF_CAPTURE);
+  if (board[stone] == EMPTY) {
+    if (eyeindex[stone] == NO_MOVE)
+      include_eyepoint(stone, 0, DEFENDER_PLAY_IF_CAPTURE);
     return;
   }
   
   /* Both eyespace and boundary are excluded. */
-  if (eyeindex[m][n] != -1)
+  if (eyeindex[stone] != -1)
     return;
 
   /* Play the opposite color of (m, n) on the margin and try to
-   * capture (m, n)
+   * capture (stone)
    */
-  if (!trymove(i, j, OTHER_COLOR(p[m][n]), "check_vulnerability", m, n,
-	       EMPTY, -1, -1))
+  if (!trymove(marginal_eyepoint, OTHER_COLOR(board[stone]),
+	       "check_vulnerability", stone, EMPTY, NO_MOVE))
     return;
 
-  if (p[m][n] && attack(m, n, NULL, NULL)) {
-    int libs;
-    int libi[2], libj[2];
+  if (board[stone] && attack(stone, NULL)) {
+    int liberties;
+    int libs[2];
     /* Vulnerability found. First pick up its liberties. */
-    libs = findlib(m, n, 2, libi, libj);
+    liberties = findlib(stone, 2, libs);
     
-    if (p[m][n] == eye_color && libs == 1) {
+    if (board[stone] == eye_color && liberties == 1) {
       /* Strategy (c). */
-      include_eyepoint(m, n, 0, 0);
-      include_eyepoint(libi[0], libj[0], 0, DEFENDER_NOT_PLAY);
+      include_eyepoint(stone, 0, 0);
+      include_eyepoint(libs[0], 0, DEFENDER_NOT_PLAY);
     }
-    else if (p[m][n] == OTHER_COLOR(eye_color)) {
-      if (libs > 1
-	  || countstones(m, n) > 6
-	  || eyesize + countstones(m, n) + 1 > MAX_EYE_SIZE) {
+    else if (board[stone] == OTHER_COLOR(eye_color)) {
+      if (liberties > 1
+	  || countstones(stone) > 6
+	  || eyesize + countstones(stone) + 1 > MAX_EYE_SIZE) {
 	/* Strategy (b). */
-	include_eyepoint(i, j, 0, ATTACKER_PLAY_SAFE);
+	include_eyepoint(marginal_eyepoint, 0, ATTACKER_PLAY_SAFE);
       }
       else {
 	/* Strategy (a) */
 	int k;
-	int stonei[6], stonej[6];
-	int size = findstones(m, n, 6, stonei, stonej);
+	int stones[6];
+	int size = findstones(stone, 6, stones);
 	gg_assert(size <= 6);
 	for (k=0; k<size; k++)
-	  include_eyepoint(stonei[k], stonej[k], 0, 0);
-	include_eyepoint(libi[0], libj[0], 0, DEFENDER_PLAY_IF_CAPTURE);
+	  include_eyepoint(stones[k], 0, 0);
+	include_eyepoint(libs[0], 0, DEFENDER_PLAY_IF_CAPTURE);
       }
     }
   }
@@ -382,10 +394,11 @@ check_vulnerability(int i, int j, int m, int n)
 
 
 static void
-print_eyespace(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
-	       struct half_eye_data heye[MAX_BOARD][MAX_BOARD])
+print_eyespace(struct eye_data eyedata[BOARDMAX],
+	       struct half_eye_data heye[BOARDMAX])
 {
   int m, n;
+  int ii;
   int k;
   int mini, maxi;
   int minj, maxj;
@@ -396,8 +409,8 @@ print_eyespace(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
   minj = board_size;
   maxj = -1;
   for (k = 0; k < eyesize; k++) {
-    m = eyei[k];
-    n = eyej[k];
+    m = I(eye[k]);
+    n = J(eye[k]);
     if (m < mini) mini = m;
     if (m > maxi) maxi = m;
     if (n < minj) minj = n;
@@ -413,19 +426,23 @@ print_eyespace(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
   for (m = mini; m <= maxi; m++) {
     gprintf(""); /* Get the indentation right. */
     for (n = minj; n <= maxj; n++) {
-      if (eyeindex[m][n] >= 0) {
-	if (p[m][n] == EMPTY) {
-	  if (eyedata[m][n].marginal)
+      ii = POS(m, n);
+
+      if (eyeindex[ii] >= 0) {
+	if (board[ii] == EMPTY) {
+	  if (eyedata[ii].marginal)
 	    gprintf("%o!");
-	  else if (halfeye(heye, m, n))
+	  else if (is_halfeye(heye, ii))
 	    gprintf("%oh");
 	  else
 	    gprintf("%o.");
-	} else if (halfeye(heye, m, n))
+	}
+	else if (is_halfeye(heye, ii))
 	  gprintf("%oH");
 	else
 	  gprintf("%oX");
-      } else
+      }
+      else
 	gprintf("%o ");
     }
     gprintf("\n");
@@ -434,14 +451,15 @@ print_eyespace(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
 
 
 static int
-prepare_eyespace(int m, int n, struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
-		 struct half_eye_data heye[MAX_BOARD][MAX_BOARD])
+prepare_eyespace(int pos, struct eye_data eyedata[BOARDMAX],
+		 struct half_eye_data heye[BOARDMAX])
 {
   int i, j;
+  int ii;
   int k;
   
   /* Set `eye_color' to the owner of the eye. */
-  eye_color = eyedata[m][n].color;
+  eye_color = eyedata[pos].color;
   if (eye_color == BLACK_BORDER)
     eye_color = BLACK;
   if (eye_color == WHITE_BORDER)
@@ -455,13 +473,15 @@ prepare_eyespace(int m, int n, struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
    */
   for (i = 0; i < board_size; ++i)
     for (j = 0; j < board_size; ++j) {
-      if (eyedata[i][j].origini == m && eyedata[i][j].originj == n) {
-	include_eyepoint(i, j, eyedata[i][j].marginal == 0, 0);
-	if (halfeye(heye, i, j)) {
-	  for (k=0; k<heye[i][j].num_attacks; k++)
-	    include_eyepoint(heye[i][j].ai[k], heye[i][j].aj[k], 0, 0);
-	  for (k=0; k<heye[i][j].num_defends; k++)
-	    include_eyepoint(heye[i][j].di[k], heye[i][j].dj[k], 0, 0);
+      ii = POS(i, j);
+
+      if (eyedata[ii].origin == pos) {
+	include_eyepoint(ii, eyedata[ii].marginal == 0, 0);
+	if (is_halfeye(heye, ii)) {
+	  for (k=0; k<heye[ii].num_attacks; k++)
+	    include_eyepoint(heye[ii].attack_point[k], 0, 0);
+	  for (k=0; k<heye[ii].num_defends; k++)
+	    include_eyepoint(heye[ii].defense_point[k], 0, 0);
 	}
       }
     }
@@ -481,20 +501,20 @@ prepare_eyespace(int m, int n, struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
   boundary_size = 0;
   for (i = 0; i < board_size; ++i)
     for (j = 0; j < board_size; ++j) {
-      if (p[i][j] != eye_color)
+      ii = POS(i, j);
+
+      if (board[ii] != eye_color)
 	continue;
-      if (eyeindex[i][j] >= 0)
+      if (eyeindex[ii] >= 0)
 	continue;
-      if ((   i > 0	       && proper_eye[i-1][j])
-	  || (i < board_size-1 && proper_eye[i+1][j])
-	  || (j > 0	       && proper_eye[i][j-1])
-	  || (j < board_size-1 && proper_eye[i][j+1])) {
-	int oi, oj;
-	find_origin(i, j, &oi, &oj);
-	if (eyeindex[oi][oj] != -2) {
-	  eyeindex[oi][oj] = -2;
-	  boundaryi[boundary_size] = oi;
-	  boundaryj[boundary_size] = oj;
+      if ((   i > 0	       && proper_eye[ii-NS])
+	  || (i < board_size-1 && proper_eye[ii+NS])
+	  || (j > 0	       && proper_eye[ii-1])
+	  || (j < board_size-1 && proper_eye[ii+1])) {
+	int origin = find_origin(ii);
+	if (eyeindex[origin] != -2) {
+	  eyeindex[origin] = -2;
+	  boundary[boundary_size] = origin;
 	  boundary_size++;
 	  gg_assert(boundary_size <= MAX_BOUNDARY_STRINGS);
 	}
@@ -534,17 +554,16 @@ prepare_eyespace(int m, int n, struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
    */
 
   for (k=0; k<eyesize; k++) {
-    i = eyei[k];
-    j = eyej[k];
-    if (!eyedata[i][j].marginal)
+    ii = eye[k];
+    if (!eyedata[ii].marginal)
       continue;
     /* Found a margin. Now look for a vulnerable stone outside the
      * eyespace.
      */
-    check_vulnerability(i, j, i-1, j);
-    check_vulnerability(i, j, i+1, j);
-    check_vulnerability(i, j, i, j-1);
-    check_vulnerability(i, j, i, j+1);
+    check_vulnerability(ii, ii-NS);
+    check_vulnerability(ii, ii+NS);
+    check_vulnerability(ii, ii-1);
+    check_vulnerability(ii, ii+1);
   }
 
   /* Verify that the effective eyespace still isn't too large. */
@@ -555,8 +574,7 @@ prepare_eyespace(int m, int n, struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
   eyehash_clear();
 
   /* Add a pass move at the end of the eyespace list. */
-  eyei[MAX_EYE_SIZE] = -1;
-  eyej[MAX_EYE_SIZE] = -1;
+  eye[MAX_EYE_SIZE] = NO_MOVE;
 
   if (debug & DEBUG_EYES) {
     gprintf("Prepared eyespace:\n");
@@ -585,11 +603,10 @@ prepare_eyespace(int m, int n, struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
  */
 
 int
-recognize_eye2(int m, int  n, 
-	       int *attacki, int *attackj, int *defendi, int *defendj,
+recognize_eye2(int pos, int *attack_point, int *defense_point,
 	       int *max, int *min, 
-	       struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
-	       struct half_eye_data heye[MAX_BOARD][MAX_BOARD],
+	       struct eye_data eyedata[BOARDMAX],
+	       struct half_eye_data heye[BOARDMAX],
  	       int add_moves, int color)
 {
   int result1a, result1b, result2a, result2b;
@@ -598,6 +615,8 @@ recognize_eye2(int m, int  n,
   int attack_point1a, attack_point1b;
   int defense_point2a, defense_point2b;
   int save_stackp;
+  int m = I(pos);
+  int n = J(pos);
 
   /* FIXME: Remove the unused parameter entirely if it's not needed. */
   UNUSED(color);
@@ -609,7 +628,7 @@ recognize_eye2(int m, int  n,
   stackp_when_called = stackp;
   
   /* Set up all data structures necessary to analyze the eyespace. */
-  if (!prepare_eyespace(m, n, eyedata, heye))
+  if (!prepare_eyespace(POS(m, n), eyedata, heye))
     return 0;
 
   DEBUG(DEBUG_EYES, "================  MINIMIZE 1 ================\n");
@@ -645,19 +664,17 @@ recognize_eye2(int m, int  n,
   if (!result2b)
     return 0;
 
-  DEBUG(DEBUG_EYES, "Min: ko_master %C, eyes=%d, ko=%d, attack: %m\n",
-	eye_color, min1a, ko1a,	eyei[attack_point1a], eyej[attack_point1a]);
+  DEBUG(DEBUG_EYES, "Min: ko_master %C, eyes=%d, ko=%d, attack: %1m\n",
+	eye_color, min1a, ko1a,	eye[attack_point1a]);
   
-  DEBUG(DEBUG_EYES, "Min: ko_master %C, eyes=%d, ko=%d, attack: %m\n",
-	OTHER_COLOR(eye_color), min1b, ko1b,
-	eyei[attack_point1b], eyej[attack_point1b]);
+  DEBUG(DEBUG_EYES, "Min: ko_master %C, eyes=%d, ko=%d, attack: %1m\n",
+	OTHER_COLOR(eye_color), min1b, ko1b, eye[attack_point1b]);
   
-  DEBUG(DEBUG_EYES, "Max: ko_master %C, eyes=%d, ko=%d, defense: %m\n",
-	eye_color, max2a, ko2a,	eyei[defense_point2a], eyej[defense_point2a]);
+  DEBUG(DEBUG_EYES, "Max: ko_master %C, eyes=%d, ko=%d, defense: %1m\n",
+	eye_color, max2a, ko2a,	eye[defense_point2a]);
   
-  DEBUG(DEBUG_EYES, "Max: ko_master %C, eyes=%d, ko=%d, defense: %m\n",
-	OTHER_COLOR(eye_color), max2b, ko2b,
-	eyei[defense_point2b], eyej[defense_point2b]);
+  DEBUG(DEBUG_EYES, "Max: ko_master %C, eyes=%d, ko=%d, defense: %1m\n",
+	OTHER_COLOR(eye_color), max2b, ko2b, eye[defense_point2b]);
   
   *min = min1a;
   *max = max2b;
@@ -669,12 +686,12 @@ recognize_eye2(int m, int  n,
     *max = 2;
   
   if (*min != *max) {
-    if (attacki) *attacki = eyei[attack_point1a];
-    if (attackj) *attackj = eyej[attack_point1a];
-    if (defendi) *defendi = eyei[defense_point2b];
-    if (defendj) *defendj = eyej[defense_point2b];
-    DEBUG(DEBUG_EYES, "  vital point (attack):  %m\n", *attacki, *attackj);
-    DEBUG(DEBUG_EYES, "  vital point (defense): %m\n", *defendi, *defendj);
+    if (attack_point)
+      *attack_point = eye[attack_point1a];
+    if (defense_point)
+      *defense_point = eye[defense_point2b];
+    DEBUG(DEBUG_EYES, "  vital point (attack):  %1m\n", *attack_point);
+    DEBUG(DEBUG_EYES, "  vital point (defense): %1m\n", *defense_point);
   }
 
   /* FIXME: Currently the rest of the life and death analysis can't
@@ -694,12 +711,12 @@ recognize_eye2(int m, int  n,
  * want to check that an opponent move here would be suicide.
  */
 static int
-is_small_eye(int i, int j)
+is_small_eye(int pos)
 {
-  if (!((   i == 0            || p[i-1][j] == eye_color)
-	&& (i == board_size-1 || p[i+1][j] == eye_color)
-	&& (j == 0            || p[i][j-1] == eye_color)
-	&& (j == board_size-1 || p[i][j+1] == eye_color)))
+  if (!(   (!ON_BOARD(pos-NS) || board[pos-NS] == eye_color)
+	&& (!ON_BOARD(pos+NS) || board[pos+NS] == eye_color)
+	&& (!ON_BOARD(pos-1)  || board[pos-1] == eye_color)
+	&& (!ON_BOARD(pos+1)  || board[pos+1] == eye_color)))
     return 0;
   return 1;
 }
@@ -708,28 +725,28 @@ is_small_eye(int i, int j)
  * topologically false and not a marginal eye point.
  */
 static int
-is_true_eye(struct half_eye_data heye[MAX_BOARD][MAX_BOARD], int i, int j)
+is_true_eye(struct half_eye_data heye[BOARDMAX], int pos)
 {
   int other = OTHER_COLOR(eye_color);
   
   /* False eyes and other marginal eye points do not yield eyes. */
-  if (heye[i][j].type == FALSE_EYE)
+  if (heye[pos].type == FALSE_EYE)
     return 0;
   
   /* If this is a halfeye, check the vital diagonals. */
-  if (halfeye(heye, i, j)) {
+  if (is_halfeye(heye, pos)) {
     int good = 0;
     int bad = 0;
     int k;
     /* This requires attack points and defense points to be identical. */
-    for (k=0; k<heye[i][j].num_attacks; k++) {
-      int ai = heye[i][j].ai[k];
-      int aj = heye[i][j].aj[k];
-      if (p[ai][aj] == eye_color)
+    for (k=0; k<heye[pos].num_attacks; k++) {
+      int aa = heye[pos].attack_point[k];
+
+      if (board[aa] == eye_color)
 	good++;
-      else if (p[ai][aj] == other)
+      else if (board[aa] == other)
 	bad++;
-      else if (is_suicide(ai, aj, eye_color))
+      else if (is_suicide(aa, eye_color))
 	bad++;
       else
 	good++;
@@ -742,21 +759,22 @@ is_true_eye(struct half_eye_data heye[MAX_BOARD][MAX_BOARD], int i, int j)
   return 1;
 }
 
+
 /* Returns 1 if at least one string in the eyespace is captured when
- * color plays at (m, n).
+ * color plays at (pos).
  */
 static int
-life_does_capture_something(int m, int n, int color)
+life_does_capture_something(int pos, int color)
 {
   int other = OTHER_COLOR(color);
   int k;
+
   for (k=0; k<4; k++) {
-    int dm = deltai[k];
-    int dn = deltaj[k];
-    if (ON_BOARD(m+dm, n+dn)
-	&& p[m+dm][n+dn] == other
-	&& countlib(m+dm, n+dn) == 1
-	&& eyeindex[m+dm][n+dn] >= 0)
+    int d = delta[k];
+
+    if (board[pos+d] == other
+	&& countlib(pos+d) == 1
+	&& eyeindex[pos+d] >= 0)
       return 1;
   }
 
@@ -857,11 +875,11 @@ compare_max_eyes(int eyes1, int ko1, int eyes2, int ko2)
   SET_ATTACK(cache_entry, (ko), (eyes), (attack));\
   if (stackp - stackp_when_called < DEBUG_LIMIT)\
     DEBUG(DEBUG_LIFE,\
-	  "exiting minimize_eyes (%s) - result = %d, ko = %d, move %m (%H)\n",\
-	  (message), (eyes), (ko), eyei[(attack)], eyej[(attack)],\
+         "exiting minimize_eyes (%s) - result = %d, ko = %d, move %1m (%H)\n",\
+	  (message), (eyes), (ko), eye[attack],\
 	  hashdata.hashval);\
   return 1;\
-  } while(0)
+  } while (0)
 
 /*
  * We have a set of locations in eyepoints forming an eye shape.
@@ -874,13 +892,13 @@ compare_max_eyes(int eyes1, int ko1, int eyes2, int ko2)
  */
 
 static int
-minimize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
-	      struct half_eye_data heye[MAX_BOARD][MAX_BOARD],
+minimize_eyes(struct eye_data eyedata[BOARDMAX],
+	      struct half_eye_data heye[BOARDMAX],
 	      int *min, int *ko_out, int ko_in, int ko_master,
 	      int *attack_point, int cutoff_eyes, int cutoff_ko)
 {
   int other = OTHER_COLOR(eye_color);
-  int i, j;
+  int ii;
   int num_other;
   int num_moves;
   int num_eyes;
@@ -930,9 +948,8 @@ minimize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
       *attack_point = attack_point1;
     
     if (stackp - stackp_when_called < DEBUG_LIMIT)
-      DEBUG(DEBUG_LIFE, "exiting minimize_eyes - got result %d eyes, %d ko (move %m) from the cache. (%H)\n",
-	    *min, *ko_out, eyei[attack_point1], eyej[attack_point1],
-	    hashdata.hashval);
+      DEBUG(DEBUG_LIFE, "exiting minimize_eyes - got result %d eyes, %d ko (move %1m) from the cache. (%H)\n",
+	    *min, *ko_out, eye[attack_point1], hashdata.hashval);
     return 1;
   }
 
@@ -943,9 +960,9 @@ minimize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
    * FIXME: This is too simplified.
    */
   for (k=0; k<boundary_size; k++) {
-    int libi[2], libj[2];
-    if (findlib(boundaryi[k], boundaryj[k], 2, libi, libj) == 1) {
-      int index = eyeindex[libi[0]][libj[0]];
+    int libs[2];
+    if (findlib(boundary[k], 2, libs) == 1) {
+      int index = eyeindex[libs[0]];
       /* If the move is outside the eyespace, return a pass.
        * FIXME: This is of course just a workaround.
        */
@@ -968,19 +985,18 @@ minimize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
   num_moves = 0;
   num_eyes = 0;
   for (k=0; k<eyesize; k++) {
-    int i = eyei[k];
-    int j = eyej[k];
-    
-    if (p[i][j] == eye_color)
+    int ii = eye[k];
+
+    if (board[ii] == eye_color)
       continue;
 
-    if (p[i][j] == other) {
-      if (proper_eye[i][j])
+    if (board[ii] == other) {
+      if (proper_eye[ii])
 	num_other++;
       continue;
     } 
 
-    if (!is_suicide(i, j, other)) {
+    if (!is_suicide(ii, other)) {
       score = 0;
       move[num_moves] = k;
       /* Score the move. We give (preliminarily)
@@ -997,15 +1013,15 @@ minimize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
        * captures at least one of them. This is likely to be good, so
        * we give a high score.
        */
-      if (i > 0            && p[i-1][j] != eye_color)
+      if (I(ii) > 0            && board[ii-NS] != eye_color)
 	score++;
-      if (i < board_size-1 && p[i+1][j] != eye_color)
+      if (I(ii) < board_size-1 && board[ii+NS] != eye_color)
 	score++;
-      if (j > 0            && p[i][j-1] != eye_color)
+      if (J(ii) > 0            && board[ii-1] != eye_color)
 	score++;
-      if (j < board_size-1 && p[i][j+1] != eye_color)
+      if (J(ii) < board_size-1 && board[ii+1] != eye_color)
 	score++;
-      if (eyedata[i][j].marginal)
+      if (eyedata[ii].marginal)
 	score += 2;
       if (score == 0)
 	score = 5;
@@ -1013,7 +1029,7 @@ minimize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
       num_moves++;
     }
     else {
-      if (is_small_eye(i, j) && is_true_eye(heye, i,j))
+      if (is_small_eye(ii) && is_true_eye(heye, ii))
 	num_eyes++;
     }
   }
@@ -1060,22 +1076,21 @@ minimize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
       if (move_score[k] != score)
 	continue;
 
-      i = eyei[index];
-      j = eyej[index];
+      ii = eye[index];
 
-      gg_assert(i == -1 || p[i][j] == EMPTY);
+      gg_assert(ii == NO_MOVE || board[ii] == EMPTY);
 
       /* Try the move and see if we can reduce the eyes. */
       save_stackp = stackp;
       if (stackp - stackp_when_called < DEBUG_LIMIT)
-	DEBUG(DEBUG_LIFE, "minimize_eyes: trymove %s %m score %d\n",
-	      color_to_string(other), i, j, score);
+	DEBUG(DEBUG_LIFE, "minimize_eyes: trymove %s %1m score %d\n",
+	      color_to_string(other), ii, score);
       if (index == MAX_EYE_SIZE
-	  || trymove(i, j, other, "minimize_eyes", -1, -1, EMPTY, -1, -1)
+	  || trymove(ii, other, "minimize_eyes", NO_MOVE, EMPTY, NO_MOVE)
 	  || (ko_master == other
 	      && (ko_in < 3)
 	      && (is_ko = 1)   /* Intentional assignment. */
-	      && tryko(i, j, other, "minimize_eyes", EMPTY, -1, -1))) {
+	      && tryko(ii, other, "minimize_eyes", EMPTY, NO_MOVE))) {
 
 	/* The attacker has made his move. Now let's answer him and
 	 * see how many eyes we can get.
@@ -1083,8 +1098,8 @@ minimize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
 	  
 	/* But first we must check a restriction. */
 	if (index != MAX_EYE_SIZE
-	    && (eye_restrictions[i][j] & ATTACKER_PLAY_SAFE)
-	    && attack(i, j, NULL, NULL))
+	    && (eye_restrictions[ii] & ATTACKER_PLAY_SAFE)
+	    && attack(ii, NULL))
 	  result = 0;
 	else
 	  result = maximize_eyes(eyedata, heye, &max1, &ko1, ko_in + is_ko,
@@ -1110,8 +1125,8 @@ minimize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
 	/* Illegal ko capture. */
 	if (stackp - stackp_when_called < DEBUG_LIMIT)
 	  DEBUG(DEBUG_LIFE,
-		"  illegal ko capture: %s %m ko master %s ko level %d\n",
-		color_to_string(eye_color), i, j, color_to_string(ko_master),
+		"  illegal ko capture: %s %1m ko master %s ko level %d\n",
+		color_to_string(eye_color), ii, color_to_string(ko_master),
 		ko_in);
       }
 
@@ -1154,7 +1169,7 @@ minimize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
     localmin = 3;
 
   MINIMIZE_EYES_RETURN(localmin, localko, localattack, "all moves tested");
-  }
+}
 
 
 /*
@@ -1168,13 +1183,13 @@ minimize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
  */
 
 static int
-maximize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
-	      struct half_eye_data heye[MAX_BOARD][MAX_BOARD],
+maximize_eyes(struct eye_data eyedata[BOARDMAX],
+	      struct half_eye_data heye[BOARDMAX],
 	      int *max, int *ko_out, int ko_in, int ko_master,
 	      int *defense_point, int cutoff_eyes, int cutoff_ko)
 {
   int other = OTHER_COLOR(eye_color);
-  int i, j;
+  int ii;
   int min1;
   int localmax;
   int localko;
@@ -1227,9 +1242,8 @@ maximize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
       *defense_point = defense_point1;
     
     if (stackp - stackp_when_called < DEBUG_LIMIT)
-      DEBUG(DEBUG_LIFE, "exiting maximize_eyes - got result %d eyes, %d ko (move %m) from the cache. (%H)\n",
-	    *max, *ko_out, eyei[defense_point1], eyej[defense_point1],
-	    hashdata.hashval);
+      DEBUG(DEBUG_LIFE, "exiting maximize_eyes - got result %d eyes, %d ko (move %1m) from the cache. (%H)\n",
+	    *max, *ko_out, eye[defense_point1], hashdata.hashval);
     return 1;
   }
 
@@ -1245,10 +1259,9 @@ maximize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
 
   /* Collect all possible moves and see what we get. */
   for (k=0; k<eyesize; k++) {
-    int i = eyei[k];
-    int j = eyej[k];
+    int ii = eye[k];
     
-    if (p[i][j] != EMPTY)
+    if (board[ii] != EMPTY)
       continue;
 
     /* If the eye is of size 1, of the eye owners color, and 
@@ -1261,22 +1274,22 @@ maximize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
      *
      * FIXME: Add diagonal test here.
      */
-    if (is_suicide(i, j, other) && is_small_eye(i, j)) {
-      if (is_true_eye(heye, i, j))
+    if (is_suicide(ii, other) && is_small_eye(ii)) {
+      if (is_true_eye(heye, ii))
 	num_eyes++;
       continue;
     }
 
     /* Check for own suicide. */
-    if (is_suicide(i, j, eye_color))
+    if (is_suicide(ii, eye_color))
       continue;
     
     /* Check certain move restrictions. */
-    if (eye_restrictions[i][j] & DEFENDER_NOT_PLAY)
+    if (eye_restrictions[ii] & DEFENDER_NOT_PLAY)
       continue;
 
-    if ((eye_restrictions[i][j] & DEFENDER_PLAY_IF_CAPTURE)
-	&& !life_does_capture_something(i, j, eye_color))
+    if ((eye_restrictions[ii] & DEFENDER_PLAY_IF_CAPTURE)
+	&& !life_does_capture_something(ii, eye_color))
       continue;
 
     /* Score the move and save it in a list for later testing.
@@ -1292,32 +1305,32 @@ maximize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
     move[num_moves] = k;
     score = 0;
     
-    if (i > 0) {
-      if (p[i-1][j] == other)
+    if (I(ii) > 0) {
+      if (board[ii-NS] == other)
 	score += 2;
-      else if (p[i-1][j] == EMPTY)
+      else if (board[ii-NS] == EMPTY)
 	score++;
     }
-    if (i < board_size-1) {
-      if (p[i+1][j] == other)
+    if (I(ii) < board_size-1) {
+      if (board[ii+NS] == other)
 	score += 2;
-      else if (p[i+1][j] == EMPTY)
+      else if (board[ii+NS] == EMPTY)
 	score++;
     }
-    if (j > 0) {
-      if (p[i][j-1] == other)
+    if (J(ii) > 0) {
+      if (board[ii-1] == other)
 	score += 2;
-      else if (p[i][j-1] == EMPTY)
+      else if (board[ii-1] == EMPTY)
 	score++;
     }
-    if (j < board_size-1) {
-      if (p[i][j+1] == other)
+    if (J(ii) < board_size-1) {
+      if (board[ii+1] == other)
 	score += 2;
-      else if (p[i][j+1] == EMPTY)
+      else if (board[ii+1] == EMPTY)
 	score++;
     }
     
-    if (eyedata[i][j].marginal)
+    if (eyedata[ii].marginal)
       score += 2;
     
     if (score == 0)
@@ -1341,21 +1354,20 @@ maximize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
       if (move_score[k] != score)
 	continue;
       
-      i = eyei[index];
-      j = eyej[index];
+      ii = eye[index];
 
-      gg_assert(p[i][j] == EMPTY);
+      gg_assert(board[ii] == EMPTY);
       
       /* Try the move and see if we can keep the eyes. */
       save_stackp = stackp;
       if (stackp - stackp_when_called < DEBUG_LIMIT)
-	DEBUG(DEBUG_LIFE, "maximize_eyes: trymove %s %m score %d\n",
-	      color_to_string(eye_color), i, j, score);
-      if (trymove(i, j, eye_color, "maximize_eyes", -1, -1, EMPTY, -1, -1)
+	DEBUG(DEBUG_LIFE, "maximize_eyes: trymove %s %1m score %d\n",
+	      color_to_string(eye_color), ii, score);
+      if (trymove(ii, eye_color, "maximize_eyes", NO_MOVE, EMPTY, NO_MOVE)
 	  || (ko_master == eye_color
 	      && (ko_in < 3)
 	      && (is_ko = 1)   /* Intentional assignment. */
-	      && tryko(i, j, eye_color, "maximize_eyes", EMPTY, -1, -1))) {
+	      && tryko(ii, eye_color, "maximize_eyes", EMPTY, NO_MOVE))) {
 	
 	/* Ok, we made our move.  Now let the opponent do his, and see
 	 * how many eyes we can get. 
@@ -1405,8 +1417,8 @@ maximize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
 	/* Illegal ko capture. */
 	if (stackp - stackp_when_called < DEBUG_LIMIT)
 	  DEBUG(DEBUG_LIFE,
-		"  illegal ko capture: %s %m ko master %s ko level %d\n",
-		color_to_string(eye_color), i, j, color_to_string(ko_master),
+		"  illegal ko capture: %s %1m ko master %s ko level %d\n",
+		color_to_string(eye_color), ii, color_to_string(ko_master),
 		ko_in);
       }
       gg_assert(stackp == save_stackp);
@@ -1433,9 +1445,8 @@ maximize_eyes(struct eye_data eyedata[MAX_BOARD][MAX_BOARD],
   
   if (stackp - stackp_when_called < DEBUG_LIMIT)
     DEBUG(DEBUG_LIFE,
-	  "exiting maximize_eyes (1) - result = %d, ko = %d, move %m (%H)\n", 
-	  localmax, localko, eyei[local_defense], eyej[local_defense],
-	  hashdata.hashval);
+	  "exiting maximize_eyes (1) - result = %d, ko = %d, move %1m (%H)\n", 
+	  localmax, localko, eye[local_defense], hashdata.hashval);
 
   return 1;
 }
@@ -1455,23 +1466,24 @@ life_showboard()
   for (i=0; i<board_size; i++) {
     for (j=0; j<board_size; j++) {
       int c;
-      int color = p[i][j];
+      int ii = POS(i, j);
+      int color = board[ii];
 
       if (color == WHITE) {
-	if (move_in_stack(i, j, stackp_when_called))
+	if (move_in_stack(ii, stackp_when_called))
 	  c = 'o';
 	else
 	  c = 'O';
       }
       else if (color == BLACK) {
-	if (move_in_stack(i, j, stackp_when_called))
+	if (move_in_stack(ii, stackp_when_called))
 	  c = 'x';
 	else
 	  c = 'X';
       }
       else
 	c = EMPTY;
-      draw_char(i, j, c);
+      draw_char(I(ii), J(ii), c);
     }
   }
   end_draw_board();
