@@ -20,23 +20,31 @@
 #endif
 
 #define  _GMP_C_  1
-#include <time.h>
+
 #include <assert.h>
-#include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
 #include <string.h>
 
+#if TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
 #ifdef __MINGW32__
 #include <windows.h>
-#include <windows32/sockets.h>
+#include <winsock.h>
+#include <io.h>
 #endif
 
 #ifdef HAVE_VISUAL_C
@@ -66,7 +74,7 @@ typedef enum  {
   query_game, query_bufSize, query_protocol, query_stones,
   query_bTime, query_wTime, query_charSet, query_rules, query_handicap,
   query_boardSize, query_timeLimit, query_color, query_who,
-  query_max,
+  query_max
 } Query;
 
 
