@@ -6,7 +6,7 @@
  * files gtp.c and gtp.h are licensed under less restrictive     *
  * terms than the rest of GNU Go.                                *
  *                                                               *
- * Copyright 2001 by the Free Software Foundation.               *
+ * Copyright 2001, 2002 by the Free Software Foundation.         *
  *                                                               *
  * Permission is hereby granted, free of charge, to any person   *
  * obtaining a copy of this file gtp.c, to deal in the Software  *
@@ -58,9 +58,13 @@
  */
 static int gtp_boardsize = -1;
 
-/* Read stdin linewise and interpret as GTP commands. */
+/* Vertex transformation hooks. */
+static gtp_transform_ptr vertex_transform_input_hook = NULL;
+static gtp_transform_ptr vertex_transform_output_hook = NULL;
+
+/* Read filehandle gtp_input linewise and interpret as GTP commands. */
 void
-gtp_main_loop(struct gtp_command commands[])
+gtp_main_loop(struct gtp_command commands[], FILE *gtp_input)
 {
   char line[GTP_BUFSIZE];
   char command[GTP_BUFSIZE];
@@ -71,8 +75,8 @@ gtp_main_loop(struct gtp_command commands[])
   int status = GTP_OK;
   
   while (status == GTP_OK) {
-    /* Read a line from stdin. */
-    if (!fgets(line, GTP_BUFSIZE, stdin))
+    /* Read a line from gtp_input. */
+    if (!fgets(line, GTP_BUFSIZE, gtp_input))
       break; /* EOF or some error */
 
     /* Remove comments. */
@@ -109,7 +113,6 @@ gtp_main_loop(struct gtp_command commands[])
   }
 }
 
-
 /* Set the board size used in coordinate conversions. */
 void
 gtp_internal_set_boardsize(int size)
@@ -117,6 +120,18 @@ gtp_internal_set_boardsize(int size)
   gtp_boardsize = size;
 }
 
+/* If you need to transform the coordinates on input or output, use
+ * this functions to set hook functions which are called any time
+ * coordinates are read or about to be written. In GNU Go this is used
+ * to simulate rotated boards in regression tests.
+ */
+void
+gtp_set_vertex_transform_hooks(gtp_transform_ptr in, gtp_transform_ptr out)
+{
+  vertex_transform_input_hook = in;
+  vertex_transform_output_hook = out;
+}
+
 /*
  * This function works like printf, except that it only understands
  * very few of the standard formats, to be precise %c, %d, %f, %s.
@@ -129,7 +144,7 @@ gtp_mprintf(const char *fmt, ...)
   va_list ap;
   va_start(ap, fmt);
   
-  for ( ; *fmt ; ++fmt ) {
+  for (; *fmt ; ++fmt) {
     if (*fmt == '%') {
       switch (*++fmt) {
       case 'c':
@@ -161,12 +176,7 @@ gtp_mprintf(const char *fmt, ...)
       {
 	int m = va_arg(ap, int);
 	int n = va_arg(ap, int);
-	if (m == -1 && n == -1)
-	  fputs("PASS", stdout);
-	else if ((m<0) || (n<0) || (m>=gtp_boardsize) || (n>=gtp_boardsize))
-	  fprintf(stdout, "??");
-	else
-	  fprintf(stdout, "%c%d", 'A' + n + (n >= 8), gtp_boardsize - m);
+        gtp_print_vertex(m, n);
 	break;
       }
       case 'C':
@@ -325,6 +335,9 @@ gtp_decode_coord(char *s, int *i, int *j)
   if (*i < 0 || *i >= gtp_boardsize || *j < 0 || *j >= gtp_boardsize)
     return 0;
 
+  if (vertex_transform_input_hook != NULL)
+    (*vertex_transform_input_hook)(*i, *j, i, j);
+
   return n;
 }
 
@@ -381,6 +394,7 @@ void
 gtp_print_vertices(int n, int movei[], int movej[])
 {
   int k;
+  int ri, rj;
   
   assert(gtp_boardsize > 0);
   
@@ -393,9 +407,15 @@ gtp_print_vertices(int n, int movei[], int movej[])
     else if (movei[k] < 0 || movei[k] >= gtp_boardsize
 	     || movej[k] < 0 || movej[k] >= gtp_boardsize)
       gtp_printf("??");
-    else
-      gtp_printf("%c%d", 'A' + movej[k] + (movej[k] >= 8),
-		 gtp_boardsize - movei[k]);
+    else {
+      if (vertex_transform_output_hook != NULL)
+	(*vertex_transform_output_hook)(movei[k], movej[k], &ri, &rj);
+      else {
+	ri = movei[k];
+	rj = movej[k];
+      }
+      gtp_printf("%c%d", 'A' + rj + (rj >= 8), gtp_boardsize - ri);
+    }
   }
 }
 
