@@ -1,28 +1,25 @@
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
- * http://www.gnu.org/software/gnugo/ for more information.      *
- *                                                               *
- * Copyright 1999, 2000, 2001 by the Free Software Foundation.   *
- *                                                               *
- * This program is free software; you can redistribute it and/or *
- * modify it under the terms of the GNU General Public License   *
- * as published by the Free Software Foundation - version 2.     *
- *                                                               *
- * This program is distributed in the hope that it will be       *
- * useful, but WITHOUT ANY WARRANTY; without even the implied    *
- * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
- * PURPOSE.  See the GNU General Public License in file COPYING  *
- * for more details.                                             *
- *                                                               *
- * You should have received a copy of the GNU General Public     *
- * License along with this program; if not, write to the Free    *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
- * Boston, MA 02111, USA.                                        *
-\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
+ * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * http://www.gnu.org/software/gnugo/ for more information.          *
+ *                                                                   *
+ * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ *                                                                   *
+ * This program is free software; you can redistribute it and/or     *
+ * modify it under the terms of the GNU General Public License as    *
+ * published by the Free Software Foundation - version 2             *
+ *                                                                   *
+ * This program is distributed in the hope that it will be useful,   *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
+ * GNU General Public License in file COPYING for more details.      *
+ *                                                                   *
+ * You should have received a copy of the GNU General Public         *
+ * License along with this program; if not, write to the Free        *
+ * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Boston, MA 02111, USA.                                            *
+\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "gnugo.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -30,7 +27,6 @@
 #include <assert.h>
 #include <ctype.h>
 
-#include "gnugo.h"
 #include "liberty.h"
 #include "interface.h"
 #include "sgftree.h"
@@ -61,11 +57,10 @@ static int last_move_j;      /* -""-                          */
 static int current_score_estimate = NO_SCORE;
 
 static void endgame(Gameinfo *gameinfo);
-static void nngs_endgame(Gameinfo *gameinfo);
-static void showcapture(Position *pos, char *line);
-static void showdefense(Position *pos, char *line);
+static void showcapture(char *line);
+static void showdefense(char *line);
 static void ascii_goto(Gameinfo *gameinfo, char *line);
-static int ascii2pos(Position *pos, char *line, int *i, int *j);
+static int ascii2pos(char *line, int *i, int *j);
 
 /* If sgf game info is written can't reset parameters like handicap, etc. */
 static int sgf_initialized = 0;
@@ -98,7 +93,7 @@ make_letterbar(int boardsize, char *letterbar)
 
 /* This array contains +'s and -'s for the empty board positions.
  * hspot_size contains the board size that the grid has been
- * initialised to.
+ * initialized to.
  */
 
 static int hspot_size;
@@ -194,108 +189,108 @@ set_handicap_spots(int boardsize)
  */
 
 static void
-ascii_showboard(Position *pos)
+ascii_showboard(void)
 {
-   int i, j;
-   char letterbar[64];
-   int last_pos_was_move;
-   int pos_is_move;
-   int dead;
+  int i, j;
+  char letterbar[64];
+  int last_pos_was_move;
+  int pos_is_move;
+  int dead;
+  
+  make_letterbar(board_size, letterbar);
+  set_handicap_spots(board_size);
+
+  printf("\n");
+  printf("    White has captured %d pieces\n", black_captured);
+  printf("    Black has captured %d pieces\n", white_captured);
+  if (showscore) {
+    if (current_score_estimate == NO_SCORE)
+      printf("    No score estimate is available yet.\n");
+    else if (current_score_estimate < 0)
+      printf("    Estimated score: Black is ahead by %d\n",
+	     -current_score_estimate);
+    else if (current_score_estimate > 0)
+      printf("    Estimated score: White is ahead by %d\n",
+	     current_score_estimate);
+    else
+      printf("    Estimated score: Even!\n");
+  }
    
-   make_letterbar(pos->boardsize, letterbar);
-   set_handicap_spots(pos->boardsize);
+  printf("\n");
 
-   printf("\n");
-   printf("    White has captured %d pieces\n", pos->black_captured);
-   printf("    Black has captured %d pieces\n", pos->white_captured);
-   if (showscore) {
-     if (current_score_estimate == NO_SCORE)
-       printf("    No score estimate is available yet.\n");
-     else if (current_score_estimate < 0)
-       printf("    Estimated score: Black is ahead by %d\n",
-	      -current_score_estimate);
-     else if (current_score_estimate > 0)
-       printf("    Estimated score: White is ahead by %d\n",
-	      current_score_estimate);
-     else
-       printf("    Estimated score: Even!\n");
-   }
-   
-   printf("\n");
-
-   fflush (stdout);
-   printf("%s%s\n", (emacs ? "EMACS1\n" : ""), letterbar);
-   fflush (stdout);
-
-   for (i = 0; i < pos->boardsize; i++) {
-     printf(" %2d", pos->boardsize-i);
-     last_pos_was_move = 0;
-     for (j = 0; j < pos->boardsize; j++) {
-       if (last_move_i == i && last_move_j == j)
-	 pos_is_move = 128;
-       else
-	 pos_is_move = 0;
-       dead = (matcher_status(i, j)==DEAD) && showdead;
-       switch (pos->board[i][j] + pos_is_move + last_pos_was_move) {
-       case EMPTY+128:
-       case EMPTY:
-	 printf(" %c", hspots[i][j]);
-	 last_pos_was_move = 0;
-	 break;
-       case BLACK:
-	 printf(" %c", dead ? 'x' : 'X');
-	 last_pos_was_move = 0;
-	 break;
-       case WHITE:
-	 printf(" %c", dead ? 'o' : 'O');
-	 last_pos_was_move = 0;
-	 break;
-       case BLACK+128:
-	 printf("(%c)", 'X');
-	 last_pos_was_move = 256;
-	 break;
-       case WHITE+128:
-	 printf("(%c)", 'O');
-	 last_pos_was_move = 256;
-	 break;
-       case EMPTY+256:
-	 printf("%c", hspots[i][j]);
-	 last_pos_was_move = 0;
-	 break;
-       case BLACK+256:
-	 printf("%c", dead ? 'x' : 'X');
-	 last_pos_was_move = 0;
-	 break;
-       case WHITE+256:
-	 printf("%c", dead ? 'o' : 'O');
-	 last_pos_was_move = 0;
-	 break;
-       default: 
-	 fprintf(stderr,"Illegal board value %d\n", pos->board[i][j]);
-	 exit(EXIT_FAILURE);
-	 break;
-       }
-     }
-
-     if (last_pos_was_move == 0) {
-       if (pos->boardsize > 10)
-	 printf(" %2d", pos->boardsize-i);
-       else
-	 printf(" %1d", pos->boardsize-i);
-     }
-     else {
-       if (pos->boardsize > 10)
-	 printf("%2d", pos->boardsize-i);
-       else
-         printf("%1d", pos->boardsize-i);
-     }
-     printf("\n");
-   }
-
-   fflush (stdout);
-   printf("%s\n\n", letterbar);
-   fflush (stdout);
-   
+  fflush(stdout);
+  printf("%s%s\n", (emacs ? "EMACS1\n" : ""), letterbar);
+  fflush(stdout);
+  
+  for (i = 0; i < board_size; i++) {
+    printf(" %2d", board_size - i);
+    last_pos_was_move = 0;
+    for (j = 0; j < board_size; j++) {
+      if (last_move_i == i && last_move_j == j)
+	pos_is_move = 128;
+      else
+	pos_is_move = 0;
+      dead = (matcher_status(POS(i, j))==DEAD) && showdead;
+      switch (BOARD(i, j) + pos_is_move + last_pos_was_move) {
+	case EMPTY+128:
+	case EMPTY:
+	  printf(" %c", hspots[i][j]);
+	  last_pos_was_move = 0;
+	  break;
+	case BLACK:
+	  printf(" %c", dead ? 'x' : 'X');
+	  last_pos_was_move = 0;
+	  break;
+	case WHITE:
+	  printf(" %c", dead ? 'o' : 'O');
+	  last_pos_was_move = 0;
+	  break;
+	case BLACK+128:
+	  printf("(%c)", 'X');
+	  last_pos_was_move = 256;
+	  break;
+	case WHITE+128:
+	  printf("(%c)", 'O');
+	  last_pos_was_move = 256;
+	  break;
+	case EMPTY+256:
+	  printf("%c", hspots[i][j]);
+	  last_pos_was_move = 0;
+	  break;
+	case BLACK+256:
+	  printf("%c", dead ? 'x' : 'X');
+	  last_pos_was_move = 0;
+	  break;
+	case WHITE+256:
+	  printf("%c", dead ? 'o' : 'O');
+	  last_pos_was_move = 0;
+	  break;
+	default: 
+	  fprintf(stderr, "Illegal board value %d\n", BOARD(i, j));
+	  exit(EXIT_FAILURE);
+	  break;
+      }
+    }
+    
+    if (last_pos_was_move == 0) {
+      if (board_size > 10)
+	printf(" %2d", board_size - i);
+      else
+	printf(" %1d", board_size - i);
+    }
+    else {
+      if (board_size > 10)
+	printf("%2d", board_size - i);
+      else
+	printf("%1d", board_size - i);
+    }
+    printf("\n");
+  }
+  
+  fflush(stdout);
+  printf("%s\n\n", letterbar);
+  fflush(stdout);
+  
 }  /* end ascii_showboard */
 
 /*
@@ -344,11 +339,11 @@ enum commands {INVALID=-1, END, EXIT, QUIT, RESIGN,
 	       SETHANDICAP, SETBOARDSIZE, SETKOMI,
 	       SETDEPTH,
                INFO, DISPLAY, SHOWBOARD, HELP, UNDO, COMMENT, SCORE,
-               CMD_DEAD,CMD_BACK,CMD_FORWARD,CMD_LAST,
-               CMD_CAPTURE,CMD_DEFEND,
-               CMD_HELPDEBUG,CMD_SHOWAREA,CMD_SHOWMOYO,CMD_SHOWTERRI,
-               CMD_GOTO,CMD_SAVE,CMD_LOAD, CMD_SHOWDRAGONS,CMD_LISTDRAGONS,
-	       NNGS, SETHURRY, SETLEVEL, NEW, COUNT
+               CMD_DEAD, CMD_BACK, CMD_FORWARD, CMD_LAST,
+               CMD_CAPTURE, CMD_DEFEND,
+               CMD_HELPDEBUG, CMD_SHOWAREA, CMD_SHOWMOYO, CMD_SHOWTERRI,
+               CMD_GOTO, CMD_SAVE, CMD_LOAD, CMD_SHOWDRAGONS, CMD_LISTDRAGONS,
+	       SETHURRY, SETLEVEL, NEW, COUNT
 };
 
 
@@ -364,8 +359,8 @@ get_command(char *command)
 
   /* Check to see if a move was input. */
   if (!((sscanf(command, "%c%d", &c, &d) != 2)
-	|| ((c=toupper((int) c)) < 'A')
-	|| ((c=toupper((int) c)) > 'Z')
+	|| ((c = toupper((int) c)) < 'A')
+	|| ((c = toupper((int) c)) > 'Z')
 	|| (c == 'I')))
     return MOVE;
 
@@ -399,7 +394,6 @@ get_command(char *command)
   if (!strncmp(command, "hurry", 5)) return SETHURRY;
   if (!strncmp(command, "level", 5)) return SETLEVEL;
   if (!strncmp(command, "pass", 4)) return PASS;
-  if (!strncmp(command, "nngs", 4)) return NNGS;
   if (!strncmp(command, "save", 3)) return CMD_SAVE;
   if (!strncmp(command, "load", 3)) return CMD_LOAD;
   if (!strncmp(command, "end", 3)) return END;
@@ -438,7 +432,7 @@ init_sgf(Gameinfo *ginfo, SGFNode *root)
 
   sgffile_write_gameinfo(ginfo, "ascii");
   if (ginfo->handicap > 0)
-    gnugo_recordboard(&ginfo->position, root);
+    gnugo_recordboard(root);
   else
     ginfo->to_move = BLACK;
 }
@@ -457,34 +451,27 @@ computer_move(Gameinfo *gameinfo, int *passes)
   init_sgf(gameinfo, sgftree.root);
   
   /* Generate computer move. */
-  move_val = gnugo_genmove(&gameinfo->position, &i, &j, gameinfo->to_move, 
-			   gameinfo->move_number);
+  move_val = gnugo_genmove(&i, &j, gameinfo->to_move);
   if (showscore) {
-    gnugo_estimate_score(&gameinfo->position, &lower_bound, &upper_bound);
+    gnugo_estimate_score(&lower_bound, &upper_bound);
     current_score_estimate = (int) ((lower_bound + upper_bound) / 2.0);
   }
     
   last_move_i = i;
   last_move_j = j;
   
-  mprintf("%s(%d): %m\n", color_to_string(gameinfo->to_move),
-	  gameinfo->move_number, i, j);
-  if (is_pass(i, j))
+  mprintf("%s(%d): %m\n", color_to_string(gameinfo->to_move), movenum+1, i, j);
+  if (is_pass(POS(i, j)))
     (*passes)++;
   else
     *passes = 0;
 
-#if 1
-  gnugo_play_move(&gameinfo->position, i, j, gameinfo->to_move);
+  gnugo_play_move(i, j, gameinfo->to_move);
   curnode = sgfAddPlay(curnode, gameinfo->to_move, i, j);
 
   sgffile_move_made(i, j, gameinfo->to_move, move_val);
   
-  gameinfo->move_number++;
-  gameinfo->to_move = gameinfo->to_move == BLACK ? WHITE : BLACK;
-#else
-  gameinfo_play_move(gameinfo, i, j, gameinfo->to_move);
-#endif
+  gameinfo->to_move = OTHER_COLOR(gameinfo->to_move);
 }
 
 
@@ -497,20 +484,19 @@ do_move(Gameinfo *gameinfo, char *command, int *passes, int force)
 {
   int i, j;
 
-  if (!ascii2pos(&gameinfo->position, command, &i, &j)) {
+  if (!ascii2pos(command, &i, &j)) {
     printf("\nInvalid move: %s\n", command);
     return;
   }
   
-  if (!gnugo_is_legal(&gameinfo->position, i, j, gameinfo->to_move)) {
+  if (!gnugo_is_legal(i, j, gameinfo->to_move)) {
     printf("\nIllegal move: %s", command);
     return;
   }
 
   *passes = 0;
-  gameinfo->move_number++;
   TRACE("\nyour move: %m\n\n", i, j);
-  gnugo_play_move(&gameinfo->position, i, j, gameinfo->to_move);
+  gnugo_play_move(i, j, gameinfo->to_move);
   /* FIXME: This call to init_sgf should not be here. */
   init_sgf(gameinfo, sgftree.root);
   sgffile_move_made(i, j, gameinfo->to_move, 0);
@@ -520,7 +506,7 @@ do_move(Gameinfo *gameinfo, char *command, int *passes, int force)
   last_move_j = j;
   
   if (opt_showboard && !emacs) {
-    ascii_showboard(&gameinfo->position);
+    ascii_showboard();
     printf("GNU Go is thinking...\n");
   }
   if (force) {
@@ -541,8 +527,7 @@ static void
 do_pass(Gameinfo *gameinfo, int *passes, int force)
 {
   (*passes)++;
-  gnugo_play_move(&gameinfo->position, -1, -1, gameinfo->to_move);
-  gameinfo->move_number++;
+  gnugo_play_move(-1, -1, gameinfo->to_move);
   init_sgf(gameinfo, sgftree.root);
   sgffile_move_made(-1, -1, gameinfo->to_move, 0);
   curnode = sgfAddPlay(curnode, gameinfo->to_move, -1, -1);
@@ -572,20 +557,16 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
   char *line_ptr = line;
   char *command;
   char *tmpstring;
-  int nngs = 0; /* NNGS style customization */
   int state = 1;
   
 #ifdef HAVE_SETLINEBUF
-  setlinebuf(stdout); /* Need at least line buffer NNGS and gnugo-gnugo */
+  setlinebuf(stdout); /* Need at least line buffer gnugo-gnugo */
 #else
   setbuf(stdout, NULL); /* else set it to completely UNBUFFERED */
 #endif
   
   while (state == 1) {
 
-    /* Start the move numbering at 0... */
-    /* when playing with gmp it starts at 0. */
-    gameinfo->move_number = 0;
     sgftree = *tree;
     
     /* No score is estimated yet. */
@@ -603,12 +584,11 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
       if (sz)
 	sgfOverwritePropertyInt(sgftree.root, "SZ", sz);
       if (sgfGetIntProperty(sgftree.root, "SZ", &sz)) 
-	gnugo_clear_position(&gameinfo->position, sz, gameinfo->position.komi);
+	gnugo_clear_board(sz);
       if (gameinfo->handicap == 0)
 	gameinfo->to_move = BLACK;
       else {
-	gameinfo->handicap = gnugo_placehand(&gameinfo->position, 
-					     gameinfo->handicap);
+	gameinfo->handicap = gnugo_placehand(gameinfo->handicap);
 	sgfOverwritePropertyInt(sgftree.root, "HA", gameinfo->handicap);
 	gameinfo->to_move = WHITE;
       }
@@ -627,19 +607,17 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
     while (passes < 2 && !time_to_die) {
       /* Display game board. */
       if (opt_showboard)
-	ascii_showboard(&gameinfo->position);
+	ascii_showboard();
       
       /* Print the prompt */
-      if (gameinfo->to_move == WHITE)
-	printf("White(%d): ", gameinfo->move_number+1);
-      if (gameinfo->to_move == BLACK)
-	printf("Black(%d): ", gameinfo->move_number+1);
-      
+      mprintf("%s(%d): ", color_to_string(gameinfo->to_move), movenum + 1);
+
       /* Read a line of input. */
       line_ptr = line;
-      if (!fgets(line, 80, stdin))
-	break; /* EOF or some error */
-      
+      if (!fgets(line, 80, stdin)) {
+	printf("\nThanks! for playing GNU Go.\n\n");
+	return ;
+      }      
       while (command = strtok(line_ptr, ";"), line_ptr = 0, command) {
 	
 	/* Get the command or move. */
@@ -669,12 +647,12 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  gameinfo_print(gameinfo);
 	  break;
 	case SETBOARDSIZE:
-	  if (gameinfo->move_number > 0) {
+	  if (movenum > 0) {
 	    printf("Boardsize can be modified on move 1 only!\n");
 	    break;
 	  }
 	  if (sgf_initialized) {
-	    printf("Boardsize cannot be modified after game record is started!\n");
+	    printf("Boardsize cannot be changed after record is started!\n");
 	    break;
 	  }
 	  command += 10;
@@ -688,21 +666,19 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  }
 	  sz = num;
 	  /* Init board. */
-	  gnugo_clear_position(&gameinfo->position, sz, 
-			       gameinfo->position.komi);
+	  gnugo_clear_board(sz);
 	  /* In case max handicap changes on smaller board. */
-	  gameinfo->handicap = gnugo_placehand(&gameinfo->position, 
-					       gameinfo->handicap);
+	  gameinfo->handicap = gnugo_placehand(gameinfo->handicap);
 	  sgfOverwritePropertyInt(sgftree.root, "SZ", sz);
 	  sgfOverwritePropertyInt(sgftree.root, "HA", gameinfo->handicap);
 	  break;
 	case SETHANDICAP:
-	  if (gameinfo->move_number > 0) {
+	  if (movenum > 0) {
 	    printf("Handicap can be modified on move 1 only!\n");
 	    break;
 	  }
 	  if (sgf_initialized) {
-	    printf("Handicap cannot be modified after game record is started!\n");
+	    printf("Handicap cannot be changed after game is started!\n");
 	    break;
 	  }
 	  command += 9;
@@ -715,17 +691,16 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	    break;
 	  }
 	  /* Init board. */
-	  gnugo_clear_position(&gameinfo->position,
-			       gameinfo->position.boardsize,
-			       gameinfo->position.komi);
-	  /* place stones on board but don't record sgf in case we change more info */
-	  gameinfo->handicap = gnugo_placehand(&gameinfo->position, num);
+	  gnugo_clear_board(board_size);
+	  /* Place stones on board but don't record sgf 
+	   * in case we change more info. */
+	  gameinfo->handicap = gnugo_placehand(num);
 	  sgfOverwritePropertyInt(sgftree.root, "HA", gameinfo->handicap);
 	  printf("\nSet handicap to %d\n", gameinfo->handicap);
 	  gameinfo->to_move = WHITE;
 	  break;
 	case SETKOMI:
-	  if (gameinfo->move_number > 0) {
+	  if (movenum > 0) {
 	    printf("Komi can be modified on move 1 only!\n");
 	    break;
 	  }
@@ -738,8 +713,8 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	    printf("\nInvalid command syntax!\n");
 	    break;
 	  }
-	  gameinfo->position.komi = fnum;
-	  printf("\nSet Komi to %.1f\n", fnum);
+	  komi = fnum;
+	  printf("\nSet Komi to %.1f\n", komi);
 	  break;
 	case SETDEPTH:
 	  {
@@ -779,7 +754,7 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  }
 	case DISPLAY:
 	  if (!opt_showboard)
-	    ascii_showboard(&gameinfo->position);
+	    ascii_showboard();
 	  break;
 	case FORCE:
 	  command += 6; /* skip the force part... */
@@ -837,38 +812,30 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  gameinfo->computer_player = OTHER_COLOR(gameinfo->computer_player);
 	  computer_move(gameinfo, &passes);
 	  break;
-	  /*FIXME: Don't forget boardupdate after back! */
 	case UNDO:
 	case CMD_BACK:
-	  /*reloading the game*/
-	  if (gameinfo->move_number > 0) {
-	    gameinfo->to_move = gnugo_play_sgftree(&gameinfo->position,
-						   sgftree.root,
-						   &gameinfo->move_number,
-						   &curnode);
-	    gameinfo->move_number--;
+	  if (gnugo_undo_move(1)) {
+	    sgffile_write_comment("undo");
+	    curnode = curnode->parent;
+	    gameinfo->to_move = OTHER_COLOR(gameinfo->to_move);
 	  }
 	  else
-	    printf("\nBeginning of game.\n");
+	    printf("\nCan't undo.\n");
 	  break;
 	case CMD_FORWARD:
 	  if (curnode->child) {
-	    gameinfo->to_move = gnugo_play_sgfnode(&gameinfo->position,
-						   curnode->child,
+	    gameinfo->to_move = gnugo_play_sgfnode(curnode->child,
 						   gameinfo->to_move);
 	    curnode = curnode->child;
-	    gameinfo->move_number++;
 	  }
 	  else
 	    printf("\nEnd of game tree.\n");
 	  break;
 	case CMD_LAST:
 	  while (curnode->child) {
-	    gameinfo->to_move = gnugo_play_sgfnode(&gameinfo->position,
-						   curnode->child,
+	    gameinfo->to_move = gnugo_play_sgfnode(curnode->child,
 						   gameinfo->to_move);
 	    curnode = curnode->child;
-	    gameinfo->move_number++;
 	  }
 	  break;
 	case COMMENT:
@@ -886,11 +853,11 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  break;
 	case CMD_CAPTURE:
 	  strtok(command, " ");
-	  showcapture(&gameinfo->position, strtok(NULL, " "));
+	  showcapture(strtok(NULL, " "));
 	  break;
 	case CMD_DEFEND:
 	  strtok(command, " ");
-	  showdefense(&gameinfo->position, strtok(NULL, " "));
+	  showdefense(strtok(NULL, " "));
 	  break;
 	case CMD_SHOWMOYO:
 	  tmp = printmoyo;
@@ -927,13 +894,12 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  if (tmpstring) {
 	    /* discard newline */
 	    tmpstring[strlen(tmpstring)-1] = 0;
-	    sgf_write_header(sgftree.root, 1, gameinfo->seed, 
-			     gameinfo->position.komi);
+	    sgf_write_header(sgftree.root, 1, random_seed, komi, level);
 	    writesgf(sgftree.root, tmpstring);
 	    sgf_initialized = 0;
 	    printf("You may resume the game");
-	    printf(" with -l %s --mode ascii\n",tmpstring);
-	    printf("or load %s\n",tmpstring);
+	    printf(" with -l %s --mode ascii\n", tmpstring);
+	    printf("or load %s\n", tmpstring);
 	  }
 	  else
 	    printf("Please specify filename\n");
@@ -961,19 +927,11 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  examine_position(gameinfo->to_move, EXAMINE_DRAGONS);
 	  show_dragons();
 	  break;
-	case NNGS:
-	  /* Setting the nngs flag will customize the ascii interface
-	   * for use by the NNGS connecting program. This mode of
-	   * operation is unlikely to be useful for other purposes.
-	   */
-	  nngs = 1;
-	  break;
-	  /* These commands are only valid at the end of the game */
 	case COUNT:
 	case NEW:
 	case INVALID:
 	default:
-	  printf("\nInvalid command: %s",command);
+	  printf("\nInvalid command: %s", command);
 	  break;
 	}
       }
@@ -982,7 +940,7 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
     /* two passes : game over */
     
     if (passes >= 2)
-      gnugo_who_wins(&gameinfo->position, gameinfo->computer_player, stdout);
+      gnugo_who_wins(gameinfo->computer_player, stdout);
     printf("\nIf you disagree, we may count the game together.\n");
     printf("You may optionally save the game as an SGF file.\n");
     state = 0;
@@ -991,20 +949,19 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 Type \"save <filename>\" to save,\n\
      \"count\" to recount,\n\
      \"quit\" to quit\n\
- or  \"game \" to play again\n");
+ or  \"game\" to play again\n");
       line_ptr = line;
       if (!fgets(line, 80, stdin))
 	break;
       command = strtok(line_ptr, "");
-      switch(get_command(command)) {
+      switch (get_command(command)) {
       case CMD_SAVE:
 	strtok(command, " ");
 	tmpstring = strtok(NULL, " ");
 	if (tmpstring) {
 	  /* discard newline */
 	  tmpstring[strlen(tmpstring)-1] = 0;
-	  sgf_write_header(sgftree.root, 1, gameinfo->seed, 
-			   gameinfo->position.komi);
+	  sgf_write_header(sgftree.root, 1, random_seed, komi, level);
 	  writesgf(sgftree.root, tmpstring);
 	    sgf_initialized = 0;
 	}
@@ -1017,10 +974,7 @@ Type \"save <filename>\" to save,\n\
 	break;
 	
       case COUNT:
-	if (nngs)
-	  nngs_endgame(gameinfo);
-	else
-	  endgame(gameinfo);
+	endgame(gameinfo);
 	break;
 
       case QUIT:
@@ -1034,6 +988,8 @@ Type \"save <filename>\" to save,\n\
     passes = 0;
     showdead = 0;
     sgf_initialized = 0;
+    /* Play a different game next time. */
+    update_random_seed();
   }
   printf("\nThanks for playing GNU Go.\n\n");
 }
@@ -1058,13 +1014,10 @@ endgame(Gameinfo *gameinfo)
   int done = 0;
   int i, j;
   int xyzzy = 1;
-  float komi;
-
-  komi = gameinfo->position.komi;
-
+  
   printf("\nGame over. Let's count!.\n");  
   showdead = 1;
-  ascii_showboard(&gameinfo->position);
+  ascii_showboard();
   while (!done) {
     printf("Dead stones are marked with small letters (x,o).\n");
     printf("\nIf you don't agree, enter the location of a group\n\
@@ -1100,104 +1053,33 @@ to toggle its state or \"done\".\n");
     }      
     else if (!strncmp(line, "undo", 4)) {
       printf("UNDO not allowed anymore. The status of the stones now toggles after entering the location of it.\n");
-      ascii_showboard(&gameinfo->position);
+      ascii_showboard();
     }
     else {
-      if (!ascii2pos(&gameinfo->position, line, &i, &j)
-	  || gameinfo->position.board[i][j] == EMPTY)
+      if (!ascii2pos(line, &i, &j) || BOARD(i, j) == EMPTY)
 	printf("\ninvalid!\n");
       else {
-	int status = matcher_status(i, j);
+	int status = matcher_status(POS(i, j));
 	status = (status == DEAD) ? ALIVE : DEAD;
-	change_matcher_status(i, j, status);
-	ascii_showboard(&gameinfo->position);
+	change_matcher_status(POS(i, j), status);
+	ascii_showboard();
       }
     }
   }
-  gnugo_who_wins(&gameinfo->position, gameinfo->computer_player, stdout);
-}
-
-
-/*
- * nngs_endgame() records the strings the opponent says are dead and
- * compares this to our own opinion.
- */
-
-static void
-nngs_endgame(Gameinfo *gameinfo)
-{
-  char line[12];
-  int i, j;
-  int worm_status[MAX_BOARD][MAX_BOARD];
-  int m, n;
-  static const char *snames[] = 
-    {"dead", "alive", "critical", "unknown"};
-  
-  for (m = 0; m < gameinfo->position.boardsize; m++)
-    for (n = 0; n < gameinfo->position.boardsize; n++)
-      worm_status[m][n] = ALIVE;
-  
-  while (1) {
-    printf("Bring out your dead!\n");
-
-    if (!fgets(line, 12, stdin))
-      return; /* EOF or some error */
-    
-    for (i = 0; i < 12; i++)
-      line[i] = (isupper ((int) line[i]) ? tolower ((int) line[i]) : line[i]);
-
-    if (!strncmp(line, "done", 4))
-      break;
-    else if (!strncmp(line, "quit", 4))
-      return;
-    else if (!strncmp(line, "pass", 4))  /* ignore */
-      continue;
-    else if (!strncmp(line, "undo", 4)) {
-      for (m = 0; m < gameinfo->position.boardsize; m++)
-	for (n = 0; n < gameinfo->position.boardsize; n++)
-	  worm_status[m][n] = ALIVE;
-    }
-    else {
-      if (!ascii2pos(&gameinfo->position, line, &i, &j)
-	  || gameinfo->position.board[i][j] == EMPTY)
-	printf("\ninvalid!\n");
-      else {
-	for (m = 0; m < gameinfo->position.boardsize; m++)
-	  for (n = 0; n < gameinfo->position.boardsize; n++)
-	    if (is_same_worm(m, n, i, j))
-	      worm_status[m][n] = DEAD;
-      }
-    }
-  }
-
-  for (m = 0; m < gameinfo->position.boardsize; m++)
-    for (n = 0; n < gameinfo->position.boardsize; n++)
-      if (is_worm_origin(m, n, m, n)
-	  && gameinfo->position.board[m][n] != EMPTY) {
-	if (worm_status[m][n] != dragon_status(m, n)) {
-	  char buf[120];
-	  sprintf(buf,
-		  "Disagreement over %c%d: GNU Go says %s, opponent says %s.\n",
- 		  'A'+n+(n>=8), gameinfo->position.boardsize-m,
-		  snames[dragon_status(m, n)],
-		  snames[worm_status[m][n]]);
-	  mprintf(buf);
-	  sgfAddComment(curnode, buf);
-	}
-      }
+  gnugo_who_wins(gameinfo->computer_player, stdout);
 }
 
 
 static void
-showcapture(Position *pos, char *line)
+showcapture(char *line)
 {
   int i, j, x, y;
   if (line)
-    if (!ascii2pos(pos, line, &i, &j) || pos->board[i][j] == EMPTY) {
+    if (!ascii2pos(line, &i, &j) || BOARD(i, j) == EMPTY) {
       printf("\ninvalid point!\n");
       return;
     }
-  if (gnugo_attack(pos, i, j, &x, &y))
+  if (gnugo_attack(i, j, &x, &y))
     mprintf("\nSuccessfull attack of %m at %m\n", i, j, x, y);
   else
     mprintf("\n%m cannot be attacked\n", i, j);
@@ -1205,17 +1087,17 @@ showcapture(Position *pos, char *line)
 
 
 static void
-showdefense(Position *pos, char *line)
+showdefense(char *line)
 {
   int i, j, x, y;
   if (line)
-    if (!ascii2pos(pos, line, &i, &j) || pos->board[i][j] == EMPTY) {
+    if (!ascii2pos(line, &i, &j) || BOARD(i, j) == EMPTY) {
       printf("\ninvalid point!\n");
       return;
     }
 
-    if (gnugo_attack(pos, i, j, &x, &y)) {
-      if (gnugo_find_defense(pos, i, j, &x, &y))
+    if (gnugo_attack(i, j, &x, &y)) {
+      if (gnugo_find_defense(i, j, &x, &y))
         mprintf("\nSuccessfull defense of %m at %m\n", i, j, x, y);
       else
         mprintf("\n%m cannot be defended\n", i, j);
@@ -1241,16 +1123,17 @@ ascii_goto(Gameinfo *gameinfo, char *line)
   }
   printf("goto %i\n", movenumber);
   curnode = sgftree.root;
-  gameinfo->to_move = gnugo_play_sgftree(&gameinfo->position, curnode,
-					 &movenumber, &curnode);
-  gameinfo->move_number = movenumber-1;
+  gameinfo->to_move = gnugo_play_sgftree(curnode, &movenumber, &curnode);
   return;
 }
 
 
-/* Convert a coordinate pair from ascii text to two integers. */
+/* Convert a coordinate pair from ascii text to two integers.
+ * FIXME: Check that this function is indeed equivalent to
+ * string_to_location() and then replace it.
+ */
 static int
-ascii2pos(Position *pos, char *line, int *i, int *j)
+ascii2pos(char *line, int *i, int *j)
 {
   int d;
   char c;
@@ -1265,9 +1148,9 @@ ascii2pos(Position *pos, char *line, int *i, int *j)
   if (tolower((int) c) > 'i')
     --*j;
   
-  *i = pos->boardsize - d;
+  *i = board_size - d;
 
-  if (*i < 0 || *i >= pos->boardsize || *j < 0 || *j >= pos->boardsize)
+  if (*i < 0 || *i >= board_size || *j < 0 || *j >= board_size)
     return 0;
   
   return 1;
