@@ -1,25 +1,25 @@
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
- * http://www.gnu.org/software/gnugo/ for more information.      *
- *                                                               *
- * Copyright 1999, 2000, 2001 by the Free Software Foundation.   *
- *                                                               *
- * This program is free software; you can redistribute it and/or *
- * modify it under the terms of the GNU General Public License   *
- * as published by the Free Software Foundation - version 2.     *
- *                                                               *
- * This program is distributed in the hope that it will be       *
- * useful, but WITHOUT ANY WARRANTY; without even the implied    *
- * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
- * PURPOSE.  See the GNU General Public License in file COPYING  *
- * for more details.                                             *
- *                                                               *
- * You should have received a copy of the GNU General Public     *
- * License along with this program; if not, write to the Free    *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
- * Boston, MA 02111, USA.                                        *
-\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
+ * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * http://www.gnu.org/software/gnugo/ for more information.          *
+ *                                                                   *
+ * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ *                                                                   *
+ * This program is free software; you can redistribute it and/or     *
+ * modify it under the terms of the GNU General Public License as    *
+ * published by the Free Software Foundation - version 2             *
+ *                                                                   *
+ * This program is distributed in the hope that it will be useful,   *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
+ * GNU General Public License in file COPYING for more details.      *
+ *                                                                   *
+ * You should have received a copy of the GNU General Public         *
+ * License along with this program; if not, write to the Free        *
+ * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Boston, MA 02111, USA.                                            *
+\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
+#include "gnugo.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,7 +32,6 @@
 #include "liberty.h" /* to get to the stats */
 
 #include "sgftree.h"
-#include "gnugo.h"
 #include "random.h"
 #include "gg_utils.h"
 
@@ -44,7 +43,7 @@ play_solo(Gameinfo *gameinfo, int moves)
   int move_val;
   double t1, t2;
   int save_moves = moves;
-  int boardsize = gameinfo->position.boardsize;
+  int boardsize = gnugo_get_boardsize();
 
   struct stats_data totalstats;
   int total_owl_count = 0;
@@ -66,31 +65,28 @@ play_solo(Gameinfo *gameinfo, int moves)
       do {
 	i = (gg_rand() % 4) + (gg_rand() % (boardsize - 4));
 	j = (gg_rand() % 4) + (gg_rand() % (boardsize - 4));
-      } while (!gnugo_is_legal(&gameinfo->position, i, j, 
-			       gameinfo->to_move));
+      } while (!gnugo_is_legal(i, j, gameinfo->to_move));
       
       gameinfo_play_move(gameinfo, i, j, gameinfo->to_move);
     } while (--n > 0);
   }
   
-  t1 = gg_gettimeofday();
-  memset(&totalstats,'\0',sizeof(totalstats));
+  t1 = gg_cputime();
+  memset(&totalstats, '\0', sizeof(totalstats));
   while (passes < 2 && --moves >= 0 && !time_to_die) {
     reset_owl_node_counter();
-    move_val = gnugo_genmove(&gameinfo->position, &i, &j,
-			     gameinfo->to_move,
-			     gameinfo->move_number);
+    move_val = gnugo_genmove(&i, &j, gameinfo->to_move);
     gameinfo_play_move(gameinfo, i, j, gameinfo->to_move);
 
     if (move_val < 0) {
       ++passes;
-      printf("%s(%d): Pass\n", gameinfo->to_move==BLACK ? "Black" : "White",
-	     gameinfo->move_number);
+      printf("%s(%d): Pass\n", gameinfo->to_move == BLACK ? "Black" : "White",
+	     movenum);
     }
     else {
       passes = 0;
-      gprintf("%s(%d): %m\n", gameinfo->to_move==BLACK ? "Black" : "White",
-	      gameinfo->move_number, i, j);
+      gprintf("%s(%d): %m\n", gameinfo->to_move == BLACK ? "Black" : "White",
+	      movenum, i, j);
     }
 
     totalstats.nodes               += stats.nodes;
@@ -100,10 +96,10 @@ play_solo(Gameinfo *gameinfo, int moves)
     totalstats.hash_collisions     += stats.hash_collisions;
     total_owl_count                += get_owl_node_counter();
   }
-  t2 = gg_gettimeofday();
+  t2 = gg_cputime();
   
   /* Two passes and it's over. (EMPTY == BOTH) */
-  gnugo_who_wins(&gameinfo->position, EMPTY, stdout);
+  gnugo_who_wins(EMPTY, stdout);
 
 #if 0
   if (t2 == t1)
@@ -111,9 +107,9 @@ play_solo(Gameinfo *gameinfo, int moves)
   else
     printf("%.3f moves/sec\n", (save_moves-moves)/(t2-t1));
 #else
-  printf("%10d moves played in %0.3f seconds\n", save_moves-moves,t2-t1);
+  printf("%10d moves played in %0.3f seconds\n", save_moves-moves, t2-t1);
   if (save_moves != moves)
-    printf("%10.3f seconds/move\n", (t2-t1)/(save_moves-moves) );
+    printf("%10.3f seconds/move\n", (t2-t1)/(save_moves-moves));
   printf("%10d nodes\n", totalstats.nodes);
   printf("%10d positions entered\n", totalstats.position_entered);
   printf("%10d position hits\n", totalstats.position_hits);
@@ -136,20 +132,16 @@ play_solo(Gameinfo *gameinfo, int moves)
  */
 
 void 
-load_and_analyze_sgf_file(SGFNode *head, Gameinfo *gameinfo, 
-			  const char *untilstr, int benchmark)
+load_and_analyze_sgf_file(Gameinfo *gameinfo, int benchmark)
 {
   int i, j;
   int next;
   int r;
   
-  /* We load the header to get correct boardsize, komi, and handicap
-   * for writing.
-   */
-  gameinfo_load_sgfheader(gameinfo, head); 
+  next = gameinfo->to_move;
+  gameinfo->computer_player = next;
   sgffile_write_gameinfo(gameinfo, "load and analyze");
-  next = gameinfo_play_sgftree(gameinfo, head, untilstr);
-  
+
   if (benchmark) {
     for (r = 0; r < benchmark; ++r) {
       genmove(&i, &j, next);
@@ -159,13 +151,14 @@ load_and_analyze_sgf_file(SGFNode *head, Gameinfo *gameinfo,
   else {
     genmove(&i, &j, next);
     
-    if (is_pass(i, j)) {
-      gprintf("%s move: PASS!\n", next == WHITE ? "white (o)" : "black (X)");
+    if (is_pass(POS(i, j))) {
+      gprintf("%s move: PASS!\n", next == WHITE ? "white (O)" : "black (X)");
       sgffile_move_made(i, j, next, 0);
     }
     else {
-      gprintf("%s move %m\n", next == WHITE ? "white (o)" : "black (X)", i,j);
-      gnugo_play_move(&gameinfo->position, i, j, next);
+      gprintf("%s move %m\n", next == WHITE ? "white (O)" : "black (X)",
+	      i, j);
+      gnugo_play_move(i, j, next);
       sgffile_move_made(i, j, next, 0);
     }
   }
@@ -174,66 +167,67 @@ load_and_analyze_sgf_file(SGFNode *head, Gameinfo *gameinfo,
 
 /*
  * Load SGF file and score the game
- * untilstr:
- * end  - finish the game by selfplaying from the end of the file until 
- *        two passes
- * last - estimate territorial balance at the end of the of the file
- * move - load file until move is reached and estimate territorial balance
- *
- * aftermath - like 'end' but also plays out the aftermath
+ * scoringmode:
+ * estimate  - estimate territorial balance
+ * finish    - finish the game by selfplaying and then count the score quickly
+ * aftermath - like 'finish' but also play out the aftermath in order to
+ *             get an accurate score
  */
 
 void 
 load_and_score_sgf_file(SGFTree *tree, Gameinfo *gameinfo, 
-			const char *untilstr)
+			const char *scoringmode)
 {
-  int i, j, move_val, pass;
+  int i, j, move_val;
   int until;
-  float komi;
   float result;
   char *tempc = NULL;
   char dummy;
   char text[250];
   char winner;
   int next;
-  pass = 0;
-
-  assert(tree);
-  gameinfo_load_sgfheader(gameinfo, tree->root);
-  sgffile_write_gameinfo(gameinfo, "load and score");
-  next = gameinfo_play_sgftree(gameinfo, tree->root, untilstr);
-  komi = gameinfo->position.komi;
-  doing_scoring = 1;
+  int pass = 0;
+  SGFTree score_tree;
+  SGFNode *node;
   
-  until = atoi(untilstr);
-  if (!strcmp(untilstr, "end") || !strcmp(untilstr, "aftermath")) {
+  sgftree_clear(&score_tree);
+  node = sgftreeCreateHeaderNode(&score_tree, board_size, komi);
+  sgftreeSetLastNode(&score_tree, node);
+  sgftree_printboard(&score_tree);
+  
+  sgffile_write_gameinfo(gameinfo, "load and score");
+  next = gameinfo->to_move;
+  sgffile_printboard(next);
+  doing_scoring = 1;
+  reset_engine();
+  
+  if (!strcmp(scoringmode, "finish") || !strcmp(scoringmode, "aftermath")) {
     until = 9999;
     do {
       move_val = genmove_conservative(&i, &j, next);
-      play_move(i, j, next);
       if (move_val >= 0) {
 	pass = 0;
-	gprintf("%d %s move %m\n", gameinfo->move_number,
-		next == WHITE ? "white (o)" : "black (X)", i, j);
+	gprintf("%d %s move %m\n", movenum,
+		next == WHITE ? "white (O)" : "black (X)", i, j);
       }
       else {
 	++pass;
-	gprintf("%d %s move : PASS!\n", gameinfo->move_number, 
-		next == WHITE ? "white (o)" : "black (X)");
+	gprintf("%d %s move : PASS!\n", movenum, 
+		next == WHITE ? "white (O)" : "black (X)");
       }
+      play_move(POS(i, j), next);
       sgffile_move_made(i, j, next, move_val);
-      sgftreeAddPlay(tree, 0, next, i, j);
-      gameinfo->move_number++;
+      sgftreeAddPlay(&score_tree, NULL, next, i, j);
       next = OTHER_COLOR(next);
-    } while ((gameinfo->move_number <= until) && (pass < 2));
+    } while (movenum <= until && pass < 2);
 
     if (pass >= 2) {
+      node = score_tree.lastnode;
       /* Calculate the score */
-      if (!strcmp(untilstr, "aftermath"))
-	score = aftermath_compute_score(next, komi);
+      if (!strcmp(scoringmode, "aftermath"))
+	score = aftermath_compute_score(next, komi, &score_tree);
       else
-	score = gnugo_estimate_score(&gameinfo->position,
-				     &lower_bound, &upper_bound);
+	score = gnugo_estimate_score(&lower_bound, &upper_bound);
 
       if (score < 0.0) {
 	sprintf(text, "Black wins by %1.1f points\n", -score);
@@ -248,13 +242,13 @@ load_and_score_sgf_file(SGFTree *tree, Gameinfo *gameinfo,
 	winner = '0';
       }
       fputs(text, stdout);
-      sgfAddComment(tree->lastnode, text);
+      sgfAddComment(node, text);
       sgffile_write_comment(text);
       if (sgfGetCharProperty(tree->root, "RE", &tempc)) {
 	if (sscanf(tempc, "%1c%f", &dummy, &result) == 2) {
 	  fprintf(stdout, "Result from file: %1.1f\n", result);
 	  fputs("GNU Go result and result from file are ", stdout);
-	  if ((result == fabs(score)) && (winner == dummy))
+	  if (result == fabs(score) && winner == dummy)
 	    fputs("identical\n", stdout);
 	  else
 	    fputs("different\n", stdout);
@@ -271,15 +265,22 @@ load_and_score_sgf_file(SGFTree *tree, Gameinfo *gameinfo,
 	  }
 	}
       }
-      sgfWriteResult(tree->root, score, 1);
+      sgfWriteResult(score_tree.root, score, 1);
     }
   }
   doing_scoring = 0;
 
-  if (!strcmp(untilstr, "aftermath"))
-    return;
 
-  move_val = genmove_conservative(&i, &j, next);
+  if (!strcmp(scoringmode, "aftermath")) {
+    if (gameinfo->outfile)
+      writesgf(score_tree.root, gameinfo->outfilename);
+    return;
+  }
+
+  /* Before we call estimate_score() we must make sure that the dragon
+   * status is computed. Therefore the call to examine_position().
+   */
+  examine_position(next, EXAMINE_ALL);
   score = estimate_score(NULL, NULL);
 
   fprintf(stdout, "\n%s seems to win by %1.1f points\n",
