@@ -1,24 +1,25 @@
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
- * http://www.gnu.org/software/gnugo/ for more information.      *
- *                                                               *
- * Copyright 1999, 2000, 2001 by the Free Software Foundation.   *
- *                                                               *
- * This program is free software; you can redistribute it and/or *
- * modify it under the terms of the GNU General Public License   *
- * as published by the Free Software Foundation - version 2.     *
- *                                                               *
- * This program is distributed in the hope that it will be       *
- * useful, but WITHOUT ANY WARRANTY; without even the implied    *
- * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
- * PURPOSE.  See the GNU General Public License in file COPYING  *
- * for more details.                                             *
- *                                                               *
- * You should have received a copy of the GNU General Public     *
- * License along with this program; if not, write to the Free    *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
- * Boston, MA 02111, USA.                                        *
-\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
+ * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * http://www.gnu.org/software/gnugo/ for more information.          *
+ *                                                                   *
+ * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ *                                                                   *
+ * This program is free software; you can redistribute it and/or     *
+ * modify it under the terms of the GNU General Public License as    *
+ * published by the Free Software Foundation - version 2             *
+ *                                                                   *
+ * This program is distributed in the hope that it will be useful,   *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
+ * GNU General Public License in file COPYING for more details.      *
+ *                                                                   *
+ * You should have received a copy of the GNU General Public         *
+ * License along with this program; if not, write to the Free        *
+ * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Boston, MA 02111, USA.                                            *
+\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+
+#include "gnugo.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,9 +29,10 @@
 
 #include "interface.h"
 #include "sgftree.h"
-#include "gnugo.h"
+#include "gg_utils.h"
+#include "liberty.h"
 
-static void replay_node(SGFNode *node, Gameinfo *gameinfo, int color_to_test);
+static void replay_node(SGFNode *node, int color_to_test);
 
 
 /* --------------------------------------------------------------*/
@@ -62,7 +64,7 @@ play_replay(SGFNode *sgf_head, int color_to_replay)
 
   /* Get the komi. */
   if (sgfGetFloatProperty(sgf_head, "KM", &tmpf))
-    gameinfo.position.komi = tmpf;
+    komi = tmpf;
 
   if (!quiet) {
     if (sgfGetCharProperty(sgf_head, "RU", &tmpc))
@@ -92,7 +94,7 @@ play_replay(SGFNode *sgf_head, int color_to_replay)
    */
   node = sgf_head;
   while (node) {
-    replay_node(node, &gameinfo, color_to_replay);
+    replay_node(node, color_to_replay);
     node = node->child;
   }
 
@@ -106,13 +108,13 @@ play_replay(SGFNode *sgf_head, int color_to_replay)
  */
 
 static void
-replay_node(SGFNode *node, Gameinfo *gameinfo, int color_to_replay)
+replay_node(SGFNode *node, int color_to_replay)
 {
   SGFProperty *sgf_prop;  /* iterate over properties of the node */
   SGFProperty *move_prop = NULL; /* remember if we see a move property */
   int color; /* color of move to be made at this node. */
   
-  int boardsize = gameinfo->position.boardsize;
+  int boardsize = gnugo_get_boardsize();
   int m, n; /* Move from file. */
   int i, j; /* Move generated by GNU Go. */
 
@@ -124,16 +126,14 @@ replay_node(SGFNode *node, Gameinfo *gameinfo, int color_to_replay)
     switch (sgf_prop->name) {
     case SGFAB:
       /* add black */
-      gnugo_add_stone(&gameinfo->position,
-		      get_moveX(sgf_prop, boardsize),
-		      get_moveY(sgf_prop, boardsize),	BLACK);
+      gnugo_add_stone(get_moveX(sgf_prop, boardsize),
+		      get_moveY(sgf_prop, boardsize), BLACK);
       sgffile_put_stone(get_moveX(sgf_prop, boardsize),
-			get_moveY(sgf_prop, boardsize),	BLACK);
+			get_moveY(sgf_prop, boardsize), BLACK);
       break;
     case SGFAW:
       /* add white */
-      gnugo_add_stone(&gameinfo->position,
-		      get_moveX(sgf_prop, boardsize),
+      gnugo_add_stone(get_moveX(sgf_prop, boardsize),
 		      get_moveY(sgf_prop, boardsize), WHITE);
       sgffile_put_stone(get_moveX(sgf_prop, boardsize),
 			get_moveY(sgf_prop, boardsize), WHITE);
@@ -155,25 +155,34 @@ replay_node(SGFNode *node, Gameinfo *gameinfo, int color_to_replay)
 
   if (color == color_to_replay || color_to_replay == GRAY) {
     /* Get a move from the engine for color. */
-    gnugo_genmove(&gameinfo->position, &i, &j, color, gameinfo->move_number);
+    gnugo_genmove(&i, &j, color);
     /* Now report on how well the computer generated the move. */
     if (i != m || j != n || !quiet) {
-      mprintf("Move %d (%C): ", gameinfo->move_number + 1, color);
+      mprintf("Move %d (%C): ", movenum + 1, color);
     
       mprintf("GNU Go plays %m ", i, j);
-      if (!is_pass(i, j))
+      if (!gnugo_is_pass(i, j))
 	printf("(%.2f) ", potential_moves[i][j]);
       mprintf("- Game move %m ", m, n);
-      if (!is_pass(m, n) && potential_moves[m][n] > 0.0)
+      if (!gnugo_is_pass(m, n) && potential_moves[m][n] > 0.0)
 	printf("(%.2f) ", potential_moves[m][n]);
       printf("\n");
     }
+    if (i != m || j != n) {
+      char buf[127];
+      gg_snprintf(buf, 127, "GNU Go plays %s(%.2f) - Game move %s(%.2f)",
+	location_to_string(POS(i,j)),
+	gnugo_is_pass(i, j) ? 0 : potential_moves[i][j],
+	location_to_string(POS(m,n)),
+        gnugo_is_pass(m, n) && potential_moves[m][n] < 0.0 ? 0 : potential_moves[m][n]);
+      sgffile_write_comment(buf);
+      sgffile_write_circle_mark(i,j);
+    }
   }
 
   /* Finally, do play the move from the file. */
-  gnugo_play_move(&gameinfo->position, m, n, color);
+  gnugo_play_move(m, n, color);
   sgffile_move_made(m, n, color, 0);
-  gameinfo->move_number++;
 }
 
 
