@@ -1,24 +1,23 @@
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
- * http://www.gnu.org/software/gnugo/ for more information.      *
- *                                                               *
- * Copyright 1999, 2000, 2001 by the Free Software Foundation.   *
- *                                                               *
- * This program is free software; you can redistribute it and/or *
- * modify it under the terms of the GNU General Public License   *
- * as published by the Free Software Foundation - version 2.     *
- *                                                               *
- * This program is distributed in the hope that it will be       *
- * useful, but WITHOUT ANY WARRANTY; without even the implied    *
- * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
- * PURPOSE.  See the GNU General Public License in file COPYING  *
- * for more details.                                             *
- *                                                               *
- * You should have received a copy of the GNU General Public     *
- * License along with this program; if not, write to the Free    *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
- * Boston, MA 02111, USA.                                        *
-\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
+ * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * http://www.gnu.org/software/gnugo/ for more information.          *
+ *                                                                   *
+ * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ *                                                                   *
+ * This program is free software; you can redistribute it and/or     *
+ * modify it under the terms of the GNU General Public License as    *
+ * published by the Free Software Foundation - version 2             *
+ *                                                                   *
+ * This program is distributed in the hope that it will be useful,   *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
+ * GNU General Public License in file COPYING for more details.      *
+ *                                                                   *
+ * You should have received a copy of the GNU General Public         *
+ * License along with this program; if not, write to the Free        *
+ * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Boston, MA 02111, USA.                                            *
+\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 /* Extract fuseki patterns from the initial moves of a collection
  * of games.
@@ -71,17 +70,21 @@
 #include <limits.h>
 #include "../sgf/sgftree.h"
 #include "liberty.h"
+#include "gg_utils.h"
 #include "random.h"
 
 #define USAGE "\n\
-Usage: extract_fuseki files boardsize moves patterns\n\
-\n\
+Usage: extract_fuseki files boardsize moves patterns handicap strength half_board [output file]\n\
 files:     The name of a file listing sgf files to examine,\n\
            one filename per line.\n\
 boardsize: Only consider games with this size.\n\
 moves:     Number of moves considered in each game.\n\
 patterns:  Number of patterns to generate.\n\
-\n\
+handicap:  0 - no handicap, 1 - any game, 2-9 - two to nine handicap stones\n\
+           10 any handicap game\n\
+strength:  The lowest strength of the players (1k-30k)\n\
+half_board: 0 - full board patterns, 1 - half board patterns\n\
+output file: Optional (if this exists, extract_fuseki will sort the games instead)\n\
 "
 
 /* Maximum length of sgf filename. */
@@ -89,11 +92,25 @@ patterns:  Number of patterns to generate.\n\
 
 /* Number of moves to consider in each game, given as argument.*/
 int moves_per_game;
+
+/* Flag checking the setting for generating half board patterns */
+int half_board_patterns = 0;
+
 /* Maximum number of patterns to generate, given as argument.*/
 int patterns_to_extract;
 
+/* Handicap value, given as argument.*/
+int handicap_value;
+
+/* Lowest strength, given as argument.*/
+int player_strength;
+
 /* Number of games to analyze. */
 int number_of_games;
+
+/* The number of games that could not be used for some reason. */
+int *unused_games;
+
 /* Dynamically allocated list of sgf file names. */
 char **sgf_names;
 
@@ -112,11 +129,11 @@ struct invariant_hash {
 /* This is defined in engine/matchpat.c */
 extern const int transformations[8][2][2];
 
-#define TRANSFORM(i,j,ti,tj,trans) \
-do { \
-  *ti = transformations[trans][0][0] * (i) + transformations[trans][0][1] * (j); \
-  *tj = transformations[trans][1][0] * (i) + transformations[trans][1][1] * (j); \
-} while(0)
+#define TRANSFORM(i, j, ti, tj, trans) \
+  do { \
+    *ti = transformations[trans][0][0] * (i) + transformations[trans][0][1] * (j); \
+    *tj = transformations[trans][1][0] * (i) + transformations[trans][1][1] * (j); \
+  } while (0)
 
 
 /* A situation is the combination of a board position and the move to
@@ -124,7 +141,7 @@ do { \
  * as identification. If are interested in positions, we only use the first
  * hash value.
  *
- * Yes, we ignore the possibility of a hash collision.
+ * We ignore the possibility of a hash collision.
  */
 struct situation {
   struct invariant_hash pre;
@@ -161,6 +178,27 @@ struct winner {
 struct winner *winning_moves;
 int number_of_winning_moves;
 
+/*
+ * Write the files that are sorted to a specific file
+ */
+
+static void
+write_sgf_filenames(const char *name, char *filenames[])
+{
+  int n;
+  FILE *namefile = fopen(name, "a");
+  if (!namefile) {
+    fprintf(stderr, "Fatal error, couldn't open %s.\n", name);
+    exit(EXIT_FAILURE);
+  }
+  
+  for (n = 0; n < number_of_games; n++) {
+    if (unused_games[n] == 0)
+      fprintf(namefile, "%s\n", filenames[n]);
+  }
+}
+
+
 /* Read the sgf file names. These are assumed to be stored one per
  * line in the file with the name given by 'name'. The sgf file names
  * are copied into dynamically allocated memory by strdup() and
@@ -184,7 +222,15 @@ read_sgf_filenames(const char *name, char *filenames[])
   n = 0;
   while (fgets(buf, BUFSIZE, namefile) != NULL) {
     if (filenames != NULL) {
-      buf[strlen(buf) - 1] = 0; /* Delete newline character. */
+      if (buf[strlen(buf) - 2] == '\r') {
+	buf[strlen(buf) - 2] = '\0'; 
+	/* Delete newline character. PC only */
+      }
+      else {
+	buf[strlen(buf) - 1] = '\0'; 
+	/* Delete newline character. */
+      }
+      
       filenames[n] = strdup(buf);
       if (filenames[n] == NULL) {
 	fprintf(stderr, "Fatal error, strdup() failed.\n");
@@ -193,7 +239,7 @@ read_sgf_filenames(const char *name, char *filenames[])
     }
     n++;
   }
-
+  
   return n;
 }
 
@@ -205,14 +251,14 @@ init_zobrist_table(unsigned int hash[8][MAX_BOARD][MAX_BOARD])
   int m, n;
   int i, j;
   int mid = board_size/2;
-
+  
   for (m = 0; m < board_size; m++)
     for (n = 0; n < board_size; n++) {
       hash[0][m][n] = 0;
       for (k = 0; 32*k < CHAR_BIT*sizeof(hash[0][0][0]); k++)
 	hash[0][m][n] |= gg_urand() << k*32;
     }
-
+  
   /* Fill in all transformations of the hash table. */
   for (k = 1; k < 8; k++)
     for (m = 0; m < board_size; m++)
@@ -220,7 +266,7 @@ init_zobrist_table(unsigned int hash[8][MAX_BOARD][MAX_BOARD])
 	TRANSFORM(m-mid, n-mid, &i, &j, k);
 	hash[k][m][n] = hash[0][i+mid][j+mid];
       }
-
+  
   /* Debug output. */
   if (0) {
     for (k = 0; k < 8; k++) {
@@ -250,7 +296,7 @@ static void
 init_situations(void)
 {
   situation_table = calloc(moves_per_game * number_of_games,
-			  sizeof(*situation_table));
+			   sizeof(*situation_table));
   if (!situation_table) {
     fprintf(stderr, "Fatal error, failed to allocate situations table.\n");
     exit(EXIT_FAILURE);
@@ -289,9 +335,9 @@ common_hash_board(struct invariant_hash *hash, int color_to_play)
   for (m = 0; m < board_size; m++)
     for (n = 0; n < board_size; n++) {
       for (k = 0; k < 8; k++) {
-	if (p[m][n] == color_to_play)
+	if (BOARD(m, n) == color_to_play)
 	  hash->values[k] ^= O_hash[k][m][n];
-	else if (p[m][n] != EMPTY)
+	else if (BOARD(m, n) != EMPTY)
 	  hash->values[k] ^= X_hash[k][m][n];
       }
     }
@@ -303,7 +349,7 @@ hash_board(struct invariant_hash *hash, int color_to_play)
 {
   common_hash_board(hash, color_to_play);
   /* Sort the 8 hash values. */
-  qsort(hash->values, 8, sizeof(hash->values[0]), compare_numbers);
+  gg_sort(hash->values, 8, sizeof(hash->values[0]), compare_numbers);
 }
 
 /* Compute invariant hash for the current situation, i.e. position
@@ -316,14 +362,14 @@ hash_board_and_move(struct invariant_hash *hash, int color_to_play,
   int k;
   
   common_hash_board(hash, color_to_play);
-
+  
   for (k = 0; k < 8; k++)
     hash->values[k] ^= move_hash[k][m][n];
-
+  
   /* Notice that we of course must wait with sorting until we have
    * added the move to the hash values.
    */
-  qsort(hash->values, 8, sizeof(hash->values[0]), compare_numbers);
+  gg_sort(hash->values, 8, sizeof(hash->values[0]), compare_numbers);
 }
 
 
@@ -338,22 +384,28 @@ get_move_from_sgf(SGFNode *node, int *m, int *n, int *color)
 {
   SGFProperty *prop;
   int i, j;
-      
-  for (prop=node->props; prop; prop=prop->next) {
-    switch(prop->name) {
+  
+  for (prop = node->props; prop; prop = prop->next) {
+    if (!prop || !prop->name || !node) {
+      /* something wrong with the SGF file properties */
+      if (1)
+	fprintf(stderr, "Something wrong with the SGF file properties.\n");
+      return 0;
+    }
+    switch (prop->name) {
     case SGFAB:
       get_moveXY(prop, &i, &j, board_size);
       /* Put handicap stones on the board at once. */
-      add_stone(i, j, BLACK);
+      add_stone(POS(i, j), BLACK);
       break;
       
     case SGFAW:
-      fprintf(stderr, "Warning: white stone added.\n");
+      /* fprintf(stderr, "Warning: white stone added.\n"); */
       return 0;
       break;
       
     case SGFPL:
-      fprintf(stderr, "Warning: PL property encountered.\n");
+      /* fprintf(stderr, "Warning: PL property encountered.\n"); */
       return 0;
       break;
       
@@ -362,14 +414,15 @@ get_move_from_sgf(SGFNode *node, int *m, int *n, int *color)
       *color = (prop->name == SGFW) ? WHITE : BLACK;
       
       if (!get_moveXY(prop, m, n, board_size)) {
-	fprintf(stderr, "Warning: failed to get move coordinates.\n");
+	if (0)
+	  fprintf(stderr, "Warning: failed to get move coordinates.\n");
 	return 0;
       }
       return 1;
       break;
     }
   }
-
+  
   return 0;
 }
 
@@ -392,7 +445,7 @@ compare_situations(const void *a, const void *b)
   const struct situation *aa = a;
   const struct situation *bb = b;
   int k;
-
+  
   for (k = 0; k < 8; k++) {
     if (aa->pre.values[k] > bb->pre.values[k])
       return 1;
@@ -406,7 +459,7 @@ compare_situations(const void *a, const void *b)
     if (aa->post.values[k] < bb->post.values[k])
       return -1;
   }
-
+  
   return 0;
 }
 
@@ -419,7 +472,7 @@ compare_positions(const void *a, const void *b)
   const struct situation *aa = a;
   const struct situation *bb = b;
   int k;
-
+  
   for (k = 0; k < 8; k++) {
     if (aa->pre.values[k] > bb->pre.values[k])
       return 1;
@@ -439,16 +492,50 @@ compare_frequencies(const void *a, const void *b)
 {
   const struct frequency *aa = a;
   const struct frequency *bb = b;
-
+  
   if (aa->n < bb->n)
-      return 1;
+    return 1;
   
   if (aa->n > bb->n)
-      return -1;
+    return -1;
   
   return 0;
 }
 
+/*
+ * find_region answers in what region the move is
+ * there are 9 regions, corners, sides and center
+ */
+
+static int
+find_region(int m, int n)
+{
+  if (m < 7) {
+    if (n < 7)
+      return 0;
+    else if (n > 11)
+      return 1;
+    else if (n > 6 && m < 5)
+      return 6;
+  }
+  else if (m > 11) {
+    if (n < 7)
+      return 2;
+    else if (n > 11)
+      return 3;
+    else if (n > 6 && m > 13)
+      return 7;
+  }
+  else if (m > 6) {
+    if (n < 5)
+      return 4;
+    else if (n > 13)
+      return 5;
+  }
+  /* otherwise in center */
+  return 8;
+}
+
 /* If this situation is listed among the winners, fill in the pattern
  * entry of the winner struct.
  */
@@ -459,23 +546,76 @@ store_pattern_if_winner(struct invariant_hash *pre,
 {
   int k;
   struct situation s;
+  int region = 8;
+  int i, j;
+  int move_number = 1;
   s.pre = *pre;
   s.post = *post;
+
   for (k = 0; k < number_of_winning_moves; k++) {
     if (compare_situations(&situation_table[winning_moves[k].index],
 			   &s) == 0) {
       /* This is a winner. Record the pattern. */
-      int i,j;
       for (i = 0; i < board_size; i++)
 	for (j = 0; j < board_size; j++) {
-	  if (p[i][j] == EMPTY)
+	  if (BOARD(i, j) == EMPTY)
 	    winning_moves[k].pattern[i][j] = '.';
-	  else if (p[i][j] == color)
+	  else if (BOARD(i, j) == color) {
 	    winning_moves[k].pattern[i][j] = 'O';
-	  else
+	    move_number++;
+	  }
+	  else if ((color == WHITE && BOARD(i, j) == BLACK)
+		   || (color == BLACK && BOARD(i, j) == WHITE)) {
 	    winning_moves[k].pattern[i][j] = 'X';
+	    move_number++;
+	  }
+	  else { /* something is wrong */
+	    fprintf(stderr, "Error in store_pattern_if_winner: %d\n",k);
+	    winning_moves[k].pattern[i][j] = '.';
+	  }
 	}
       winning_moves[k].pattern[m][n] = '*';
+      /* add ? in areas far away from the move */
+      if (half_board_patterns == 1 && move_number > 3 && board_size == 19)
+        region = find_region(m,n);
+      if (region != 8) {
+        for (i = 0; i < board_size; i++) {
+          for (j = 0; j < board_size; j++) {
+            if (region == 0) {
+              if (i + j > 23)
+     	        winning_moves[k].pattern[i][j] = '?';
+     	    }
+            else if (region == 1) {
+              if ((i - j) > 5)
+ 	        winning_moves[k].pattern[i][j] = '?';
+ 	    }
+            else if (region == 2) {
+              if ((i + board_size - j) < 14)
+ 	        winning_moves[k].pattern[i][j] = '?';
+ 	    }
+ 	    else if (region == 3) {
+              if (i + j < 13)
+ 	        winning_moves[k].pattern[i][j] = '?';
+ 	    }
+            else if (region == 4) {
+              if (j > 10)
+ 	        winning_moves[k].pattern[i][j] = '?';
+ 	    }
+            else if (region == 5) {
+              if (j < 8)
+ 	        winning_moves[k].pattern[i][j] = '?';
+ 	    }
+            else if (region == 6) {
+              if (i > 10)
+ 	        winning_moves[k].pattern[i][j] = '?';
+ 	    }
+            else if (region == 7) {
+              if (i < 8)
+ 	        winning_moves[k].pattern[i][j] = '?';
+ 	    }
+          }
+        }
+      }
     }
   }
 }
@@ -496,14 +636,21 @@ examine_game(SGFNode *sgf, int collect_statistics)
   struct invariant_hash prehash;
   struct invariant_hash posthash;
   int color;
-
+  
   /* Call the engine to clear the board. */
   clear_board();
-
+  
   /* Loop through the first moves_per_game moves of each game. */
   for (k = 0; k < moves_per_game && node != NULL; node = node->child) {
-    if (!get_move_from_sgf(node, &m, &n, &color))
+    if (!get_move_from_sgf(node, &m, &n, &color)) {
+      if (k > 0) {
+	/* something is wrong with the file */
+	if (0)
+	  fprintf(stderr, "move number:%d\n",k);
+	return 0;
+      }
       continue;
+    }
     gg_assert(m >= 0 && m < board_size && n >= 0 && n <= board_size);
     hash_board(&prehash, color);
     hash_board_and_move(&posthash, color, m, n);
@@ -511,8 +658,8 @@ examine_game(SGFNode *sgf, int collect_statistics)
       add_situation(&prehash, &posthash);
     else
       store_pattern_if_winner(&prehash, &posthash, color, m, n);
-    play_move(m, n, color);
-
+    play_move(POS(m, n), color);
+    
     /* Debug output. */
     if (0) {
       int l;
@@ -524,16 +671,141 @@ examine_game(SGFNode *sgf, int collect_statistics)
       fprintf(stderr, "\n");
       showboard(0);
     }
-
     k++;
   }
-
-  if (!node)
+  if (!node) {
+    if (0)
+      fprintf(stderr, "Node error\n");
     return 0;
+  }
   
   return 1;
 }
 
+/* Tests if the player has enough strength in the game to be interesting 
+ * for the library
+ */
+  
+static int
+enough_strength(char *strength)
+{
+  int length = 0;
+  int i = 0;
+  int kyu = 30;
+  if (player_strength >= 30)
+    return 1;
+  
+  length = strlen(strength);
+  /* check if dan player */
+  for (i = 0; i < length; i++)
+    if (strength[i] == 'd')
+      return 1;
+  
+  /* get the kyu strength as an integer */
+  for (i = 0; i < length; i++) {
+    if (strength[i] == 'k')
+      strength[i] = '\0';
+    kyu = atoi(strength);
+  }
+  
+  if (kyu <= player_strength)
+    return 1;
+
+  /* not enough strength */
+  return 0;
+}
+
+/*
+ * Sort out the games that can be used
+ */
+
+static void
+sort_games(void)
+{
+  int k;
+  int handicap;
+  char *WR; /* white rank */
+  char *BR; /* black rank */
+  
+  for (k = 0; k < number_of_games; k++) {
+    int size;
+    SGFNode *sgf;
+    
+    /* Progress output. */
+    if (k % 500 == 0)
+      fprintf(stderr, "Sorting number %d, %s\n", k, sgf_names[k]);
+    
+    sgf = readsgffilefuseki(sgf_names[k], 0);
+    
+    if (!sgf) {
+      if (0)
+	fprintf(stderr, "Warning: Couldn't open sgf file %s.\n", sgf_names[k]);
+      unused_games[k] = 1; /* the game could not be used */
+      continue;
+    }
+    
+    else if (!sgfGetIntProperty(sgf, "SZ", &size) || size != board_size) {
+      if (0)
+	fprintf(stderr, "Warning: wrong size of board %d.\n", size);
+      unused_games[k] = 1; /* the game could not be used */
+      continue; /* Board of wrong size, ignore the game. */
+    }
+    
+    /* No handicap games */
+    else if (handicap_value == 0) {
+      if (sgfGetIntProperty(sgf, "HA", &handicap) && handicap > 1) {
+	if (0)
+	  fprintf(stderr, "No handicap games allowed %d\n", handicap);
+	unused_games[k] = 1; /* the game could not be used */
+	continue;
+      }
+    }
+    
+    /* Only handicap games */
+    else if (handicap_value > 1) {
+      if (!sgfGetIntProperty(sgf, "HA", &handicap)) {
+	if (0)
+	  fprintf(stderr, "Not a handicap game %d\n", handicap);
+	unused_games[k] = 1; /* the game could not be used */
+	continue;
+      }
+      
+      /* only specific handicap games */
+      else if (handicap_value != 10 && handicap != handicap_value) {
+	if (0)
+	  fprintf(stderr, "Wrong number of handicap stones %d\n", handicap);
+	unused_games[k] = 1; /* the game could not be used */
+	continue;
+      }
+    }
+    
+    if (unused_games[k] == 0) {
+      /* examine strength of players in the game and compare it 
+       * with minimum player strength 
+       */
+      if (sgfGetCharProperty(sgf, "BR", &BR) && !enough_strength(BR)) {
+	if (0)
+	  fprintf(stderr, "Wrong strength: %s.\n", BR);
+	unused_games[k] = 1; /* the game could not be used */
+	continue;
+      }
+      
+      /* examine strength of players in the game and compare it with 
+       * minimum player strength */
+      else if (sgfGetCharProperty(sgf, "WR", &WR) && !enough_strength(WR)) {
+	if (0)
+	  fprintf(stderr, "Wrong strength: %s.\n", WR);
+	unused_games[k] = 1; /* the game could not be used */
+	continue;
+      }
+    }
+    
+    /* Free memory of SGF file */
+    sgfFreeNode(sgf);
+  }
+}
+
+
 /* Play through the initial moves of all games and collect hash values
  * for the encountered situations.
  */
@@ -541,26 +813,88 @@ static void
 collect_situations(void)
 {
   int k;
+  int handicap;
+  char *WR; /* white rank */
+  char *BR; /* black rank */
+  
   init_situations();
   for (k = 0; k < number_of_games; k++) {
     int size;
     SGFNode *sgf;
-
-    /* Debug output. */
-    if (0)
-      fprintf(stderr, "Reading %s\n", sgf_names[k]);
-
-    sgf = readsgffile(sgf_names[k]);
+    
+    /* Progress output. */
+    if (k % 500 == 0)
+      fprintf(stderr, "Reading number %d, %s\n", k, sgf_names[k]);
+    
+    sgf = readsgffilefuseki(sgf_names[k], moves_per_game);
+    
     if (!sgf) {
-      fprintf(stderr, "Warning: Couldn't open sgf file %s.\n", sgf_names[k]);
+      if (0)
+	fprintf(stderr, "Warning: Couldn't open sgf file %s.\n", sgf_names[k]);
+      unused_games[k] = 1; /* the game could not be used */
       continue;
     }
     
-    if (!sgfGetIntProperty(sgf, "SZ", &size) || size != board_size)
+    else if (!sgfGetIntProperty(sgf, "SZ", &size) || size != board_size) {
+      if (0)
+	fprintf(stderr, "Warning: wrong size of board %d.\n", size);
+      unused_games[k] = 1; /* the game could not be used */
       continue; /* Board of wrong size, ignore the game. */
-
-    if (!examine_game(sgf, 1))
-      fprintf(stderr, "Warning: Problem with sgf file %s.\n", sgf_names[k]);
+    }
+    
+    /* No handicap games */
+    else if (handicap_value == 0) {
+      if (sgfGetIntProperty(sgf, "HA", &handicap) && handicap > 1) {
+	if (0)
+	  fprintf(stderr, "No handicap games allowed %d\n", handicap);
+	unused_games[k] = 1; /* the game could not be used */
+	continue;
+      }
+    }
+    
+    /* Only handicap games */
+    else if (handicap_value > 1) {
+      if (!sgfGetIntProperty(sgf, "HA", &handicap)) {
+	if (0)
+	  fprintf(stderr, "Not a handicap game %d\n", handicap);
+	unused_games[k] = 1; /* the game could not be used */
+	continue;
+      }
+      
+      /* only specific handicap games */
+      else if (handicap_value != 10 && handicap != handicap_value) {
+	if (0)
+	  fprintf(stderr, "Wrong number of handicap stones %d\n", handicap);
+	unused_games[k] = 1; /* the game could not be used */
+	continue;
+      }
+    }
+    
+    /* examine strength of players in the game and compare it with 
+     * minimum player strength */
+    if (sgfGetCharProperty(sgf, "BR", &BR) && !enough_strength(BR)) {
+      if (0)
+	fprintf(stderr, "Wrong strength: %s.\n", BR);
+      unused_games[k] = 1; /* the game could not be used */
+      continue;
+    }
+    /* examine strength of players in the game and compare it with 
+     * minimum player strength */
+    else if (sgfGetCharProperty(sgf, "WR", &WR) && !enough_strength(WR)) {
+      if (0)
+	fprintf(stderr, "Wrong strength: %s.\n", WR);
+      unused_games[k] = 1; /* the game could not be used */
+      continue;
+    }
+    
+    if (!examine_game(sgf, 1)) {
+      if (0)
+	fprintf(stderr, "Warning: Problem with sgf file %s.\n", sgf_names[k]);
+      unused_games[k] = 1; /* the game could not be used */
+    }
+    
+    /* Free memory of SGF file */
+    sgfFreeNode(sgf);
   }
 }
 
@@ -572,9 +906,9 @@ analyze_statistics(void)
 {
   int k;
   /* Sort all the collected situations. */
-  qsort(situation_table, number_of_situations, sizeof(*situation_table),
-	compare_situations);
-
+  gg_sort(situation_table, number_of_situations, sizeof(*situation_table),
+	  compare_situations);
+  
   /* Debug output. */
   if (0) {
     int i, k;
@@ -588,7 +922,7 @@ analyze_statistics(void)
       fprintf(stderr, "\n");
     }
   }
-
+  
   /* Set up frequency table. */
   frequency_table = calloc(number_of_situations, sizeof(*frequency_table));
   if (!frequency_table) {
@@ -607,11 +941,11 @@ analyze_statistics(void)
     }
     frequency_table[number_of_distinct_positions-1].n++;
   }
-
+  
   /* Sort the frequency table, in falling order. */
-  qsort(frequency_table, number_of_distinct_positions,
-	sizeof(*frequency_table), compare_frequencies);
-
+  gg_sort(frequency_table, number_of_distinct_positions,
+	  sizeof(*frequency_table), compare_frequencies);
+  
   /* Debug output. */
   if (0) {
     int l;
@@ -620,7 +954,7 @@ analyze_statistics(void)
 	      frequency_table[l].index);
     }
   }
-
+  
   /* Set up winners array. */
   winning_moves = calloc(patterns_to_extract, sizeof(*winning_moves));
   if (!winning_moves) {
@@ -637,18 +971,18 @@ analyze_statistics(void)
   for (k = 0; k < number_of_distinct_positions; k++) {
     int index = frequency_table[k].index;
     int i;
-
+    
     /* Build a new frequency table for the different moves in this position. */
     struct frequency move_frequencies[MAX_BOARD * MAX_BOARD];
     int number_of_moves = 0;
-    for (i=index; ;i++) {
+    for (i = index; ;i++) {
       if (i == number_of_situations
 	  || (i > index
 	      && compare_positions(&situation_table[i],
 				   &situation_table[i-1]) != 0))
 	break;
-      if (i==index || compare_situations(&situation_table[i],
-					 &situation_table[i-1]) != 0) {
+      if (i == index || compare_situations(&situation_table[i],
+					   &situation_table[i-1]) != 0) {
 	move_frequencies[number_of_moves].index = i;
 	move_frequencies[number_of_moves].n = 0;
 	number_of_moves++;
@@ -657,9 +991,9 @@ analyze_statistics(void)
     }
     
     /* Sort the moves, in falling order. */
-    qsort(move_frequencies, number_of_moves,
-	  sizeof(*move_frequencies), compare_frequencies);
-
+    gg_sort(move_frequencies, number_of_moves,
+	    sizeof(*move_frequencies), compare_frequencies);
+    
     /* Debug output. */
     if (0) {
       for (i = 0; i < number_of_moves; i++) {
@@ -667,12 +1001,17 @@ analyze_statistics(void)
 		move_frequencies[i].index);
       }
     }
-
+    
     /* Add the moves to the list of winners. */
     for (i = 0; i < number_of_moves; i++) {
+      /* This is where the cut-off of moves is decided */
       if (10 * move_frequencies[i].n < move_frequencies[0].n
 	  && move_frequencies[i].n < 10)
 	break;
+      /* Take away any move that hasn't been made by at least 2 people */
+      if (move_frequencies[i].n < 2)
+	break;
+      
       winning_moves[number_of_winning_moves].index = move_frequencies[i].index;
       winning_moves[number_of_winning_moves].position_frequency =
 	frequency_table[k].n;
@@ -687,7 +1026,7 @@ analyze_statistics(void)
     if (number_of_winning_moves == patterns_to_extract)
       break;
   }
-
+  
   /* Debug output. */
   if (0) {
     int i;
@@ -707,16 +1046,30 @@ static void
 generate_patterns(void)
 {
   int k;
+  SGFNode *sgf;
   for (k = 0; k < number_of_games; k++) {
-    int size;
-    SGFNode *sgf = readsgffile(sgf_names[k]);
+    
+    /* Progress output. */
+    if (k % 500 == 0)
+      fprintf(stderr, "Generating number %d, %s\n", k, sgf_names[k]);
+    
+    /* Check if this game is a valid game */
+    if (unused_games[k]) {
+      if (0)
+	fprintf(stderr, "Not used\n");
+      continue;
+    }
+    
+    sgf = readsgffilefuseki(sgf_names[k], moves_per_game);
     if (!sgf) {
       fprintf(stderr, "Warning: Couldn't open sgf file %s.\n", sgf_names[k]);
       continue;
     }
-    if (!sgfGetIntProperty(sgf, "SZ", &size) || size != board_size)
-      continue; /* Board of wrong size, ignore the game. */
+    
     (void) examine_game(sgf, 0);
+    
+    /* Free memory of SGF file */
+    sgfFreeNode(sgf);
   }
 }
 
@@ -724,36 +1077,57 @@ generate_patterns(void)
 static void
 print_patterns(void)
 {
-  int k;
+  int k, l;
   int m, n;
+  l = 1;
   for (k = 0; k < number_of_winning_moves; k++) {
-    printf("Pattern Fuseki%d\n", k+1);
-    printf("# %d/%d\n\n", winning_moves[k].move_frequency,
-	   winning_moves[k].position_frequency);
-    for (m = 0; m < board_size; m++) {
-      printf("|");
+    
+    /* do not print errornous patterns */
+    if (winning_moves[k].pattern[0][0] != '\0') {
+      printf("Pattern Fuseki%d\n", l);
+      printf("# %d/%d\n\n", 
+	     winning_moves[k].move_frequency,
+	     winning_moves[k].position_frequency);
+      printf("+");
       for (n = 0; n < board_size; n++) {
-	printf("%c", winning_moves[k].pattern[m][n]);
+	printf("-");
       }
-      printf("\n");
+      printf("+\n");
+      for (m = 0; m < board_size; m++) {
+	printf("|");
+	for (n = 0; n < board_size; n++) {
+	  if (winning_moves[k].pattern[m][n] == '\0') {
+	    fprintf(stderr, "Something wrong in print pattern\n");
+	    printf(".");
+	  }
+	  else
+	    printf("%c", winning_moves[k].pattern[m][n]);
+	}
+	printf("|\n");
+      }
+      printf("+");
+      for (n = 0; n < board_size; n++) {
+	printf("-");
+      }
+      printf("+");
+      printf("\n\n:8,-,value(%d)\n\n\n", winning_moves[k].move_frequency);
+      l++;
     }
-    printf("+");
-    for (n = 0; n < board_size; n++) {
-      printf("-");
-    }
-    printf("\n\n:8,-,value(%d)\n\n\n", winning_moves[k].move_frequency);
   }
 }
 
 int
 main(int argc, char *argv[])
 {
+  int number_of_unused_games = 0;
+  int i = 0;
+  
   /* Check number of arguments. */
-  if (argc < 5) {
+  if (argc < 6) {
     fprintf(stderr, USAGE);
     exit(EXIT_FAILURE);
   }
-
+  
   /* Check arguments. */
   board_size = atoi(argv[2]);
   if (board_size % 2 == 0) {
@@ -768,15 +1142,37 @@ main(int argc, char *argv[])
   if (moves_per_game < 1 || moves_per_game > 20)
     fprintf(stderr, "Warning: strange number of moves per game: %d.\n",
 	    moves_per_game);
-
+  
   patterns_to_extract = atoi(argv[4]);
   if (patterns_to_extract < 1)
     fprintf(stderr, "Warning: strange number of patterns to extract: %d.\n",
 	    patterns_to_extract);
-
+  
+  handicap_value = atoi(argv[5]);
+  if (handicap_value < 0 || handicap_value > 2)
+    fprintf(stderr, "Warning: wrong handicap value: %d.\n",
+	    handicap_value);
+  
+  player_strength = atoi(argv[6]);
+  if (player_strength < 0 || player_strength > 30)
+    fprintf(stderr, "Warning: wrong lowest strength: %d.\n",
+	    player_strength);
+  
+  half_board_patterns = atoi(argv[7]);
+  if (half_board_patterns != 0 && half_board_patterns != 1) {
+    fprintf(stderr, "Warning: incorrect half_board_flag (0 or 1). Setting the value to 0.\n");
+    half_board_patterns = 0;
+  }
   /* Count the number of sgf files. */
   number_of_games = read_sgf_filenames(argv[1], NULL);
-
+  
+  /* Allocate space for the list of unused files. */
+  unused_games = calloc(number_of_games, sizeof(int));
+  if (unused_games == NULL) {
+    fprintf(stderr, "Fatal error, failed to allocate memory.\n");
+    exit(EXIT_FAILURE);
+  }
+  
   /* Allocate space for the list of sgf file names. */
   sgf_names = calloc(number_of_games, sizeof(*sgf_names));
   if (sgf_names == NULL) {
@@ -786,28 +1182,47 @@ main(int argc, char *argv[])
   
   /* Read the list of sgf files and store in memory. */
   (void) read_sgf_filenames(argv[1], sgf_names);
-
-  /* Build tables of random numbers for Zobrist hashing. */
-  init_zobrist_numbers();
   
-  /* Play through the initial moves of all games and collect hash values
-   * for the encountered situations.
-   */
-  collect_situations();
-
-  /* Find the most common positions and moves, for which we want to
-   * generate patterns.
-   */
-  analyze_statistics();
-
-  /* Generate patterns from the chosen positions and moves.
-   */
-  generate_patterns();
-
-  /* Write the patterns to stdout in patterns.db format.
-   */
-  print_patterns();
-
+  /* Save memory by sorting out the games that can be used first */
+  if (argv[8] != NULL) {
+    sort_games();
+    write_sgf_filenames(argv[8], sgf_names);
+  }
+  
+  else {
+    /* Build tables of random numbers for Zobrist hashing. */
+    init_zobrist_numbers();
+    
+    /* Play through the initial moves of all games and collect hash values
+     * for the encountered situations.
+     */
+    collect_situations();
+    fprintf(stderr, "collect OK.\n");
+    
+    /* Find the most common positions and moves, for which we want to
+     * generate patterns.
+     */
+    analyze_statistics();
+    fprintf(stderr, "analyze OK.\n");
+    
+    /* Generate patterns from the chosen positions and moves.
+     */
+    generate_patterns();
+    fprintf(stderr, "generate OK.\n");
+    
+    /* Write the patterns to stdout in patterns.db format.
+     */
+    print_patterns();
+    
+    /* Tell the user everything worked out fine */
+    fprintf(stderr, "The pattern database was produced with no errors.\n");
+    for (i = 0; i < number_of_games; i++) {
+      if (unused_games[i]) number_of_unused_games++;
+    }
+    fprintf(stderr, "Out of %d games, %d were not used.\n", 
+	    number_of_games, number_of_unused_games);
+  }
+  
   return 0;
 }
 
