@@ -1,24 +1,23 @@
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
- * http://www.gnu.org/software/gnugo/ for more information.      *
- *                                                               *
- * Copyright 1999, 2000, 2001 by the Free Software Foundation.   *
- *                                                               *
- * This program is free software; you can redistribute it and/or *
- * modify it under the terms of the GNU General Public License   *
- * as published by the Free Software Foundation - version 2.     *
- *                                                               *
- * This program is distributed in the hope that it will be       *
- * useful, but WITHOUT ANY WARRANTY; without even the implied    *
- * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
- * PURPOSE.  See the GNU General Public License in file COPYING  *
- * for more details.                                             *
- *                                                               *
- * You should have received a copy of the GNU General Public     *
- * License along with this program; if not, write to the Free    *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
- * Boston, MA 02111, USA.                                        *
-\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
+ * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * http://www.gnu.org/software/gnugo/ for more information.          *
+ *                                                                   *
+ * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ *                                                                   *
+ * This program is free software; you can redistribute it and/or     *
+ * modify it under the terms of the GNU General Public License as    *
+ * published by the Free Software Foundation - version 2             *
+ *                                                                   *
+ * This program is distributed in the hope that it will be useful,   *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
+ * GNU General Public License in file COPYING for more details.      *
+ *                                                                   *
+ * You should have received a copy of the GNU General Public         *
+ * License along with this program; if not, write to the Free        *
+ * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Boston, MA 02111, USA.                                            *
+\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 /* Convert joseki from sgf format to patterns.db format. */
 
@@ -31,6 +30,10 @@
 #include "../sgf/sgftree.h"
 #include "liberty.h"
 
+#ifndef VARIADIC_MACROS
+int trace_dummy;
+#endif
+
 #define USAGE "\
 Usage : joseki prefix filename\n\
 "
@@ -133,16 +136,16 @@ write_diagram(int movei, int movej, int color, int marki, int markj,
   int i, j;
   
   for (i = -1; i <= marki; i++) {
-    for (j=markj; j<board_size; j++) {
+    for (j = markj; j < board_size; j++) {
       if (i == -1)
 	printf("-");
       else if (labels && labels[i][j])
 	printf("%c", labels[i][j]);
       else if (i == movei && j == movej)
 	printf("*");
-      else if (p[i][j] == color)
+      else if (BOARD(i, j) == color)
 	printf("O");
-      else if (p[i][j] == OTHER_COLOR(color))
+      else if (BOARD(i, j) == OTHER_COLOR(color))
 	printf("X");
       else
 	printf(".");
@@ -171,7 +174,7 @@ write_colon_line(int move_type, char *text)
       p += 2;
   }
 
-  printf(":8,s");
+  printf(":8,sF");
   switch (move_type) {
   case URGENT:
     printf("U");
@@ -237,24 +240,29 @@ make_pattern(int movei, int movej, int color,
    */
   if (labels
       || selected_line_exists(text, ';')
-      || selected_line_exists(text, '>'))
+      || selected_line_exists(text, '>')) {
     write_diagram(movei, movej, color, marki, markj, labels);
 
-  printf("\n");
+    printf("\n");
 
-  /* Write constraint and action lines. */
-  write_selected_lines(text, ';');
-  write_selected_lines(text, '>');
-  if (move_type == ANTISUJI)
-    printf(">antisuji(*);\n");
-  printf("\n\n");
+    /* Write constraint and action lines. */
+    write_selected_lines(text, ';');
+    write_selected_lines(text, '>');
+    if (move_type == ANTISUJI)
+      printf(">antisuji(*);\n");
+    printf("\n");
+  }
+  else if (move_type == ANTISUJI)
+    printf(">antisuji(*);\n\n");
+
+  printf("\n");
   
   /* Basic sanity checking. We do this at the end to simplify debugging. */
   if (multiple_marks)
     fprintf(stderr, "Warning: Multiple square marks in pattern %s%d\n",
 	    prefix, pattern_number);
 
-  if (is_suicide(movei, movej, color)) {
+  if (is_suicide(POS(movei, movej), color)) {
     fprintf(stderr, "Error: Illegal move in pattern %s%d\n",
 	    prefix, pattern_number);
     exit(EXIT_FAILURE);
@@ -287,7 +295,7 @@ analyze_node(SGFNode *node, const char *prefix)
    * a comment.
    */
   for (prop = node->props; prop; prop = prop->next) {
-    switch(prop->name) {
+    switch (prop->name) {
     case SGFSQ: /* Square */
     case SGFMA: /* Mark */
       if (marki != -1)
@@ -328,7 +336,7 @@ analyze_node(SGFNode *node, const char *prefix)
   /* Traverse child, if any. */
   if (node->child) {
     if (movei != -1)
-      tryko(movei, movej, color, NULL, EMPTY, -1, -1);
+      tryko(POS(movei, movej), color, NULL, EMPTY, NO_MOVE);
     analyze_node(node->child, prefix);
     if (movei != -1)
       popgo();
@@ -363,30 +371,29 @@ main(int argc, char *argv[])
   }
 
 #define PREAMBLE "\
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #\n\
-# This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   #\n\
-# http://www.gnu.org/software/gnugo/ for more information.      #\n\
-#                                                               #\n\
-# Copyright 1999, 2000, 2001 by the Free Software Foundation.   #\n\
-#                                                               #\n\
-# This program is free software; you can redistribute it and/or #\n\
-# modify it under the terms of the GNU General Public License   #\n\
-# as published by the Free Software Foundation - version 2.     #\n\
-#                                                               #\n\
-# This program is distributed in the hope that it will be       #\n\
-# useful, but WITHOUT ANY WARRANTY; without even the implied    #\n\
-# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       #\n\
-# PURPOSE.  See the GNU General Public License in file COPYING  #\n\
-# for more details.                                             #\n\
-#                                                               #\n\
-# You should have received a copy of the GNU General Public     #\n\
-# License along with this program; if not, write to the Free    #\n\
-# Software Foundation, Inc., 59 Temple Place - Suite 330,       #\n\
-# Boston, MA 02111, USA.                                        #\n\
-#                                                               #\n\
-# This file is automatically generated by joseki. Do not edit   #\n\
-# It directly. Instead, edit the corresponding database.        #\n\
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #\n\
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #\n\
+# This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       #\n\
+# http://www.gnu.org/software/gnugo/ for more information.          #\n\
+#                                                                   #\n\
+# Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. #\n\
+#                                                                   #\n\
+# This program is free software; you can redistribute it and/or     #\n\
+# modify it under the terms of the GNU General Public License as    #\n\
+# published by the Free Software Foundation - version 2             #\n\
+#                                                                   #\n\
+# This program is distributed in the hope that it will be useful,   #\n\
+# but WITHOUT ANY WARRANTY; without even the implied warranty of    #\n\
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     #\n\
+# GNU General Public License in file COPYING for more details.      #\n\
+#                                                                   #\n\
+# You should have received a copy of the GNU General Public         #\n\
+# License along with this program; if not, write to the Free        #\n\
+# Software Foundation, Inc., 59 Temple Place - Suite 330,           #\n\
+# Boston, MA 02111, USA.                                            #\n\
+#                                                                   #\n\
+# This file is automatically generated by joseki. Do not edit       #\n\
+# it directly. Instead, edit the corresponding sgf file.            #\n\
+# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #\n\
 \n\n\
 "
 
