@@ -160,7 +160,7 @@
 /* # */
 
 /* # */
-/* # outdated FF3 paramaters */
+/* # outdated FF3 properties */
 /* # */
 #define SGFBS 21314
 #define SGFWS 21335
@@ -169,6 +169,11 @@
 #define SGFSC 17235
 
 /* # */
-/* # and some random ones used by CGoban */
+/* # some random ones used by CGoban */
 /* # */
 #define SGFSY 22867
+
+/* # */
+/* # nonstandard SGF property used by GNU Go to mark illegal moves */
+/* # */
+#define SGFIL 19529
