@@ -1,25 +1,23 @@
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
- * http://www.gnu.org/software/gnugo/ for more information.      *
- *                                                               *
- * Copyright 1999, 2000, 2001 by the Free Software Foundation.   *
- *                                                               *
- * This program is free software; you can redistribute it and/or *
- * modify it under the terms of the GNU General Public License   *
- * as published by the Free Software Foundation - version 2.     *
- *                                                               *
- * This program is distributed in the hope that it will be       *
- * useful, but WITHOUT ANY WARRANTY; without even the implied    *
- * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
- * PURPOSE.  See the GNU General Public License in file COPYING  *
- * for more details.                                             *
- *                                                               *
- * You should have received a copy of the GNU General Public     *
- * License along with this program; if not, write to the Free    *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
- * Boston, MA 02111, USA                                         *
-\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
-
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
+ * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * http://www.gnu.org/software/gnugo/ for more information.          *
+ *                                                                   *
+ * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ *                                                                   *
+ * This program is free software; you can redistribute it and/or     *
+ * modify it under the terms of the GNU General Public License as    *
+ * published by the Free Software Foundation - version 2             *
+ *                                                                   *
+ * This program is distributed in the hope that it will be useful,   *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
+ * GNU General Public License in file COPYING for more details.      *
+ *                                                                   *
+ * You should have received a copy of the GNU General Public         *
+ * License along with this program; if not, write to the Free        *
+ * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Boston, MA 02111, USA.                                            *
+\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -37,7 +35,7 @@ int
 get_moveX(SGFProperty *property, int boardsize)
 {
   int i;
-  if (!property->value[1])
+  if (strlen(property->value) < 2)
     return -1;
 
   i = toupper((int) property->value[1]) - 'A';
@@ -55,7 +53,7 @@ int
 get_moveY(SGFProperty *property, int boardsize)
 {
   int j;
-  if (!property->value[0])
+  if (strlen(property->value) < 2)
     return -1;
 
   j = toupper((int) property->value[0]) - 'A';
@@ -79,7 +77,10 @@ get_moveXY(SGFProperty *property, int *i, int *j, int boardsize)
   *i = get_moveX(property, boardsize);
   *j = get_moveY(property, boardsize);
   
-  return !is_pass(*i, *j);
+  if (*i == -1 && *j == -1)
+    return 0;
+
+  return 1;
 }
 
 
@@ -93,7 +94,7 @@ show_sgf_properties(SGFNode *node)
   SGFProperty *sgf_prop;
   int propcount;
 
-  propcount=0;
+  propcount = 0;
 
   printf("P: ");
   if (!node->props) {
