@@ -1,24 +1,23 @@
-/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
- * http://www.gnu.org/software/gnugo/ for more information.      *
- *                                                               *
- * Copyright 1999, 2000, 2001 by the Free Software Foundation.   *
- *                                                               *
- * This program is free software; you can redistribute it and/or *
- * modify it under the terms of the GNU General Public License   *
- * as published by the Free Software Foundation - version 2.     *
- *                                                               *
- * This program is distributed in the hope that it will be       *
- * useful, but WITHOUT ANY WARRANTY; without even the implied    *
- * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
- * PURPOSE.  See the GNU General Public License in file COPYING  *
- * for more details.                                             *
- *                                                               *
- * You should have received a copy of the GNU General Public     *
- * License along with this program; if not, write to the Free    *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
- * Boston, MA 02111, USA.                                        *
-\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
+ * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * http://www.gnu.org/software/gnugo/ for more information.          *
+ *                                                                   *
+ * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ *                                                                   *
+ * This program is free software; you can redistribute it and/or     *
+ * modify it under the terms of the GNU General Public License as    *
+ * published by the Free Software Foundation - version 2             *
+ *                                                                   *
+ * This program is distributed in the hope that it will be useful,   *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     *
+ * GNU General Public License in file COPYING for more details.      *
+ *                                                                   *
+ * You should have received a copy of the GNU General Public         *
+ * License along with this program; if not, write to the Free        *
+ * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Boston, MA 02111, USA.                                            *
+\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 /*  Parts of this code were given to us by Tommy Thorn */
 
@@ -32,7 +31,6 @@
 #include <string.h>
 #include <assert.h>
 
-#include "liberty.h"
 
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -48,7 +46,11 @@
 #include "sgftree.h"
 #include "gg_utils.h"
 
+#define STRICT_SGF 's'
+#define LAX_SGF    'l'
 
+/* Set this to 1 if you want warnings for missing GM and FF properties. */
+#define VERBOSE_WARNINGS 0
 
 /* ================================================================ */
 /*                     Some utility functions.                      */
@@ -99,7 +101,28 @@ xrealloc(void *pt, unsigned int size)
 SGFNode *
 sgfNewNode()
 {
-  return xalloc(sizeof(SGFNode));
+  SGFNode *newnode;
+  newnode = xalloc(sizeof(SGFNode));
+  newnode->next = NULL;
+  newnode->props = NULL;
+  newnode->parent = NULL;
+  newnode->child = NULL;
+  return newnode;
+}
+
+/*
+ * Recursively free an sgf node
+ */
+
+void
+sgfFreeNode(SGFNode *node)
+{
+  if (node == NULL)
+    return;
+  sgfFreeNode(node->next);
+  sgfFreeNode(node->child);
+  sgfFreeProperty(node->props);
+  free(node);
 }
 
 
@@ -158,7 +181,7 @@ sgfGetIntProperty(SGFNode *node, const char *name, int *value)
   SGFProperty *prop;
   short nam = name[0] | name[1] << 8;
 
-  for (prop = node->props; prop; prop=prop->next)
+  for (prop = node->props; prop; prop = prop->next)
     if (prop->name == nam) {
       *value = atoi(prop->value);
       return 1;
@@ -220,7 +243,7 @@ sgfOverwriteProperty(SGFNode *node, const char *name, const char *text)
   SGFProperty *prop;
   short nam = name[0] | name[1] << 8;
 
-  for (prop = node->props; prop; prop=prop->next)
+  for (prop = node->props; prop; prop = prop->next)
     if (prop->name == nam) {
       prop->value = xrealloc(prop->value, strlen(text)+1);
       strcpy(prop->value, text);
@@ -333,6 +356,7 @@ sgfMkProperty(const char *name, const  char *value,
 
   prop = (SGFProperty *) xalloc(sizeof(SGFProperty));
   prop->value = xalloc(strlen(value)+1);
+  prop->next = NULL;
   if (strlen(name) == 1) 
     prop->name = name[0] | (short) (' ' << 8);
   else
@@ -347,6 +371,21 @@ sgfMkProperty(const char *name, const  char *value,
   return prop;
 }
 
+/*
+ * Recursively free an SGF property.
+ *
+ */
+
+void
+sgfFreeProperty(SGFProperty *prop)
+{
+  if (prop == NULL)
+    return;
+  sgfFreeProperty(prop->next);
+  free(prop->value);
+  free(prop);
+}
+
 
 /* ================================================================ */
 /*                        High level functions                      */
@@ -658,10 +697,6 @@ sgfAddChild(SGFNode *node)
  */
 
 
-#define STRICT_SGF 's'
-#define LAX_SGF    'l'
-
-
 static void parse_error(const char *msg, int arg);
 static void nexttoken(void);
 static void match(int expected);
@@ -749,7 +784,8 @@ propvalue(char *buffer, int size)
 	lookahead = sgf_getch();
 	if (lookahead == '\n') 
 	  lookahead = sgf_getch();
-      } else if (lookahead == '\n') {
+      }
+      else if (lookahead == '\n') {
 	lookahead = sgf_getch();
 	if (lookahead == '\r') 
 	  lookahead = sgf_getch();
@@ -850,6 +886,111 @@ gametree(SGFNode **p, SGFNode *parent, int mode)
 }
 
 
+/*
+ * Fuseki readers
+ * Reads an SGF file for extract_fuseki in a compact way
+ */
+
+static void
+gametreefuseki(SGFNode **p, SGFNode *parent, int mode, 
+	       int moves_per_game, int i)
+{
+  if (mode == STRICT_SGF)
+    match('(');
+  else
+    for (;;) {
+      if (lookahead == -1) {
+	parse_error("Empty file?", 0);
+	break;
+      }
+      if (lookahead == '(') {
+	while (lookahead == '(')
+	  nexttoken();
+	if (lookahead == ';')
+	  break;
+      }
+      nexttoken();
+    }
+  
+  /* The head is parsed */
+  {
+
+    SGFNode *head = sgfNewNode();
+    SGFNode *last;
+    head->parent = parent;
+    *p = head;
+    
+    last = sequence(head);
+    p = &last->child;
+    while (lookahead == '(') {
+      if (last->props 
+	  && (last->props->name == SGFB || last->props->name == SGFW))
+	i++;
+      /* break after number_of_moves moves in SGF file */
+      if (i >= moves_per_game) { 
+	last->child = NULL;
+	last->next = NULL;
+	break;
+      }
+      else {
+	gametreefuseki(p, last->parent, mode, moves_per_game, i);
+	p = &((*p)->next);
+      }
+    }
+    if (mode == STRICT_SGF)
+      match(')');
+  }
+}
+
+SGFNode *
+readsgffilefuseki(const char *filename, int moves_per_game)
+{
+  SGFNode *root;
+  int tmpi = 0;
+
+  if (strcmp(filename, "-") == 0)
+    sgffile = stdin;
+  else
+    sgffile = fopen(filename, "r");
+
+  if (!sgffile)
+    return NULL;
+
+
+  nexttoken();
+  gametreefuseki(&root, NULL, LAX_SGF, moves_per_game, 0);
+
+  fclose(sgffile);
+
+  if (sgferr) {
+    fprintf(stderr, "Parse error: %s at position %d\n", sgferr, sgferrpos);
+    return NULL;
+  }
+
+  /* perform some simple checks on the file */
+  if (!sgfGetIntProperty(root, "GM", &tmpi)) {
+    if (VERBOSE_WARNINGS)
+      fprintf(stderr, "Couldn't find the game type (GM) attribute!\n");
+  }
+  else if (tmpi != 1) {
+    fprintf(stderr, "SGF file might be for game other than go: %d\n", tmpi);
+    fprintf(stderr, "Trying to load anyway.\n");
+  }
+
+  if (!sgfGetIntProperty(root, "FF", &tmpi)) {
+    if (VERBOSE_WARNINGS)
+      fprintf(stderr, "Can not determine SGF spec version (FF)!\n");
+  }
+  else if ((tmpi < 3 || tmpi > 4) && VERBOSE_WARNINGS)
+    fprintf(stderr, "Unsupported SGF spec version: %d\n", tmpi);
+
+  return root;
+}
+
+
+
+
+
 /*
  * Wrapper around readsgf which reads from a file rather than a string.
  * Returns NULL if file will not open, or some other parsing error.
@@ -882,24 +1023,20 @@ readsgffile(const char *filename)
 
   /* perform some simple checks on the file */
   if (!sgfGetIntProperty(root, "GM", &tmpi)) {
-    fprintf(stderr, "Couldn't find the game type (GM) attribute!\n");
+    if (VERBOSE_WARNINGS)
+      fprintf(stderr, "Couldn't find the game type (GM) attribute!\n");
   }
-  if (tmpi != 1) {
+  else if (tmpi != 1) {
     fprintf(stderr, "SGF file might be for game other than go: %d\n", tmpi);
     fprintf(stderr, "Trying to load anyway.\n");
   }
 
-
-#if 0
-  if (debug & DEBUG_LOADSGF) {
-    if (!sgfGetIntProperty(root, "FF", &tmpi)) {
+  if (!sgfGetIntProperty(root, "FF", &tmpi)) {
+    if (VERBOSE_WARNINGS)
       fprintf(stderr, "Can not determine SGF spec version (FF)!\n");
-    }
-    if (tmpi < 3 || tmpi > 4) {
-      fprintf(stderr, "Unsupported SGF spec version: %d\n", tmpi);
-    }
   }
-#endif
+  else if ((tmpi < 3 || tmpi > 4) && VERBOSE_WARNINGS)
+    fprintf(stderr, "Unsupported SGF spec version: %d\n", tmpi);
 
   return root;
 }
@@ -1059,11 +1196,11 @@ sgfPrintCommentProperty(FILE *file, SGFNode *node, const char *name)
   SGFProperty *prop;
   short nam = name[0] | name[1] << 8;
 
-  for (prop = node->props; prop; prop=prop->next) {
+  for (prop = node->props; prop; prop = prop->next) {
     if (prop->name == nam) {
-      prop->name|=0x20;  /*indicate already printed*/
+      prop->name |= 0x20;  /*indicate already printed*/
       if (first) {
-	if (name[1]==' ')
+	if (name[1] == ' ')
 	  fprintf(file, "%c[%s", name[0], prop->value);
 	else
 	  fprintf(file, "%s[%s", name, prop->value);
@@ -1077,7 +1214,8 @@ sgfPrintCommentProperty(FILE *file, SGFNode *node, const char *name)
   if (!first) {
     fprintf(file, "]");
     return 1;
-  } else
+  }
+  else
     return 0;
 }
 
@@ -1197,7 +1335,7 @@ unparse_game(FILE *file, SGFNode *node, int root)
 }
 
 void
-sgf_write_header(SGFNode *root, int overwrite, int seed, float komi)
+sgf_write_header(SGFNode *root, int overwrite, int seed, float komi, int level)
 {
   time_t curtime = time(NULL);
   struct tm *loctime = localtime(&curtime);
@@ -1261,7 +1399,8 @@ main()
     fprintf(stderr, "Parse error:");
     fprintf(stderr, sgferr, sgferrarg);
     fprintf(stderr, " at position %d\n", sgferrpos);
-  } else {
+  }
+  else {
     unparse_game(stdin, game, 1);
     write(1, output, outputp - output);
   }
